@@ -1,5 +1,7 @@
 # frozen_string_literal: true
 
+# :markup: markdown
+
 module ActionCable
   module SubscriptionAdapter
     class Base
@@ -25,6 +27,10 @@ module ActionCable
       def shutdown
         raise NotImplementedError
       end
+
+      def identifier
+        @server.config.cable[:id] ||= "ActionCable-PID-#{$$}"
+      end
     end
   end
 end
