@@ -25,9 +25,12 @@ class ActionCable::Connection::AuthorizationTest < ActionCable::TestCase
         "HTTP_HOST" => "localhost", "HTTP_ORIGIN" => "http://rubyonrails.com"
 
       connection = Connection.new(server, env)
-      connection.websocket.expects(:close)
 
-      connection.process
+      assert_called_with(connection.websocket, :transmit, [{ type: "disconnect", reason: "unauthorized", reconnect: false }.to_json]) do
+        assert_called(connection.websocket, :close) do
+          connection.process
+        end
+      end
     end
   end
 end
