@@ -1,3 +1,7 @@
+# frozen_string_literal: true
+
+# :markup: markdown
+
 module ActionController
   module Railties
     module Helpers
@@ -5,7 +9,7 @@ module ActionController
         super
         return unless klass.respond_to?(:helpers_path=)
 
-        if namespace = klass.parents.detect { |m| m.respond_to?(:railtie_helpers_paths) }
+        if namespace = klass.module_parents.detect { |m| m.respond_to?(:railtie_helpers_paths) }
           paths = namespace.railtie_helpers_paths
         else
           paths = ActionController::Helpers.helpers_path
