@@ -65,19 +65,59 @@ class RedirectController < ActionController::Base
   end
 
   def redirect_back_with_status
-    redirect_back(fallback_location: "/things/stuff", status: 307)
+    redirect_back_or_to "/things/stuff", status: 307
   end
 
   def redirect_back_with_status_and_fallback_location_to_another_host
-    redirect_back(fallback_location: "http://www.rubyonrails.org/", status: 307)
+    redirect_back_or_to "http://www.rubyonrails.org/", status: 307
   end
 
   def safe_redirect_back_with_status
-    redirect_back(fallback_location: "/things/stuff", status: 307, allow_other_host: false)
+    redirect_back_or_to "/things/stuff", status: 307, allow_other_host: false
   end
 
   def safe_redirect_back_with_status_and_fallback_location_to_another_host
-    redirect_back(fallback_location: "http://www.rubyonrails.org/", status: 307, allow_other_host: false)
+    redirect_back_or_to "http://www.rubyonrails.org/", status: 307, allow_other_host: false
+  end
+
+  def safe_redirect_to_root
+    redirect_to url_from("/")
+  end
+
+  def unsafe_redirect
+    redirect_to "http://www.rubyonrails.org/"
+  end
+
+  def unsafe_redirect_back
+    redirect_back_or_to "http://www.rubyonrails.org/"
+  end
+
+  def unsafe_redirect_malformed
+    redirect_to "http:///www.rubyonrails.org/"
+  end
+
+  def unsafe_redirect_protocol_relative_double_slash
+    redirect_to "//www.rubyonrails.org/"
+  end
+
+  def unsafe_redirect_protocol_relative_triple_slash
+    redirect_to "///www.rubyonrails.org/"
+  end
+
+  def unsafe_redirect_with_illegal_http_header_value_character
+    redirect_to "javascript:alert(document.domain)\b", allow_other_host: true
+  end
+
+  def only_path_redirect
+    redirect_to action: "other_host", only_path: true
+  end
+
+  def safe_redirect_with_fallback
+    redirect_to url_from(params[:redirect_url]) || "/fallback"
+  end
+
+  def redirect_back_with_explicit_fallback_kwarg
+    redirect_back(fallback_location: "/things/stuff", status: 307)
   end
 
   def host_redirect
@@ -92,6 +132,16 @@ class RedirectController < ActionController::Base
     redirect_to "http://www.rubyonrails.org/"
   end
 
+  def redirect_to_url_with_stringlike
+    stringlike = Object.new
+
+    def stringlike.to_str
+      "http://www.rubyonrails.org/"
+    end
+
+    redirect_to stringlike
+  end
+
   def redirect_to_url_with_unescaped_query_string
     redirect_to "http://example.com/query?status=new"
   end
@@ -153,15 +203,10 @@ class RedirectController < ActionController::Base
     redirect_to "\000/lol\r\nwat"
   end
 
-  def unsafe_redirect_with_illegal_http_header_value_character
-    redirect_to "javascript:alert(document.domain)\b"
-  end
-
-
   def rescue_errors(e) raise e end
 
   private
-    def dashbord_url(id, message)
+    def dashboard_url(id, message)
       url_for action: "dashboard", params: { "id" => id, "message" => message }
     end
 end
@@ -266,6 +311,12 @@ class RedirectTest < ActionController::TestCase
     assert_redirected_to "http://www.rubyonrails.org/"
   end
 
+  def test_redirect_to_url_with_stringlike
+    get :redirect_to_url_with_stringlike
+    assert_response :redirect
+    assert_redirected_to "http://www.rubyonrails.org/"
+  end
+
   def test_redirect_to_url_with_unescaped_query_string
     get :redirect_to_url_with_unescaped_query_string
     assert_response :redirect
@@ -339,12 +390,28 @@ class RedirectTest < ActionController::TestCase
     assert_equal "http://www.rubyonrails.org/", redirect_to_url
   end
 
+  def test_safe_redirect_to_root
+    get :safe_redirect_to_root
+
+    assert_equal "http://test.host/", redirect_to_url
+  end
+
+  def test_redirect_back_with_explicit_fallback_kwarg
+    referer = "http://www.example.com/coming/from"
+    @request.env["HTTP_REFERER"] = referer
+
+    get :redirect_back_with_explicit_fallback_kwarg
+
+    assert_response 307
+    assert_equal referer, redirect_to_url
+  end
+
   def test_redirect_to_record
     with_routing do |set|
       set.draw do
         resources :workshops
 
-        ActiveSupport::Deprecation.silence do
+        ActionDispatch.deprecator.silence do
           get ":controller/:action"
         end
       end
@@ -366,7 +433,7 @@ class RedirectTest < ActionController::TestCase
           resources :workshops
         end
 
-        ActiveSupport::Deprecation.silence do
+        ActionDispatch.deprecator.silence do
           get ":controller/:action"
         end
       end
@@ -384,7 +451,7 @@ class RedirectTest < ActionController::TestCase
           resources :workshops
         end
 
-        ActiveSupport::Deprecation.silence do
+        ActionDispatch.deprecator.silence do
           get ":controller/:action"
         end
       end
@@ -431,7 +498,7 @@ class RedirectTest < ActionController::TestCase
   def test_redirect_to_with_block_and_accepted_options
     with_routing do |set|
       set.draw do
-        ActiveSupport::Deprecation.silence do
+        ActionDispatch.deprecator.silence do
           get ":controller/:action"
         end
       end
@@ -443,17 +510,121 @@ class RedirectTest < ActionController::TestCase
     end
   end
 
-  def test_unsafe_redirect_with_illegal_http_header_value_character
-    error = assert_raise(ActionController::Redirecting::UnsafeRedirectError) do
-      get :unsafe_redirect_with_illegal_http_header_value_character
+  def test_unsafe_redirect
+    with_raise_on_open_redirects do
+      error = assert_raise(ActionController::Redirecting::UnsafeRedirectError) do
+        get :unsafe_redirect
+      end
+
+      assert_equal "Unsafe redirect to \"http://www.rubyonrails.org/\", pass allow_other_host: true to redirect anyway.", error.message
     end
-
-    msg = "The redirect URL javascript:alert(document.domain)\b contains one or more illegal HTTP header field character. " \
-      "Set of legal characters defined in https://datatracker.ietf.org/doc/html/rfc7230#section-3.2.6"
-
-    assert_equal msg, error.message
   end
 
+  def test_unsafe_redirect_back
+    with_raise_on_open_redirects do
+      error = assert_raise(ActionController::Redirecting::UnsafeRedirectError) do
+        get :unsafe_redirect_back
+      end
+
+      assert_equal "Unsafe redirect to \"http://www.rubyonrails.org/\", pass allow_other_host: true to redirect anyway.", error.message
+    end
+  end
+
+  def test_unsafe_redirect_with_malformed_url
+    with_raise_on_open_redirects do
+      error = assert_raise(ActionController::Redirecting::UnsafeRedirectError) do
+        get :unsafe_redirect_malformed
+      end
+
+      assert_equal "Unsafe redirect to \"http:///www.rubyonrails.org/\", pass allow_other_host: true to redirect anyway.", error.message
+    end
+  end
+
+  def test_unsafe_redirect_with_protocol_relative_double_slash_url
+    with_raise_on_open_redirects do
+      error = assert_raise(ActionController::Redirecting::UnsafeRedirectError) do
+        get :unsafe_redirect_protocol_relative_double_slash
+      end
+
+      assert_equal "Unsafe redirect to \"//www.rubyonrails.org/\", pass allow_other_host: true to redirect anyway.", error.message
+    end
+  end
+
+  def test_unsafe_redirect_with_protocol_relative_triple_slash_url
+    with_raise_on_open_redirects do
+      error = assert_raise(ActionController::Redirecting::UnsafeRedirectError) do
+        get :unsafe_redirect_protocol_relative_triple_slash
+      end
+
+      assert_equal "Unsafe redirect to \"///www.rubyonrails.org/\", pass allow_other_host: true to redirect anyway.", error.message
+    end
+  end
+
+  def test_unsafe_redirect_with_illegal_http_header_value_character
+    with_raise_on_open_redirects do
+      error = assert_raise(ActionController::Redirecting::UnsafeRedirectError) do
+        get :unsafe_redirect_with_illegal_http_header_value_character
+      end
+
+      msg = "The redirect URL javascript:alert(document.domain)\b contains one or more illegal HTTP header field character. " \
+        "Set of legal characters defined in https://datatracker.ietf.org/doc/html/rfc7230#section-3.2.6"
+
+      assert_equal msg, error.message
+    end
+  end
+
+  def test_only_path_redirect
+    with_raise_on_open_redirects do
+      get :only_path_redirect
+      assert_response :redirect
+      assert_redirected_to "/redirect/other_host"
+    end
+  end
+
+  def test_url_from
+    with_raise_on_open_redirects do
+      get :safe_redirect_with_fallback, params: { redirect_url: "http://test.host/app" }
+      assert_response :redirect
+      assert_redirected_to "http://test.host/app"
+    end
+  end
+
+  def test_url_from_fallback
+    with_raise_on_open_redirects do
+      get :safe_redirect_with_fallback, params: { redirect_url: "http://www.rubyonrails.org/" }
+      assert_response :redirect
+      assert_redirected_to "http://test.host/fallback"
+
+      get :safe_redirect_with_fallback, params: { redirect_url: "" }
+      assert_response :redirect
+      assert_redirected_to "http://test.host/fallback"
+    end
+  end
+
+  def test_redirect_to_instrumentation
+    payload = nil
+
+    subscriber = proc do |event|
+      payload = event.payload
+    end
+
+    ActiveSupport::Notifications.subscribed(subscriber, "redirect_to.action_controller") do
+      get :simple_redirect
+    end
+
+    assert_equal request, payload[:request]
+    assert_equal 302, payload[:status]
+    assert_equal "http://test.host/redirect/hello_world", payload[:location]
+  end
+
+  private
+    def with_raise_on_open_redirects
+      old_raise_on_open_redirects = ActionController::Base.raise_on_open_redirects
+      ActionController::Base.raise_on_open_redirects = true
+      yield
+    ensure
+      ActionController::Base.raise_on_open_redirects = old_raise_on_open_redirects
+    end
 end
 
 module ModuleTest
