@@ -103,6 +103,24 @@ class RequestForgeryProtectionControllerUsingNullSession < ActionController::Bas
   end
 end
 
+class RequestForgeryProtectionControllerUsingCustomStrategy < ActionController::Base
+  include RequestForgeryProtectionActions
+
+  class FakeException < Exception; end
+
+  class CustomStrategy
+    def initialize(controller)
+      @controller = controller
+    end
+
+    def handle_unverified_request
+      raise FakeException, "Raised a fake exception."
+    end
+  end
+
+  protect_from_forgery only: %w(index meta same_origin_js negotiate_same_origin), with: CustomStrategy
+end
+
 class PrependProtectForgeryBaseController < ActionController::Base
   before_action :custom_action
   attr_accessor :called_callbacks
@@ -172,18 +190,64 @@ class SkipProtectionController < ActionController::Base
   attr_accessor :skip_requested
 end
 
+class SkipProtectionWhenUnprotectedController < ActionController::Base
+  include RequestForgeryProtectionActions
+  skip_forgery_protection
+end
+
+class CookieCsrfTokenStorageStrategyController < ActionController::Base
+  include RequestForgeryProtectionActions
+
+  after_action :commit_token, only: :cookie
+
+  protect_from_forgery only: %w(index meta same_origin_js negotiate_same_origin), with: :exception, store: :cookie
+
+  def reset
+    reset_csrf_token(request)
+    head :ok
+  end
+
+  def cookie
+    render inline: "<%= csrf_meta_tags %>"
+  end
+
+  private
+    def commit_token
+      request.commit_csrf_token
+    end
+end
+
+class CustomCsrfTokenStorageStrategyController < ActionController::Base
+  include RequestForgeryProtectionActions
+
+  class CustomStrategy
+    def fetch(request)
+      request.env[:custom_storage]
+    end
+
+    def store(request, csrf_token)
+      request.env[:custom_storage] = csrf_token
+    end
+
+    def reset(request)
+      request.env[:custom_storage] = nil
+    end
+  end
+
+  protect_from_forgery only: %w(index meta same_origin_js negotiate_same_origin),
+    with: :reset_session,
+    store: CustomStrategy.new
+end
+
 # common test methods
 module RequestForgeryProtectionTests
   def setup
-    @old_urlsafe_csrf_tokens = ActionController::Base.urlsafe_csrf_tokens
-    ActionController::Base.urlsafe_csrf_tokens = true
     @token = Base64.urlsafe_encode64("railstestrailstestrailstestrails")
     @old_request_forgery_protection_token = ActionController::Base.request_forgery_protection_token
     ActionController::Base.request_forgery_protection_token = :custom_authenticity_token
   end
 
   def teardown
-    ActionController::Base.urlsafe_csrf_tokens = @old_urlsafe_csrf_tokens
     ActionController::Base.request_forgery_protection_token = @old_request_forgery_protection_token
   end
 
@@ -374,7 +438,7 @@ module RequestForgeryProtectionTests
   end
 
   def test_should_allow_post_with_token
-    session[:_csrf_token] = @token
+    initialize_csrf_token
     @controller.stub :form_authenticity_token, @token do
       assert_not_blocked { post :index, params: { custom_authenticity_token: @token } }
     end
@@ -383,73 +447,60 @@ module RequestForgeryProtectionTests
   def test_should_allow_post_with_strict_encoded_token
     token_length = (ActionController::RequestForgeryProtection::AUTHENTICITY_TOKEN_LENGTH * 4.0 / 3).ceil
     token_including_url_unsafe_chars = "+/".ljust(token_length, "A")
-    session[:_csrf_token] = token_including_url_unsafe_chars
+    initialize_csrf_token(token_including_url_unsafe_chars)
     @controller.stub :form_authenticity_token, token_including_url_unsafe_chars do
       assert_not_blocked { post :index, params: { custom_authenticity_token: token_including_url_unsafe_chars } }
     end
   end
 
-  def test_should_allow_post_with_urlsafe_token_when_migrating
-    config_before = ActionController::Base.urlsafe_csrf_tokens
-    ActionController::Base.urlsafe_csrf_tokens = false
-    token_length = (ActionController::RequestForgeryProtection::AUTHENTICITY_TOKEN_LENGTH * 4.0 / 3).ceil
-    token_including_url_safe_chars = "-_".ljust(token_length, "A")
-    session[:_csrf_token] = token_including_url_safe_chars
-    @controller.stub :form_authenticity_token, token_including_url_safe_chars do
-      assert_not_blocked { post :index, params: { custom_authenticity_token: token_including_url_safe_chars } }
-    end
-  ensure
-    ActionController::Base.urlsafe_csrf_tokens = config_before
-  end
-
   def test_should_allow_patch_with_token
-    session[:_csrf_token] = @token
+    initialize_csrf_token
     @controller.stub :form_authenticity_token, @token do
       assert_not_blocked { patch :index, params: { custom_authenticity_token: @token } }
     end
   end
 
   def test_should_allow_put_with_token
-    session[:_csrf_token] = @token
+    initialize_csrf_token
     @controller.stub :form_authenticity_token, @token do
       assert_not_blocked { put :index, params: { custom_authenticity_token: @token } }
     end
   end
 
   def test_should_allow_delete_with_token
-    session[:_csrf_token] = @token
+    initialize_csrf_token
     @controller.stub :form_authenticity_token, @token do
       assert_not_blocked { delete :index, params: { custom_authenticity_token: @token } }
     end
   end
 
   def test_should_allow_post_with_token_in_header
-    session[:_csrf_token] = @token
+    initialize_csrf_token
     @request.env["HTTP_X_CSRF_TOKEN"] = @token
     assert_not_blocked { post :index }
   end
 
   def test_should_allow_delete_with_token_in_header
-    session[:_csrf_token] = @token
+    initialize_csrf_token
     @request.env["HTTP_X_CSRF_TOKEN"] = @token
     assert_not_blocked { delete :index }
   end
 
   def test_should_allow_patch_with_token_in_header
-    session[:_csrf_token] = @token
+    initialize_csrf_token
     @request.env["HTTP_X_CSRF_TOKEN"] = @token
     assert_not_blocked { patch :index }
   end
 
   def test_should_allow_put_with_token_in_header
-    session[:_csrf_token] = @token
+    initialize_csrf_token
     @request.env["HTTP_X_CSRF_TOKEN"] = @token
     assert_not_blocked { put :index }
   end
 
   def test_should_allow_post_with_origin_checking_and_correct_origin
     forgery_protection_origin_check do
-      session[:_csrf_token] = @token
+      initialize_csrf_token
       @controller.stub :form_authenticity_token, @token do
         assert_not_blocked do
           @request.set_header "HTTP_ORIGIN", "http://test.host"
@@ -461,7 +512,7 @@ module RequestForgeryProtectionTests
 
   def test_should_allow_post_with_origin_checking_and_no_origin
     forgery_protection_origin_check do
-      session[:_csrf_token] = @token
+      initialize_csrf_token
       @controller.stub :form_authenticity_token, @token do
         assert_not_blocked do
           post :index, params: { custom_authenticity_token: @token }
@@ -472,7 +523,7 @@ module RequestForgeryProtectionTests
 
   def test_should_raise_for_post_with_null_origin
     forgery_protection_origin_check do
-      session[:_csrf_token] = @token
+      initialize_csrf_token
       @controller.stub :form_authenticity_token, @token do
         exception = assert_raises(ActionController::InvalidAuthenticityToken) do
           @request.set_header "HTTP_ORIGIN", "null"
@@ -489,7 +540,7 @@ module RequestForgeryProtectionTests
     ActionController::Base.logger = logger
 
     forgery_protection_origin_check do
-      session[:_csrf_token] = @token
+      initialize_csrf_token
       @controller.stub :form_authenticity_token, @token do
         assert_blocked do
           @request.set_header "HTTP_ORIGIN", "http://bad.host"
@@ -506,6 +557,7 @@ module RequestForgeryProtectionTests
     ActionController::Base.logger = old_logger
   end
 
+
   def test_should_warn_on_missing_csrf_token
     old_logger = ActionController::Base.logger
     logger = ActiveSupport::LogSubscriber::TestHelper::MockLogger.new
@@ -591,7 +643,7 @@ module RequestForgeryProtectionTests
 
   # Allow non-GET requests since GET is all a remote <script> tag can muster.
   def test_should_allow_non_get_js_without_xhr_header
-    session[:_csrf_token] = @token
+    initialize_csrf_token
     assert_cross_origin_not_blocked { post :same_origin_js, params: { custom_authenticity_token: @token } }
     assert_cross_origin_not_blocked { post :same_origin_js, params: { format: "js", custom_authenticity_token: @token } }
     assert_cross_origin_not_blocked do
@@ -616,12 +668,25 @@ module RequestForgeryProtectionTests
     end
   end
 
+  def test_csrf_token_is_not_saved_if_it_is_nil
+    @controller.commit_csrf_token(@request)
+    assert_nil fetch_csrf_token
+  end
+
   def test_should_not_raise_error_if_token_is_not_a_string
     assert_blocked do
       patch :index, params: { custom_authenticity_token: { foo: "bar" } }
     end
   end
 
+  def initialize_csrf_token(token = @token, session = self.session)
+    session[:_csrf_token] = token
+  end
+
+  def fetch_csrf_token
+    session[:_csrf_token]
+  end
+
   def assert_blocked
     session[:something_like_user_id] = 1
     yield
@@ -630,18 +695,18 @@ module RequestForgeryProtectionTests
   end
 
   def assert_not_blocked(&block)
+    session[:something_like_user_id] = 1
     assert_nothing_raised(&block)
+    assert_equal 1, session[:something_like_user_id]
     assert_response :success
   end
 
-  def assert_cross_origin_blocked
-    assert_raises(ActionController::InvalidCrossOriginRequest) do
-      yield
-    end
+  def assert_cross_origin_blocked(&block)
+    assert_raises(ActionController::InvalidCrossOriginRequest, &block)
   end
 
-  def assert_cross_origin_not_blocked
-    assert_not_blocked { yield }
+  def assert_cross_origin_not_blocked(&block)
+    assert_not_blocked(&block)
   end
 
   def forgery_protection_origin_check
@@ -701,13 +766,36 @@ end
 
 class RequestForgeryProtectionControllerUsingExceptionTest < ActionController::TestCase
   include RequestForgeryProtectionTests
-  def assert_blocked
-    assert_raises(ActionController::InvalidAuthenticityToken) do
-      yield
+
+  def assert_blocked(&block)
+    assert_raises(ActionController::InvalidAuthenticityToken, &block)
+  end
+
+  def test_raised_exception_message_explains_why_it_occurred
+    forgery_protection_origin_check do
+      initialize_csrf_token
+      @controller.stub :form_authenticity_token, @token do
+        exception = assert_raises(ActionController::InvalidAuthenticityToken) do
+          @request.set_header "HTTP_ORIGIN", "http://bad.host"
+          post :index, params: { custom_authenticity_token: @token }
+        end
+        assert_match(
+          "HTTP Origin header (http://bad.host) didn't match request.base_url (http://test.host)",
+          exception.message
+        )
+      end
     end
   end
 end
 
+class RequestForgeryProtectionControllerUsingCustomStrategyTest < ActionController::TestCase
+  include RequestForgeryProtectionTests
+
+  def assert_blocked(&block)
+    assert_raises(RequestForgeryProtectionControllerUsingCustomStrategy::FakeException, &block)
+  end
+end
+
 class PrependProtectForgeryBaseControllerTest < ActionController::TestCase
   PrependTrueController = Class.new(PrependProtectForgeryBaseController) do
     protect_from_forgery prepend: true
@@ -832,7 +920,7 @@ class PerFormTokensControllerTest < ActionController::TestCase
   def test_per_form_token_is_same_size_as_global_token
     get :index
     expected = ActionController::RequestForgeryProtection::AUTHENTICITY_TOKEN_LENGTH
-    actual = @controller.send(:per_form_csrf_token, session, "/path", "post").size
+    actual = @controller.send(:per_form_csrf_token, nil, "/path", "post").size
     assert_equal expected, actual
   end
 
@@ -872,9 +960,10 @@ class PerFormTokensControllerTest < ActionController::TestCase
 
     # Set invalid URI in PATH_INFO
     @request.env["PATH_INFO"] = "/foo/bar<"
-    assert_raise ActionController::InvalidAuthenticityToken do
+    exception = assert_raises(ActionController::InvalidAuthenticityToken) do
       post :post_one, params: { custom_authenticity_token: form_token }
     end
+    assert_match "Can't verify CSRF token authenticity.", exception.message
   end
 
   def test_rejects_token_for_incorrect_path
@@ -969,7 +1058,7 @@ class PerFormTokensControllerTest < ActionController::TestCase
 
     unmasked_token = @controller.send(:unmask_token, Base64.urlsafe_decode64(token))
 
-    assert_not_equal @controller.send(:real_csrf_token, session), unmasked_token
+    assert_not_equal @controller.send(:real_csrf_token), unmasked_token
   end
 
   def test_returns_hmacd_token
@@ -979,13 +1068,13 @@ class PerFormTokensControllerTest < ActionController::TestCase
 
     unmasked_token = @controller.send(:unmask_token, Base64.urlsafe_decode64(token))
 
-    assert_equal @controller.send(:global_csrf_token, session), unmasked_token
+    assert_equal @controller.send(:global_csrf_token), unmasked_token
   end
 
   def test_accepts_old_csrf_token
     get :index
 
-    non_hmac_token = @controller.send(:mask_token, @controller.send(:real_csrf_token, session))
+    non_hmac_token = @controller.send(:mask_token, @controller.send(:real_csrf_token))
 
     # This is required because PATH_INFO isn't reset between requests.
     @request.env["PATH_INFO"] = "/per_form_tokens/post_one"
@@ -1023,6 +1112,71 @@ class PerFormTokensControllerTest < ActionController::TestCase
     assert_response :success
   end
 
+  def test_handles_empty_path_as_request_path
+    get :index, params: { form_path: "" }
+
+    form_token = assert_presence_and_fetch_form_csrf_token
+
+    # This is required because PATH_INFO isn't reset between requests.
+    @request.env["PATH_INFO"] = "/per_form_tokens"
+    assert_nothing_raised do
+      post :post_one, params: { custom_authenticity_token: form_token }
+    end
+    assert_response :success
+  end
+
+  def test_handles_relative_paths
+    get :index, params: { form_path: "post_one" }
+
+    form_token = assert_presence_and_fetch_form_csrf_token
+
+    # This is required because PATH_INFO isn't reset between requests.
+    @request.env["PATH_INFO"] = "/per_form_tokens/post_one"
+    assert_nothing_raised do
+      post :post_one, params: { custom_authenticity_token: form_token }
+    end
+    assert_response :success
+  end
+
+  def test_handles_relative_paths_with_dot
+    get :index, params: { form_path: "./post_one" }
+
+    form_token = assert_presence_and_fetch_form_csrf_token
+
+    # This is required because PATH_INFO isn't reset between requests.
+    @request.env["PATH_INFO"] = "/per_form_tokens/post_one"
+    assert_nothing_raised do
+      post :post_one, params: { custom_authenticity_token: form_token }
+    end
+    assert_response :success
+  end
+
+  def test_handles_query_string
+    get :index, params: { form_path: "./post_one?a=b" }
+
+    form_token = assert_presence_and_fetch_form_csrf_token
+
+    # This is required because PATH_INFO isn't reset between requests.
+    @request.env["PATH_INFO"] = "/per_form_tokens/post_one"
+    assert_nothing_raised do
+      post :post_one, params: { custom_authenticity_token: form_token }
+    end
+    assert_response :success
+  end
+
+  def test_handles_fragment
+    get :index, params: { form_path: "./post_one#a" }
+
+    form_token = assert_presence_and_fetch_form_csrf_token
+
+    # This is required because PATH_INFO isn't reset between requests.
+    @request.env["PATH_INFO"] = "/per_form_tokens/post_one"
+    assert_nothing_raised do
+      post :post_one, params: { custom_authenticity_token: form_token }
+    end
+    assert_response :success
+  end
+
   def test_ignores_origin_during_generation
     get :index, params: { form_path: "https://example.com/per_form_tokens/post_one/" }
 
@@ -1072,7 +1226,7 @@ class PerFormTokensControllerTest < ActionController::TestCase
 
     def assert_matches_session_token_on_server(form_token, method = "post")
       actual = @controller.send(:unmask_token, Base64.urlsafe_decode64(form_token))
-      expected = @controller.send(:per_form_csrf_token, session, "/per_form_tokens/post_one", method)
+      expected = @controller.send(:per_form_csrf_token, nil, "/per_form_tokens/post_one", method)
       assert_equal expected, actual
     end
 end
@@ -1088,10 +1242,8 @@ class SkipProtectionControllerTest < ActionController::TestCase
     assert_not_blocked { post :index }
   end
 
-  def assert_blocked
-    assert_raises(ActionController::InvalidAuthenticityToken) do
-      yield
-    end
+  def assert_blocked(&block)
+    assert_raises(ActionController::InvalidAuthenticityToken, &block)
   end
 
   def assert_not_blocked(&block)
@@ -1099,3 +1251,185 @@ class SkipProtectionControllerTest < ActionController::TestCase
     assert_response :success
   end
 end
+
+class SkipProtectionWhenUnprotectedControllerTest < ActionController::TestCase
+  def test_should_allow_skip_request_when_protection_is_not_set
+    assert_not_blocked { post :index }
+  end
+
+  def assert_not_blocked(&block)
+    assert_nothing_raised(&block)
+    assert_response :success
+  end
+end
+
+class CookieCsrfTokenStorageStrategyControllerTest < ActionController::TestCase
+  include RequestForgeryProtectionTests
+
+  class TestSession < ActionController::TestSession
+    attr_reader :id_was
+
+    def initialize(id_was)
+      super()
+      @id_was = id_was
+    end
+  end
+
+  class NullSessionDummyKeyGenerator
+    def generate_key(secret, length = nil)
+      "03312270731a2ed0d11ed091c2338a06"
+    end
+  end
+
+  def setup
+    @request.env[ActionDispatch::Cookies::GENERATOR_KEY] = NullSessionDummyKeyGenerator.new
+    @request.env[ActionDispatch::Cookies::COOKIES_ROTATIONS] = ActiveSupport::Messages::RotationConfiguration.new
+    super
+  end
+
+  def test_csrf_token_is_stored_in_cookie
+    get :cookie
+    assert_not session.key?(:_csrf_token)
+    assert cookies.key?(:csrf_token)
+  end
+
+  def test_csrf_token_is_stored_in_custom_cookie
+    @controller.csrf_token_storage_strategy =
+      ActionController::RequestForgeryProtection::CookieStore.new(:custom_cookie)
+    get :cookie
+    assert_not cookies.key?(:csrf_token)
+    assert cookies.key?(:custom_cookie)
+  end
+
+  def test_csrf_token_cookie_has_same_site_lax
+    get :cookie
+    assert_set_cookie_attributes("csrf_token", "SameSite=Lax")
+  end
+
+  include CookieAssertions
+
+  def test_csrf_token_cookie_is_http_only
+    get :cookie
+
+    cookies = parse_set_cookies_headers(@response.headers["Set-Cookie"])
+    csrf_token_cookie = cookies["csrf_token"]
+    assert csrf_token_cookie["httponly"]
+  end
+
+  def test_csrf_token_cookie_is_permanent
+    get :cookie
+    assert_match(%r(#{20.years.from_now.utc.year}), @response.headers["Set-Cookie"])
+  end
+
+  def test_reset_csrf_token_deletes_cookie
+    get :cookie
+    get :reset
+    assert_nil cookies[:csrf_token]
+  end
+
+  def test_should_allow_when_session_id_in_cookie_matches_session_id
+    initialize_csrf_token
+
+    @controller.stub :form_authenticity_token, @token do
+      assert_not_blocked { post :index, params: { custom_authenticity_token: @token } }
+    end
+  end
+
+  def test_should_not_allow_when_session_id_in_cookie_does_not_match_session_id
+    initialize_csrf_token(@token, ActionController::TestSession.new)
+
+    @controller.stub :form_authenticity_token, @token do
+      assert_blocked { post :index, params: { custom_authenticity_token: @token } }
+    end
+  end
+
+  def test_should_allow_when_session_id_in_cookie_and_session_id_are_nil
+    @request.session = ActionController::TestSession.new({}, nil)
+    initialize_csrf_token(@token, nil)
+
+    @controller.stub :form_authenticity_token, @token do
+      assert_not_blocked { post :index, params: { custom_authenticity_token: @token } }
+    end
+  end
+
+  def test_should_not_allow_when_session_id_in_cookie_but_session_id_is_nil
+    initialize_csrf_token
+    @request.session = ActionController::TestSession.new({}, nil)
+
+    @controller.stub :form_authenticity_token, @token do
+      assert_blocked { post :index, params: { custom_authenticity_token: @token } }
+    end
+  end
+
+  def test_should_allow_when_session_id_in_cookie_is_nil_and_session_created_before_token_validation
+    initialize_csrf_token(@token, nil)
+    @request.session = TestSession.new(nil)
+
+    @controller.stub :form_authenticity_token, @token do
+      assert_not_blocked { post :index, params: { custom_authenticity_token: @token } }
+    end
+  end
+
+  def test_should_allow_when_session_id_in_cookie_is_nil_and_session_reset_before_token_validation
+    initialize_csrf_token
+    @request.session = TestSession.new(session.id)
+
+    @controller.stub :form_authenticity_token, @token do
+      assert_not_blocked { post :index, params: { custom_authenticity_token: @token } }
+    end
+  end
+
+  def test_should_not_allow_when_session_id_in_cookie_but_request_made_with_no_session
+    initialize_csrf_token
+    @request.session = TestSession.new(nil)
+
+    @controller.stub :form_authenticity_token, @token do
+      assert_blocked { post :index, params: { custom_authenticity_token: @token } }
+    end
+  end
+
+  def initialize_csrf_token(token = @token, session = self.session)
+    cookies.encrypted[:csrf_token] = {
+      value: {
+        token: token,
+        session_id: session&.id,
+      }.to_json,
+      httponly: true,
+      same_site: :lax,
+    }
+  end
+
+  def fetch_csrf_token
+    contents = request.cookie_jar.encrypted[:csrf_token]
+    return nil if contents.nil?
+
+    value = JSON.parse(contents)
+    return nil unless value["session_id"]&.fetch("public_id") == request.session.id_was&.public_id
+
+    value["token"]
+  end
+
+  def assert_blocked(&block)
+    assert_raises(ActionController::InvalidAuthenticityToken, &block)
+  end
+
+  def assert_not_blocked(&block)
+    assert_nothing_raised(&block)
+    assert_response :success
+  end
+end
+
+class CustomCsrfTokenStorageStrategyControllerTest < ActionController::TestCase
+  include RequestForgeryProtectionTests
+
+  def test_csrf_token_is_stored_in_custom_location
+    post :index
+    @controller.commit_csrf_token(@request)
+    assert_not session.key?(:_csrf_token)
+    assert_not_nil request.env[:custom_storage]
+  end
+
+  def initialize_csrf_token(token = @token)
+    request.env[:custom_storage] = token
+  end
+end
