@@ -17,7 +17,7 @@ class UriReservedCharactersRoutingTest < ActiveSupport::TestCase
   def setup
     @set = ActionDispatch::Routing::RouteSet.new
     @set.draw do
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get ":controller/:action/:variable/*additional"
       end
     end
@@ -216,7 +216,7 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
   def test_regexp_precedence
     rs.draw do
       get "/whois/:domain", constraints: {
-        domain: /\w+\.[\w\.]+/ },
+        domain: /\w+\.[\w.]+/ },
         to: lambda { |env| [200, {}, %w{regexp}] }
 
       get "/whois/:id", to: lambda { |env| [200, {}, %w{id}] }
@@ -244,6 +244,18 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
     assert_equal "clients", get(URI("http://clients.example.org/"))
   end
 
+  def test_format_symbol_constraints
+    rs.draw do
+      get "/api", constraints: { format: :json },
+                 to: lambda { |env| [200, {}, %w{json}] }
+      get "/api", constraints: { format: :xml },
+                 to: lambda { |env| [200, {}, %w{xml}] }
+    end
+
+    assert_equal "json", get(URI("http://www.example.org/api.json"))
+    assert_equal "xml", get(URI("http://clients.example.org/api.xml"))
+  end
+
   def test_lambda_constraints
     rs.draw do
       get "/", constraints: lambda { |req|
@@ -318,7 +330,7 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
   end
 
   def test_default_setup
-    rs.draw { ActiveSupport::Deprecation.silence { get "/:controller(/:action(/:id))" } }
+    rs.draw { ActionDispatch.deprecator.silence { get "/:controller(/:action(/:id))" } }
     assert_equal({ controller: "content", action: "index" }, rs.recognize_path("/content"))
     assert_equal({ controller: "content", action: "list" },  rs.recognize_path("/content/list"))
     assert_equal({ controller: "content", action: "show", id: "10" }, rs.recognize_path("/content/show/10"))
@@ -339,9 +351,25 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
     assert_equal "/stuff", controller.url_for(controller: "/stuff", only_path: true)
   end
 
+  def test_route_uri_pattern
+    rs.draw { ActionDispatch.deprecator.silence { get "/:controller(/:action(/:id))" } }
+
+    get URI("http://test.host/admin/user/list/10")
+
+    assert_equal(
+      "/:controller(/:action(/:id))(.:format)",
+      controller.request.route_uri_pattern
+    )
+
+    assert_equal(
+      "/:controller(/:action(/:id))(.:format)",
+      controller.request.get_header("action_dispatch.route_uri_pattern")
+    )
+  end
+
   def test_route_with_colon_first
     rs.draw do
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get "/:controller/:action/:id", action: "index", id: nil
       end
 
@@ -352,7 +380,7 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
   end
 
   def test_route_with_regexp_for_action
-    rs.draw { ActiveSupport::Deprecation.silence { get "/:controller/:action", action: /auth[-|_].+/ } }
+    rs.draw { ActionDispatch.deprecator.silence { get "/:controller/:action", action: /auth[-|_].+/ } }
 
     assert_equal({ action: "auth_google", controller: "content" }, rs.recognize_path("/content/auth_google"))
     assert_equal({ action: "auth-twitter", controller: "content" }, rs.recognize_path("/content/auth-twitter"))
@@ -363,7 +391,7 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
 
   def test_route_with_regexp_for_controller
     rs.draw do
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get ":controller/:admintoken(/:action(/:id))", controller: /admin\/.+/
         get "/:controller(/:action(/:id))"
       end
@@ -380,7 +408,7 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
 
   def test_route_with_regexp_and_captures_for_controller
     rs.draw do
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get "/:controller(/:action(/:id))", controller: /admin\/(accounts|users)/
       end
     end
@@ -391,7 +419,7 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
 
   def test_route_with_regexp_and_dot
     rs.draw do
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get ":controller/:action/:file",
                   controller: /admin|user/,
                   action: /upload|download/,
@@ -480,9 +508,11 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
   end
 
   def test_named_route_without_hash
-    rs.draw do
-      ActiveSupport::Deprecation.silence do
-        get ":controller/:action/:id", as: "normal"
+    assert_nothing_raised do
+      rs.draw do
+        ActionDispatch.deprecator.silence do
+          get ":controller/:action/:id", as: "normal"
+        end
       end
     end
   end
@@ -536,7 +566,7 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
       get "page/:year/:month/:day/:title" => "page#show", :as => "article",
         :year => /\d+/, :month => /\d+/, :day => /\d+/
 
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get ":controller/:action/:id"
       end
     end
@@ -548,7 +578,7 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
   end
 
   def test_changing_controller
-    rs.draw { ActiveSupport::Deprecation.silence { get ":controller/:action/:id" } }
+    rs.draw { ActionDispatch.deprecator.silence { get ":controller/:action/:id" } }
 
     get URI("http://test.host/admin/user/index/10")
 
@@ -560,7 +590,7 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
     rs.draw do
       get "file/*path" => "content#show_file", :as => "path"
 
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get ":controller/:action/:id"
       end
     end
@@ -587,7 +617,7 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
 
   def test_non_controllers_cannot_be_matched
     rs.draw do
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get ":controller/:action/:id"
       end
     end
@@ -612,6 +642,34 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
         url_for(rs, controller: "content", action: "show_file", path: %w(pages boo))
   end
 
+  def test_escapes_newline_character_for_dynamic_path
+    rs.draw do
+      get "/dynamic/:dynamic_segment" => "subpath_books#show", as: :dynamic
+
+      ActionDispatch.deprecator.silence do
+        get ":controller/:action/:id"
+      end
+    end
+
+    results = rs.recognize_path("/dynamic/a%0Anewline")
+    assert(results, "Recognition should have succeeded")
+    assert_equal("a\nnewline", results[:dynamic_segment])
+  end
+
+  def test_escapes_newline_character_for_wildcard_path
+    rs.draw do
+      get "/wildcard/*wildcard_segment" => "subpath_books#show", as: :wildcard
+
+      ActionDispatch.deprecator.silence do
+        get ":controller/:action/:id"
+      end
+    end
+
+    results = rs.recognize_path("/wildcard/a%0Anewline")
+    assert(results, "Recognition should have succeeded")
+    assert_equal("a\nnewline", results[:wildcard_segment])
+  end
+
   def test_dynamic_recall_paths_allowed
     rs.draw do
       get "*path" => "content#show_file"
@@ -627,7 +685,7 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
 
   def test_backwards
     rs.draw do
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get "page/:id(/:action)" => "pages#show"
         get ":controller(/:action(/:id))"
       end
@@ -643,7 +701,7 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
     rs.draw do
       get "page(/:id)" => "content#show_page", :id => 1
 
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get ":controller/:action/:id"
       end
     end
@@ -663,7 +721,7 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
     rs.draw do
       get "page/:id" => "content#show_page", :id => 1
 
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get ":controller/:action/:id"
       end
     end
@@ -680,7 +738,7 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
   end
 
   def test_action_expiry
-    rs.draw { ActiveSupport::Deprecation.silence { get ":controller(/:action(/:id))" } }
+    rs.draw { ActionDispatch.deprecator.silence { get ":controller(/:action(/:id))" } }
     get URI("http://test.host/content/show")
     assert_equal "/content", controller.url_for(controller: "content", only_path: true)
   end
@@ -704,7 +762,7 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
         :constraints => { year: /\d{4}/ }
       )
 
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get ":controller/:action/:id"
       end
     end
@@ -719,7 +777,7 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
     rs.draw do
       get "pages(/:year(/:month(/:day)))" => "content#list_pages", :month => nil, :day => nil
 
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get ":controller/:action/:id"
       end
     end
@@ -769,7 +827,7 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
     rs.draw do
       get "categories" => "content#categories", :as => "categories"
 
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get ":controller(/:action(/:id))"
       end
     end
@@ -788,7 +846,7 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
       get "journal" => "content#list_journal",
         :date => nil, :user_id => nil
 
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get ":controller/:action/:id"
       end
     end
@@ -829,7 +887,7 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
 
   def test_subpath_recognized
     rs.draw do
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get "/books/:id/edit"    => "subpath_books#edit"
         get "/items/:id/:action" => "subpath_books"
         get "/posts/new/:action" => "subpath_books"
@@ -856,7 +914,7 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
 
   def test_subpath_generated
     rs.draw do
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get "/books/:id/edit"    => "subpath_books#edit"
         get "/items/:id/:action" => "subpath_books"
         get "/posts/new/:action" => "subpath_books"
@@ -885,7 +943,7 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
       get "cb" => "cb#ab"
       get "cc" => "cc#ac"
 
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get ":controller/:action/:id"
         get ":controller/:action/:id.:format"
       end
@@ -900,7 +958,7 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
       get "cb" => "cb#ab"
       get "cc" => "cc#ac"
 
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get ":controller/:action/:id"
         get ":controller/:action/:id.:format"
       end
@@ -911,6 +969,39 @@ class LegacyRouteSetTests < ActiveSupport::TestCase
     assert_not_nil hash
     assert_equal %w(cc ac), [hash[:controller], hash[:action]]
   end
+
+  def test_drawing_more_routes_after_eager_loading
+    rs = ::ActionDispatch::Routing::RouteSet.new
+    rs.disable_clear_and_finalize = true
+
+    rs.draw do
+      get "/plain" => "c#plain"
+      get "/:symbol" => "c#symbol"
+      get "/glob/*" => "c#glob"
+      get "/with#anchor" => "c#with_anchor"
+    end
+
+    hash = rs.recognize_path "/symbol"
+    assert_not_nil hash
+    assert_equal %w(c symbol), [hash[:controller], hash[:action]]
+
+    rs.eager_load!
+
+    rs.draw do
+      get "/more/plain" => "c#plain"
+      get "/more/:symbol" => "c#symbol"
+      get "/more/glob/*" => "c#glob"
+      get "/more/with#anchor" => "c#with_anchor"
+    end
+
+    hash = rs.recognize_path "/symbol"
+    assert_not_nil hash
+    assert_equal %w(c symbol), [hash[:controller], hash[:action]]
+
+    hash = rs.recognize_path "/more/symbol"
+    assert_not_nil hash
+    assert_equal %w(c symbol), [hash[:controller], hash[:action]]
+  end
 end
 
 class RouteSetTest < ActiveSupport::TestCase
@@ -934,7 +1025,7 @@ class RouteSetTest < ActiveSupport::TestCase
     @default_route_set ||= begin
       set = ActionDispatch::Routing::RouteSet.new
       set.draw do
-        ActiveSupport::Deprecation.silence do
+        ActionDispatch.deprecator.silence do
           get "/:controller(/:action(/:id))"
         end
       end
@@ -943,21 +1034,21 @@ class RouteSetTest < ActiveSupport::TestCase
   end
 
   def test_generate_extras
-    set.draw { ActiveSupport::Deprecation.silence { get ":controller/(:action(/:id))" } }
+    set.draw { ActionDispatch.deprecator.silence { get ":controller/(:action(/:id))" } }
     path, extras = set.generate_extras(controller: "foo", action: "bar", id: 15, this: "hello", that: "world")
     assert_equal "/foo/bar/15", path
     assert_equal %w(that this), extras.map(&:to_s).sort
   end
 
   def test_extra_keys
-    set.draw { ActiveSupport::Deprecation.silence { get ":controller/:action/:id" } }
+    set.draw { ActionDispatch.deprecator.silence { get ":controller/:action/:id" } }
     extras = set.extra_keys(controller: "foo", action: "bar", id: 15, this: "hello", that: "world")
     assert_equal %w(that this), extras.map(&:to_s).sort
   end
 
   def test_generate_extras_not_first
     set.draw do
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get ":controller/:action/:id.:format"
         get ":controller/:action/:id"
       end
@@ -969,7 +1060,7 @@ class RouteSetTest < ActiveSupport::TestCase
 
   def test_generate_not_first
     set.draw do
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get ":controller/:action/:id.:format"
         get ":controller/:action/:id"
       end
@@ -980,7 +1071,7 @@ class RouteSetTest < ActiveSupport::TestCase
 
   def test_extra_keys_not_first
     set.draw do
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get ":controller/:action/:id.:format"
         get ":controller/:action/:id"
       end
@@ -1115,7 +1206,7 @@ class RouteSetTest < ActiveSupport::TestCase
 
   def test_draw_default_route
     set.draw do
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get ":controller/:action/:id"
       end
     end
@@ -1133,7 +1224,7 @@ class RouteSetTest < ActiveSupport::TestCase
     set.draw do
       get "page/:id" => "pages#show", :id => /\d+/
 
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get "/:controller(/:action(/:id))"
       end
     end
@@ -1200,7 +1291,7 @@ class RouteSetTest < ActiveSupport::TestCase
 
   def test_recognize_with_encoded_id_and_regex
     set.draw do
-      get "page/:id" => "pages#show", :id => /[a-zA-Z0-9\+]+/
+      get "page/:id" => "pages#show", :id => /[a-zA-Z0-9+]+/
     end
 
     assert_equal({ controller: "pages", action: "show", id: "10" }, request_path_params("/page/10"))
@@ -1388,7 +1479,7 @@ class RouteSetTest < ActiveSupport::TestCase
     @set = make_set false
 
     set.draw do
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get ":controller/:id/:action"
       end
     end
@@ -1404,7 +1495,7 @@ class RouteSetTest < ActiveSupport::TestCase
     set.draw do
       get "about" => "welcome#about"
 
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get ":controller/:id/:action"
       end
     end
@@ -1417,7 +1508,7 @@ class RouteSetTest < ActiveSupport::TestCase
   end
 
   def test_generate
-    set.draw { ActiveSupport::Deprecation.silence { get ":controller/:action/:id" } }
+    set.draw { ActionDispatch.deprecator.silence { get ":controller/:action/:id" } }
 
     args = { controller: "foo", action: "bar", id: "7", x: "y" }
     assert_equal "/foo/bar/7?x=y",     url_for(set, args)
@@ -1428,7 +1519,7 @@ class RouteSetTest < ActiveSupport::TestCase
   def test_generate_with_path_prefix
     set.draw do
       scope "my" do
-        ActiveSupport::Deprecation.silence do
+        ActionDispatch.deprecator.silence do
           get ":controller(/:action(/:id))"
         end
       end
@@ -1441,7 +1532,7 @@ class RouteSetTest < ActiveSupport::TestCase
   def test_generate_with_blank_path_prefix
     set.draw do
       scope "" do
-        ActiveSupport::Deprecation.silence do
+        ActionDispatch.deprecator.silence do
           get ":controller(/:action(/:id))"
         end
       end
@@ -1455,7 +1546,7 @@ class RouteSetTest < ActiveSupport::TestCase
     @set = make_set false
 
     set.draw do
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get "/connection/manage(/:action)" => "connection/manage#index"
         get "/connection/connection" => "connection/connection#index"
         get "/connection" => "connection#index", :as => "family_connection"
@@ -1477,7 +1568,7 @@ class RouteSetTest < ActiveSupport::TestCase
     @set = make_set false
 
     set.draw do
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get ":controller(/:action(/:id))"
       end
     end
@@ -1495,7 +1586,7 @@ class RouteSetTest < ActiveSupport::TestCase
     set.draw do
       get "show_weblog/:parameter" => "weblog#show"
 
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get ":controller(/:action(/:id))"
       end
     end
@@ -1525,7 +1616,7 @@ class RouteSetTest < ActiveSupport::TestCase
   def test_expiry_determination_should_consider_values_with_to_param
     @set = make_set false
 
-    set.draw { ActiveSupport::Deprecation.silence { get "projects/:project_id/:controller/:action" } }
+    set.draw { ActionDispatch.deprecator.silence { get "projects/:project_id/:controller/:action" } }
 
     get URI("http://test.host/projects/1/weblog/show")
 
@@ -1605,12 +1696,12 @@ class RouteSetTest < ActiveSupport::TestCase
     set.draw do
       get "page/:name" => "pages#show", :constraints => lambda { |request|
         name_param = request.params[:name]
-        return true
+        true
       }
     end
     assert_equal({ controller: "pages", action: "show", name: "mypage" },
       set.recognize_path("http://subdomain.example.org/page/mypage"))
-    assert_equal(name_param, "mypage")
+    assert_equal("mypage", name_param)
   end
 
   def test_route_requirement_recognize_with_ignore_case
@@ -1702,7 +1793,7 @@ class RouteSetTest < ActiveSupport::TestCase
 
   def test_assign_route_options_with_anchor_chars
     set.draw do
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get "/cars/:action/:person/:car/", controller: "cars"
       end
     end
@@ -1714,7 +1805,7 @@ class RouteSetTest < ActiveSupport::TestCase
 
   def test_segmentation_of_dot_path
     set.draw do
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get "/books/:action.rss", controller: "books"
       end
     end
@@ -1726,7 +1817,7 @@ class RouteSetTest < ActiveSupport::TestCase
 
   def test_segmentation_of_dynamic_dot_path
     set.draw do
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get "/books(/:action(.:format))", controller: "books"
       end
     end
@@ -1743,7 +1834,7 @@ class RouteSetTest < ActiveSupport::TestCase
   end
 
   def test_slashes_are_implied
-    set.draw { ActiveSupport::Deprecation.silence { get("/:controller(/:action(/:id))") } }
+    set.draw { ActionDispatch.deprecator.silence { get("/:controller(/:action(/:id))") } }
 
     assert_equal "/content",        url_for(set, controller: "content", action: "index")
     assert_equal "/content/list",   url_for(set, controller: "content", action: "list")
@@ -1834,7 +1925,7 @@ class RouteSetTest < ActiveSupport::TestCase
                              :constraints => { page: /\d+/ },
                              :defaults => { page: 1 }
 
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get ":controller/:action/:id"
       end
     end
@@ -1844,8 +1935,6 @@ class RouteSetTest < ActiveSupport::TestCase
 
   include ActionDispatch::RoutingVerbs
 
-  alias :routes :set
-
   def test_generate_with_optional_params_recalls_last_request
     @set = make_set false
 
@@ -1860,7 +1949,7 @@ class RouteSetTest < ActiveSupport::TestCase
 
       get "blog/show/:id", controller: "blog", action: "show", id: /\d+/
 
-      ActiveSupport::Deprecation.silence do
+      ActionDispatch.deprecator.silence do
         get "blog/:controller/:action(/:id)"
       end
 
@@ -1952,7 +2041,7 @@ class RackMountIntegrationTests < ActiveSupport::TestCase
 
     get "news(.:format)" => "news#index"
 
-    ActiveSupport::Deprecation.silence do
+    ActionDispatch.deprecator.silence do
       get "comment/:id(/:action)" => "comments#show"
       get "ws/:controller(/:action(/:id))", ws: true
       get "account(/:action)" => "account#subscription"
@@ -1962,7 +2051,7 @@ class RackMountIntegrationTests < ActiveSupport::TestCase
 
     get "こんにちは/世界", controller: "news", action: "index"
 
-    ActiveSupport::Deprecation.silence do
+    ActionDispatch.deprecator.silence do
       match ":controller(/:action(/:id))(.:format)", via: :all
     end
 
@@ -2047,6 +2136,7 @@ class RackMountIntegrationTests < ActiveSupport::TestCase
     assert_equal({ controller: "geocode", action: "show", postalcode: "12345" }, @routes.recognize_path("/extended/geocode/12345"))
 
     assert_equal({ controller: "news", action: "index" }, @routes.recognize_path("/", method: :get))
+    assert_equal({ controller: "news", action: "index" }, @routes.recognize_path(nil))
     assert_equal({ controller: "news", action: "index", format: "rss" }, @routes.recognize_path("/news.rss", method: :get))
 
     assert_raise(ActionController::RoutingError) { @routes.recognize_path("/none", method: :get) }
@@ -2089,11 +2179,11 @@ class RackMountIntegrationTests < ActiveSupport::TestCase
   end
 
   def test_unicode_path
-    assert_equal({ controller: "news", action: "index" }, @routes.recognize_path(URI::DEFAULT_PARSER.escape("こんにちは/世界"), method: :get))
+    assert_equal({ controller: "news", action: "index" }, @routes.recognize_path(URI::RFC2396_Parser.new.escape("こんにちは/世界"), method: :get))
   end
 
   def test_downcased_unicode_path
-    assert_equal({ controller: "news", action: "index" }, @routes.recognize_path(URI::DEFAULT_PARSER.escape("こんにちは/世界").downcase, method: :get))
+    assert_equal({ controller: "news", action: "index" }, @routes.recognize_path(URI::RFC2396_Parser.new.escape("こんにちは/世界").downcase, method: :get))
   end
 
   private
