@@ -26,12 +26,6 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
   class Boomer
     attr_accessor :closed
 
-    class NilAnnotedSourceCodeError < StandardError
-      def annoted_source_code
-        nil
-      end
-    end
-
     def initialize(detailed = false)
       @detailed = detailed
       @closed = false
@@ -60,10 +54,6 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
       end
     end
 
-    def method_that_raises_nil_annoted_source_code
-      raise NilAnnotedSourceCodeError, "nil annoted_source_code"
-    end
-
     def call(env)
       env["action_dispatch.show_detailed_exceptions"] = @detailed
       req = ActionDispatch::Request.new(env)
@@ -71,10 +61,10 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
 
       case req.path
       when "/pass"
-        [404, { "X-Cascade" => "pass" }, self]
+        [404, { ActionDispatch::Constants::X_CASCADE => "pass" }, self]
       when "/not_found"
         controller = SimpleController.new
-        raise AbstractController::ActionNotFound.new(nil, controller, :not_found)
+        raise AbstractController::ActionNotFound.new(nil, controller, :ello)
       when "/runtime_error"
         raise RuntimeError
       when "/method_not_allowed"
@@ -108,7 +98,7 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
       when "/missing_keys"
         raise ActionController::UrlGenerationError, "No route matches"
       when "/parameter_missing"
-        raise ActionController::ParameterMissing.new(:missing_param_key, %w(valid_param_key))
+        raise ActionController::ParameterMissing.new(:invalid_param_key, %w(valid_param_key))
       when "/original_syntax_error"
         eval "broke_syntax =" # `eval` need for raise native SyntaxError at runtime
       when "/syntax_error_into_view"
@@ -123,8 +113,6 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
         raise_nested_exceptions
       when %r{/actionable_error}
         raise CustomActionableError
-      when %r{/nil_annoted_source_code_error}
-        method_that_raises_nil_annoted_source_code
       when "/utf8_template_error"
         begin
           eval "“fancy string”"
@@ -137,58 +125,66 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
     end
   end
 
+  def self.build_app(app, *args)
+    Rack::Lint.new(
+      ActionDispatch::DebugExceptions.new(
+        Rack::Lint.new(app), *args,
+      ),
+    )
+  end
+
   Interceptor = proc { |request, exception| request.set_header("int", exception) }
   BadInterceptor = proc { |request, exception| raise "bad" }
   RoutesApp = Struct.new(:routes).new(SharedTestRoutes)
-  ProductionApp  = ActionDispatch::DebugExceptions.new(Boomer.new(false), RoutesApp)
-  DevelopmentApp = ActionDispatch::DebugExceptions.new(Boomer.new(true), RoutesApp)
-  InterceptedApp = ActionDispatch::DebugExceptions.new(Boomer.new(true), RoutesApp, :default, [Interceptor])
-  BadInterceptedApp = ActionDispatch::DebugExceptions.new(Boomer.new(true), RoutesApp, :default, [BadInterceptor])
-  ApiApp = ActionDispatch::DebugExceptions.new(Boomer.new(true), RoutesApp, :api)
+  ProductionApp  = build_app(Boomer.new(false), RoutesApp)
+  DevelopmentApp = build_app(Boomer.new(true), RoutesApp)
+  InterceptedApp = build_app(Boomer.new(true), RoutesApp, :default, [Interceptor])
+  BadInterceptedApp = build_app(Boomer.new(true), RoutesApp, :default, [BadInterceptor])
+  ApiApp = build_app(Boomer.new(true), RoutesApp, :api)
 
   test "skip diagnosis if not showing detailed exceptions" do
     @app = ProductionApp
     assert_raise RuntimeError do
-      get "/", headers: { "action_dispatch.show_exceptions" => true }
+      get "/", headers: { "action_dispatch.show_exceptions" => :all }
     end
   end
 
   test "skip diagnosis if not showing exceptions" do
     @app = DevelopmentApp
     assert_raise RuntimeError do
-      get "/", headers: { "action_dispatch.show_exceptions" => false }
+      get "/", headers: { "action_dispatch.show_exceptions" => :none }
     end
   end
 
   test "raise an exception on cascade pass" do
     @app = ProductionApp
     assert_raise ActionController::RoutingError do
-      get "/pass", headers: { "action_dispatch.show_exceptions" => true }
+      get "/pass", headers: { "action_dispatch.show_exceptions" => :all }
     end
   end
 
   test "closes the response body on cascade pass" do
     boomer = Boomer.new(false)
-    @app = ActionDispatch::DebugExceptions.new(boomer)
+    @app = self.class.build_app(boomer)
     assert_raise ActionController::RoutingError do
-      get "/pass", headers: { "action_dispatch.show_exceptions" => true }
+      get "/pass", headers: { "action_dispatch.show_exceptions" => :all }
     end
     assert boomer.closed, "Expected to close the response body"
   end
 
   test "displays routes in a table when a RoutingError occurs" do
     @app = DevelopmentApp
-    get "/pass", headers: { "action_dispatch.show_exceptions" => true }
+    get "/pass", headers: { "action_dispatch.show_exceptions" => :all }
     routing_table = body[/route_table.*<.table>/m]
     assert_match "/:controller(/:action)(.:format)", routing_table
     assert_match ":controller#:action", routing_table
-    assert_no_match "&lt;|&gt;", routing_table, "there should not be escaped html in the output"
+    assert_no_match "&lt;|&gt;", routing_table, "there should not be escaped HTML in the output"
   end
 
   test "displays request and response info when a RoutingError occurs" do
     @app = DevelopmentApp
 
-    get "/pass", headers: { "action_dispatch.show_exceptions" => true }
+    get "/pass", headers: { "action_dispatch.show_exceptions" => :all }
 
     assert_select "h2", /Request/
     assert_select "h2", /Response/
@@ -197,37 +193,37 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
   test "rescue with diagnostics message" do
     @app = DevelopmentApp
 
-    get "/", headers: { "action_dispatch.show_exceptions" => true }
+    get "/", headers: { "action_dispatch.show_exceptions" => :all }
     assert_response 500
     assert_match(/<body>/, body)
     assert_match(/puke/, body)
 
-    get "/not_found", headers: { "action_dispatch.show_exceptions" => true }
+    get "/not_found", headers: { "action_dispatch.show_exceptions" => :all }
     assert_response 404
     assert_match(/<body>/, body)
     assert_match(/#{AbstractController::ActionNotFound.name}/, body)
 
-    get "/method_not_allowed", headers: { "action_dispatch.show_exceptions" => true }
+    get "/method_not_allowed", headers: { "action_dispatch.show_exceptions" => :all }
     assert_response 405
     assert_match(/<body>/, body)
     assert_match(/ActionController::MethodNotAllowed/, body)
 
-    get "/unknown_http_method", headers: { "action_dispatch.show_exceptions" => true }
+    get "/unknown_http_method", headers: { "action_dispatch.show_exceptions" => :all }
     assert_response 405
     assert_match(/<body>/, body)
     assert_match(/ActionController::UnknownHttpMethod/, body)
 
-    get "/bad_request", headers: { "action_dispatch.show_exceptions" => true }
+    get "/bad_request", headers: { "action_dispatch.show_exceptions" => :all }
     assert_response 400
     assert_match(/<body>/, body)
     assert_match(/ActionController::BadRequest/, body)
 
-    get "/parameter_missing", headers: { "action_dispatch.show_exceptions" => true }
+    get "/parameter_missing", headers: { "action_dispatch.show_exceptions" => :all }
     assert_response 400
     assert_match(/<body>/, body)
     assert_match(/ActionController::ParameterMissing/, body)
 
-    get "/invalid_mimetype", headers: { "Accept" => "text/html,*", "action_dispatch.show_exceptions" => true }
+    get "/invalid_mimetype", headers: { "Accept" => "text/html,*", "action_dispatch.show_exceptions" => :all }
     assert_response 406
     assert_match(/<body>/, body)
     assert_match(/ActionDispatch::Http::MimeNegotiation::InvalidType/, body)
@@ -235,7 +231,7 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
 
   test "rescue with text error for xhr request" do
     @app = DevelopmentApp
-    xhr_request_env = { "action_dispatch.show_exceptions" => true, "HTTP_X_REQUESTED_WITH" => "XMLHttpRequest" }
+    xhr_request_env = { "action_dispatch.show_exceptions" => :all, "HTTP_X_REQUESTED_WITH" => "XMLHttpRequest" }
 
     get "/", headers: xhr_request_env
     assert_response 500
@@ -286,70 +282,68 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
   test "rescue with JSON error for JSON API request" do
     @app = ApiApp
 
-    get "/", headers: { "action_dispatch.show_exceptions" => true }, as: :json
+    get "/", headers: { "action_dispatch.show_exceptions" => :all }, as: :json
     assert_response 500
     assert_no_match(/<header>/, body)
     assert_no_match(/<body>/, body)
     assert_equal "application/json", response.media_type
     assert_match(/RuntimeError: puke/, body)
 
-    get "/not_found", headers: { "action_dispatch.show_exceptions" => true }, as: :json
+    get "/not_found", headers: { "action_dispatch.show_exceptions" => :all }, as: :json
     assert_response 404
     assert_no_match(/<body>/, body)
     assert_equal "application/json", response.media_type
     assert_match(/#{AbstractController::ActionNotFound.name}/, body)
 
-    get "/method_not_allowed", headers: { "action_dispatch.show_exceptions" => true }, as: :json
+    get "/method_not_allowed", headers: { "action_dispatch.show_exceptions" => :all }, as: :json
     assert_response 405
     assert_no_match(/<body>/, body)
     assert_equal "application/json", response.media_type
     assert_match(/ActionController::MethodNotAllowed/, body)
 
-    get "/unknown_http_method", headers: { "action_dispatch.show_exceptions" => true }, as: :json
+    get "/unknown_http_method", headers: { "action_dispatch.show_exceptions" => :all }, as: :json
     assert_response 405
     assert_no_match(/<body>/, body)
     assert_equal "application/json", response.media_type
     assert_match(/ActionController::UnknownHttpMethod/, body)
 
-    get "/bad_request", headers: { "action_dispatch.show_exceptions" => true }, as: :json
+    get "/bad_request", headers: { "action_dispatch.show_exceptions" => :all }, as: :json
     assert_response 400
     assert_no_match(/<body>/, body)
     assert_equal "application/json", response.media_type
     assert_match(/ActionController::BadRequest/, body)
 
-    get "/parameter_missing", headers: { "action_dispatch.show_exceptions" => true }, as: :json
+    get "/parameter_missing", headers: { "action_dispatch.show_exceptions" => :all }, as: :json
     assert_response 400
     assert_no_match(/<body>/, body)
     assert_equal "application/json", response.media_type
     assert_match(/ActionController::ParameterMissing/, body)
 
-    get "/invalid_mimetype", headers: { "Accept" => "text/html,*", "action_dispatch.show_exceptions" => true }, as: :json
+    get "/invalid_mimetype", headers: { "Accept" => "text/html,*", "action_dispatch.show_exceptions" => :all }, as: :json
     assert_response 406
     assert_no_match(/<body>/, body)
     assert_equal "application/json", response.media_type
     assert_match(/ActionDispatch::Http::MimeNegotiation::InvalidType/, body)
   end
 
-  if defined?(DidYouMean) && DidYouMean.respond_to?(:correct_error)
-    test "rescue with suggestions" do
-      @app = DevelopmentApp
+  test "rescue with suggestions" do
+    @app = DevelopmentApp
 
-      get "/not_found", headers: { "action_dispatch.show_exceptions" => true }
-      assert_response 404
-      assert_select("b", /Did you mean\?/)
-      assert_select("li", "hello")
+    get "/not_found", headers: { "action_dispatch.show_exceptions" => :all }
+    assert_response 404
+    assert_select("b", /Did you mean\?/)
+    assert_select("li", "hello")
 
-      get "/parameter_missing", headers: { "action_dispatch.show_exceptions" => true }
-      assert_response 400
-      assert_select("b", /Did you mean\?/)
-      assert_select("li", "valid_param_key")
-    end
+    get "/parameter_missing", headers: { "action_dispatch.show_exceptions" => :all }
+    assert_response 400
+    assert_select("b", /Did you mean\?/)
+    assert_select("li", "valid_param_key")
   end
 
   test "rescue with HTML format for HTML API request" do
     @app = ApiApp
 
-    get "/index.html", headers: { "action_dispatch.show_exceptions" => true }
+    get "/index.html", headers: { "action_dispatch.show_exceptions" => :all }
     assert_response 500
     assert_match(/<header>/, body)
     assert_match(/<body>/, body)
@@ -360,7 +354,7 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
   test "rescue with XML format for XML API requests" do
     @app = ApiApp
 
-    get "/index.xml", headers: { "action_dispatch.show_exceptions" => true }
+    get "/index.xml", headers: { "action_dispatch.show_exceptions" => :all }
     assert_response 500
     assert_equal "application/xml", response.media_type
     assert_match(/RuntimeError: puke/, body)
@@ -374,7 +368,7 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
 
     @app = ApiApp
 
-    get "/index", headers: { "action_dispatch.show_exceptions" => true }, as: :wibble
+    get "/index", headers: { "action_dispatch.show_exceptions" => :all }, as: :wibble
     assert_response 500
     assert_equal "application/json", response.media_type
     assert_match(/RuntimeError: puke/, body)
@@ -386,16 +380,17 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
   test "does not show filtered parameters" do
     @app = DevelopmentApp
 
-    get "/", params: { "foo" => "bar" }, headers: { "action_dispatch.show_exceptions" => true,
+    get "/", params: { "foo" => "bar" }, headers: { "action_dispatch.show_exceptions" => :all,
       "action_dispatch.parameter_filter" => [:foo] }
     assert_response 500
-    assert_match("&quot;foo&quot;=&gt;&quot;[FILTERED]&quot;", body)
+
+    assert_match(CGI.escape_html({ "foo" => "[FILTERED]" }.inspect[1..-2]), body)
   end
 
   test "show registered original exception if the last exception is TemplateError" do
     @app = DevelopmentApp
 
-    get "/not_found_original_exception", headers: { "action_dispatch.show_exceptions" => true }
+    get "/not_found_original_exception", headers: { "action_dispatch.show_exceptions" => :all }
     assert_response 404
     assert_match %r{AbstractController::ActionNotFound}, body
     assert_match %r{Showing <i>.*test/dispatch/debug_exceptions_test.rb</i>}, body
@@ -404,7 +399,7 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
   test "show the last exception and cause even when the cause is mapped to resque_responses" do
     @app = DevelopmentApp
 
-    get "/cause_mapped_to_rescue_responses", headers: { "action_dispatch.show_exceptions" => true }
+    get "/cause_mapped_to_rescue_responses", headers: { "action_dispatch.show_exceptions" => :all }
     assert_response 500
     assert_match %r{ActionController::ParameterMissing}, body
     assert_match %r{NameError}, body
@@ -413,7 +408,7 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
   test "named URLs missing keys raise 500 level error" do
     @app = DevelopmentApp
 
-    get "/missing_keys", headers: { "action_dispatch.show_exceptions" => true }
+    get "/missing_keys", headers: { "action_dispatch.show_exceptions" => :all }
     assert_response 500
 
     assert_match(/ActionController::UrlGenerationError/, body)
@@ -422,7 +417,7 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
   test "show the controller name in the diagnostics template when controller name is present" do
     @app = DevelopmentApp
     get("/runtime_error", headers: {
-      "action_dispatch.show_exceptions" => true,
+      "action_dispatch.show_exceptions" => :all,
       "action_dispatch.request.parameters" => {
         "action" => "show",
         "id" => "unknown",
@@ -445,7 +440,7 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
     }
 
     get("/runtime_error", headers: {
-      "action_dispatch.show_exceptions" => true,
+      "action_dispatch.show_exceptions" => :all,
       "action_dispatch.request.parameters" => {
         "action" => "show",
         "controller" => "featured_tile"
@@ -459,14 +454,27 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
   test "sets the HTTP charset parameter" do
     @app = DevelopmentApp
 
-    get "/", headers: { "action_dispatch.show_exceptions" => true }
-    assert_equal "text/html; charset=utf-8", response.headers["Content-Type"]
+    get "/", headers: { "action_dispatch.show_exceptions" => :all }
+    assert_equal "text/html; charset=utf-8", response.headers["content-type"]
   end
 
   test "uses logger from env" do
     @app = DevelopmentApp
     output = StringIO.new
-    get "/", headers: { "action_dispatch.show_exceptions" => true, "action_dispatch.logger" => Logger.new(output) }
+    get "/", headers: { "action_dispatch.show_exceptions" => :all, "action_dispatch.logger" => Logger.new(output) }
+    assert_match(/puke/, output.rewind && output.read)
+  end
+
+  test "logs at configured log level" do
+    @app = DevelopmentApp
+    output = StringIO.new
+    logger = Logger.new(output)
+    logger.level = Logger::WARN
+
+    get "/", headers: { "action_dispatch.show_exceptions" => :all, "action_dispatch.logger" => logger, "action_dispatch.debug_exception_log_level" => Logger::INFO }
+    assert_no_match(/puke/, output.rewind && output.read)
+
+    get "/", headers: { "action_dispatch.show_exceptions" => :all, "action_dispatch.logger" => logger, "action_dispatch.debug_exception_log_level" => Logger::ERROR }
     assert_match(/puke/, output.rewind && output.read)
   end
 
@@ -477,7 +485,7 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
 
     _old, ActionView::Base.logger = ActionView::Base.logger, logger
     begin
-      get "/", headers: { "action_dispatch.show_exceptions" => true, "action_dispatch.logger" => logger }
+      get "/", headers: { "action_dispatch.show_exceptions" => :all, "action_dispatch.logger" => logger }
     ensure
       ActionView::Base.logger = _old
     end
@@ -501,7 +509,7 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
     _old, ActionView::Base.logger = ActionView::Base.logger, logger
     begin
       assert_nothing_raised do
-        get "/", headers: { "action_dispatch.show_exceptions" => true, "action_dispatch.logger" => logger }
+        get "/", headers: { "action_dispatch.show_exceptions" => :all, "action_dispatch.logger" => logger }
       end
     ensure
       ActionView::Base.logger = _old
@@ -515,7 +523,7 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
     backtrace_cleaner = ActiveSupport::BacktraceCleaner.new
 
     backtrace_cleaner.stub :clean, ["passed backtrace cleaner"] do
-      get "/", headers: { "action_dispatch.show_exceptions" => true, "action_dispatch.backtrace_cleaner" => backtrace_cleaner }
+      get "/", headers: { "action_dispatch.show_exceptions" => :all, "action_dispatch.backtrace_cleaner" => backtrace_cleaner }
       assert_match(/passed backtrace cleaner/, body)
     end
   end
@@ -527,9 +535,9 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
     backtrace_cleaner = ActiveSupport::BacktraceCleaner.new
     backtrace_cleaner.add_silencer { true }
 
-    env = { "action_dispatch.show_exceptions" => true,
-           "action_dispatch.logger" => Logger.new(output),
-           "action_dispatch.backtrace_cleaner" => backtrace_cleaner }
+    env = { "action_dispatch.show_exceptions"   => :all,
+            "action_dispatch.logger"            => Logger.new(output),
+            "action_dispatch.backtrace_cleaner" => backtrace_cleaner }
 
     get "/", headers: env
     assert_operator((output.rewind && output.read).lines.count, :>, 10)
@@ -542,9 +550,10 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
     backtrace_cleaner = ActiveSupport::BacktraceCleaner.new
     backtrace_cleaner.add_silencer { true }
 
-    env = { "action_dispatch.show_exceptions"   => true,
-            "action_dispatch.logger"            => Logger.new(output),
-            "action_dispatch.backtrace_cleaner" => backtrace_cleaner }
+    env = { "action_dispatch.show_exceptions"       => :all,
+            "action_dispatch.logger"                => Logger.new(output),
+            "action_dispatch.log_rescued_responses" => true,
+            "action_dispatch.backtrace_cleaner"     => backtrace_cleaner }
 
     assert_raises ActionController::RoutingError do
       get "/pass", headers: env
@@ -556,7 +565,7 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
     assert_equal 3, log.lines.count
   end
 
-  test "doesn't log the framework backtrace when error type is a invalid mine type" do
+  test "doesn't log the framework backtrace when error type is a invalid mime type" do
     @app = ProductionApp
 
     output = StringIO.new
@@ -564,9 +573,10 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
     backtrace_cleaner.add_silencer { true }
 
     env = { "Accept" => "text/html,*",
-            "action_dispatch.show_exceptions"   => true,
-            "action_dispatch.logger"            => Logger.new(output),
-            "action_dispatch.backtrace_cleaner" => backtrace_cleaner }
+            "action_dispatch.show_exceptions"       => :all,
+            "action_dispatch.logger"                => Logger.new(output),
+            "action_dispatch.log_rescued_responses" => true,
+            "action_dispatch.backtrace_cleaner"     => backtrace_cleaner }
 
     assert_raises ActionDispatch::Http::MimeNegotiation::InvalidType do
       get "/invalid_mimetype", headers: env
@@ -578,6 +588,53 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
     assert_equal 3, log.lines.count
   end
 
+  test "skips logging when rescued and log_rescued_responses is false" do
+    @app = DevelopmentApp
+
+    output = StringIO.new
+
+    env = { "action_dispatch.show_exceptions"       => :all,
+            "action_dispatch.logger"                => Logger.new(output),
+            "action_dispatch.log_rescued_responses" => false }
+
+    get "/parameter_missing", headers: env
+    assert_response 400
+    assert_empty (output.rewind && output.read).lines
+  end
+
+  test "does not skip logging when rescued and log_rescued_responses is true" do
+    @app = DevelopmentApp
+
+    output = StringIO.new
+
+    env = { "action_dispatch.show_exceptions"       => :all,
+            "action_dispatch.logger"                => Logger.new(output),
+            "action_dispatch.log_rescued_responses" => true }
+
+    get "/parameter_missing", headers: env
+    assert_response 400
+    assert_not_empty (output.rewind && output.read).lines
+  end
+
+  test "logs exception causes" do
+    @app = DevelopmentApp
+
+    output = StringIO.new
+
+    env = { "action_dispatch.show_exceptions"       => :all,
+            "action_dispatch.logger"                => Logger.new(output),
+            "action_dispatch.log_rescued_responses" => true }
+
+    get "/nested_exceptions", headers: env
+    assert_response 500
+    log = output.rewind && output.read
+    assert_includes log, <<~MSG
+      Causes:
+      RuntimeError (Second error)
+      RuntimeError (First error)
+    MSG
+  end
+
   test "display backtrace when error type is SyntaxError" do
     @app = DevelopmentApp
 
@@ -585,7 +642,7 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
 
     assert_response 500
     assert_select "#Application-Trace-0" do
-      assert_select "code", /syntax error, unexpected/
+      assert_select "code", /syntax error, unexpected|syntax errors found/
     end
   end
 
@@ -612,7 +669,7 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
 
     assert_response 500
     assert_select "#Application-Trace-0" do
-      assert_select "code", /syntax error, unexpected/
+      assert_select "code", /syntax error, unexpected|syntax errors found/
     end
     assert_match %r{Showing <i>.*test/dispatch/debug_exceptions_test.rb</i>}, body
   end
@@ -645,7 +702,7 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
 
       # assert application trace refers to line that calls method_that_raises is first
       assert_select "#Application-Trace-0" do
-        assert_select "code a:first", %r{test/dispatch/debug_exceptions_test\.rb:\d+:in `call}
+        assert_select "code a:first", %r{test/dispatch/debug_exceptions_test\.rb:\d+:in .*call}
       end
 
       # assert framework trace that threw the error is first
@@ -657,7 +714,7 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
 
   test "invoke interceptors before rendering" do
     @app = InterceptedApp
-    get "/intercepted_error", headers: { "action_dispatch.show_exceptions" => true }
+    get "/intercepted_error", headers: { "action_dispatch.show_exceptions" => :all }
 
     assert_equal InterceptedErrorInstance, request.get_header("int")
   end
@@ -665,7 +722,7 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
   test "bad interceptors doesn't debug exceptions" do
     @app = BadInterceptedApp
 
-    get "/puke", headers: { "action_dispatch.show_exceptions" => true }
+    get "/puke", headers: { "action_dispatch.show_exceptions" => :all }
 
     assert_response 500
     assert_match(/puke/, body)
@@ -691,19 +748,32 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
         assert_select "pre .line.active", /raise "Third error"/
       end
 
-      # assert application trace refers to line that raises the last exception
-      assert_select "#Application-Trace-0" do
-        assert_select "code a:first", %r{in `rescue in rescue in raise_nested_exceptions'}
-      end
+      if RUBY_VERSION >= "3.4"
+        # Possible Ruby 3.4-dev bug: https://bugs.ruby-lang.org/issues/19117#note-45
+        # assert application trace refers to line that raises the last exception
+        assert_select "#Application-Trace-0" do
+          assert_select "code a:first", %r{in '.*raise_nested_exceptions'}
+        end
 
-      # assert the second application trace refers to the line that raises the second exception
-      assert_select "#Application-Trace-1" do
-        assert_select "code a:first", %r{in `rescue in raise_nested_exceptions'}
+        # assert the second application trace refers to the line that raises the second exception
+        assert_select "#Application-Trace-1" do
+          assert_select "code a:first", %r{in '.*raise_nested_exceptions'}
+        end
+      else
+        # assert application trace refers to line that raises the last exception
+        assert_select "#Application-Trace-0" do
+          assert_select "code a:first", %r{in [`']rescue in rescue in .*raise_nested_exceptions'}
+        end
+
+        # assert the second application trace refers to the line that raises the second exception
+        assert_select "#Application-Trace-1" do
+          assert_select "code a:first", %r{in [`']rescue in .*raise_nested_exceptions'}
+        end
       end
 
       # assert the third application trace refers to the line that raises the first exception
       assert_select "#Application-Trace-2" do
-        assert_select "code a:first", %r{in `raise_nested_exceptions'}
+        assert_select "code a:first", %r{in [`'].*raise_nested_exceptions'}
       end
     end
   end
@@ -736,7 +806,7 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
 
   test "debug exceptions app shows diagnostics when malformed query parameters are provided by XHR" do
     @app = DevelopmentApp
-    xhr_request_env = { "action_dispatch.show_exceptions" => true, "HTTP_X_REQUESTED_WITH" => "XMLHttpRequest" }
+    xhr_request_env = { "action_dispatch.show_exceptions" => :all, "HTTP_X_REQUESTED_WITH" => "XMLHttpRequest" }
 
     get "/bad_request?x[y]=1&x[y][][w]=2", headers: xhr_request_env
 
@@ -744,20 +814,6 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
     assert_match "ActionController::BadRequest", body
   end
 
-  test "debug exceptions with misbehaving Exception#annoted_source_code" do
-    @app = DevelopmentApp
-
-    io = StringIO.new
-    logger = ActiveSupport::Logger.new(io)
-
-    get "/nil_annoted_source_code_error", headers: { "action_dispatch.show_exceptions" => true, "action_dispatch.logger" => logger }
-
-    assert_select "header h1", /DebugExceptionsTest::Boomer::NilAnnotedSourceCodeError/
-    assert_select "#container div.exception-message" do
-      assert_select "div", /nil annoted_source_code/
-    end
-  end
-
   test "debug exceptions app shows diagnostics for template errors that contain UTF-8 characters" do
     @app = DevelopmentApp
 
@@ -768,6 +824,6 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
 
     assert_response 500
     assert_select "#container p", /Showing #{__FILE__} where line #\d+ raised/
-    assert_select "#container code", /undefined local variable or method `string”'/
+    assert_select "#container code", /undefined local variable or method ['`]string”'/
   end
 end
