@@ -36,7 +36,9 @@ module ActionDispatch
       end
 
       def test_initialize
-        Mapper.new FakeSet.new
+        assert_nothing_raised do
+          Mapper.new FakeSet.new
+        end
       end
 
       def test_scope_raises_on_anchor
@@ -92,7 +94,7 @@ module ActionDispatch
         scope = Mapper::Scope.new({})
         ast = Journey::Parser.parse "/store/:name(*rest)"
         m = Mapper::Mapping.build(scope, FakeSet.new, ast, "foo", "bar", nil, [:get], nil, {}, true, options)
-        assert_equal(/.+?/, m.requirements[:rest])
+        assert_equal(/.+?/m, m.requirements[:rest])
       end
 
       def test_via_scope
@@ -127,7 +129,6 @@ module ActionDispatch
         fakeset = FakeSet.new
         mapper = Mapper.new fakeset
 
-        # FIXME: is this a desired behavior?
         mapper.get "/one/two/", to: "posts#index", as: :main
         assert_equal "/one/two(.:format)", fakeset.asts.first.to_s
       end
@@ -137,7 +138,7 @@ module ActionDispatch
         mapper = Mapper.new fakeset
         mapper.get "/*path", to: "pages#show"
         assert_equal "/*path(.:format)", fakeset.asts.first.to_s
-        assert_equal(/.+?/, fakeset.requirements.first[:path])
+        assert_equal(/.+?/m, fakeset.requirements.first[:path])
       end
 
       def test_map_wildcard_with_other_element
@@ -145,7 +146,7 @@ module ActionDispatch
         mapper = Mapper.new fakeset
         mapper.get "/*path/foo/:bar", to: "pages#show"
         assert_equal "/*path/foo/:bar(.:format)", fakeset.asts.first.to_s
-        assert_equal(/.+?/, fakeset.requirements.first[:path])
+        assert_equal(/.+?/m, fakeset.requirements.first[:path])
       end
 
       def test_map_wildcard_with_multiple_wildcard
@@ -153,8 +154,8 @@ module ActionDispatch
         mapper = Mapper.new fakeset
         mapper.get "/*foo/*bar", to: "pages#show"
         assert_equal "/*foo/*bar(.:format)", fakeset.asts.first.to_s
-        assert_equal(/.+?/, fakeset.requirements.first[:foo])
-        assert_equal(/.+?/, fakeset.requirements.first[:bar])
+        assert_equal(/.+?/m, fakeset.requirements.first[:foo])
+        assert_equal(/.+?/m, fakeset.requirements.first[:bar])
       end
 
       def test_map_wildcard_with_format_false
@@ -172,6 +173,15 @@ module ActionDispatch
         assert_equal "/*path.:format", fakeset.asts.first.to_s
       end
 
+      def test_can_pass_anchor_to_mount
+        fakeset = FakeSet.new
+        mapper = Mapper.new fakeset
+        app = lambda { |env| [200, {}, [""]] }
+        mapper.mount app => "/path", anchor: true
+        assert_equal "/path", fakeset.asts.first.to_s
+        assert fakeset.routes.first.path.anchored
+      end
+
       def test_raising_error_when_path_is_not_passed
         fakeset = FakeSet.new
         mapper = Mapper.new fakeset
@@ -193,6 +203,16 @@ module ActionDispatch
         end
       end
 
+      def test_raising_error_when_invalid_on_option_is_given
+        fakeset = FakeSet.new
+        mapper = Mapper.new fakeset
+        error = assert_raise ArgumentError do
+          mapper.get "/foo", on: :invalid_option
+        end
+
+        assert_equal "Unknown scope :invalid_option given to :on", error.message
+      end
+
       def test_scope_does_not_destructively_mutate_default_options
         fakeset = FakeSet.new
         mapper = Mapper.new fakeset
