@@ -34,7 +34,7 @@ class PermissionsPolicyTest < ActiveSupport::TestCase
 
   def test_invalid_directive_source
     exception = assert_raises(ArgumentError) do
-      @policy.vr [:non_existent]
+      @policy.geolocation [:non_existent]
     end
 
     assert_equal "Invalid HTTP permissions policy source: [:non_existent]", exception.message
@@ -63,21 +63,26 @@ class PermissionsPolicyMiddlewareTest < ActionDispatch::IntegrationTest
 
   test "html requests will set a policy" do
     @app = build_app(->(env) { [200, { Rack::CONTENT_TYPE => "text/html" }, []] })
-    # Dummy CONTENT_TYPE to avoid including backport of the following commit in
-    # a security-related patch:
-    # https://github.com/rails/rails/commit/060887d4c55a8b4038dd4662712007d07e74e625
-    get "/index", headers: { Rack::CONTENT_TYPE => 'cant/be-nil' }
 
-    assert_equal "text/html", response.headers['Content-Type']
-    assert_equal "gyroscope 'self'", response.headers['Feature-Policy']
+    get "/index"
+
+    assert_equal "gyroscope 'self'", response.headers[ActionDispatch::Constants::FEATURE_POLICY]
   end
 
   test "non-html requests will set a policy" do
     @app = build_app(->(env) { [200, { Rack::CONTENT_TYPE => "application/json" }, []] })
-    get "/index", headers: { Rack::CONTENT_TYPE => 'cant/be-nil' }
 
-    assert_equal "application/json", response.headers['Content-Type']
-    assert_equal "gyroscope 'self'", response.headers['Feature-Policy']
+    get "/index"
+
+    assert_equal "gyroscope 'self'", response.headers[ActionDispatch::Constants::FEATURE_POLICY]
+  end
+
+  test "existing policies will not be overwritten" do
+    @app = build_app(->(env) { [200, { ActionDispatch::Constants::FEATURE_POLICY => "gyroscope 'none'" }, []] })
+
+    get "/index"
+
+    assert_equal "gyroscope 'none'", response.headers[ActionDispatch::Constants::FEATURE_POLICY]
   end
 
   private
@@ -143,7 +148,7 @@ class PermissionsPolicyIntegrationTest < ActionDispatch::IntegrationTest
 
     def call(env)
       env["action_dispatch.permissions_policy"] = POLICY
-      env["action_dispatch.show_exceptions"] = false
+      env["action_dispatch.show_exceptions"] = :none
 
       @app.call(env)
     end
@@ -151,7 +156,9 @@ class PermissionsPolicyIntegrationTest < ActionDispatch::IntegrationTest
 
   APP = build_app(ROUTES) do |middleware|
     middleware.use PolicyConfigMiddleware
+    middleware.use Rack::Lint
     middleware.use ActionDispatch::PermissionsPolicy::Middleware
+    middleware.use Rack::Lint
   end
 
   def app
@@ -191,3 +198,87 @@ class PermissionsPolicyIntegrationTest < ActionDispatch::IntegrationTest
       assert_equal expected, response.headers["Feature-Policy"]
     end
 end
+
+class PermissionsPolicyWithHelpersIntegrationTest < ActionDispatch::IntegrationTest
+  module ApplicationHelper
+    def pigs_can_fly?
+      false
+    end
+  end
+
+  class ApplicationController < ActionController::Base
+    helper_method :sky_is_blue?
+    def sky_is_blue?
+      true
+    end
+  end
+
+  class PolicyController < ApplicationController
+    permissions_policy do |f|
+      f.gyroscope :none  unless helpers.pigs_can_fly?
+      f.usb       :self  if helpers.sky_is_blue?
+    end
+
+    def index
+      head :ok
+    end
+  end
+
+  ROUTES = ActionDispatch::Routing::RouteSet.new
+  ROUTES.draw do
+    scope module: "permissions_policy_with_helpers_integration_test" do
+      get "/", to: "policy#index"
+    end
+  end
+
+  POLICY = ActionDispatch::PermissionsPolicy.new do |p|
+    p.gyroscope :self
+  end
+
+  class PolicyConfigMiddleware
+    def initialize(app)
+      @app = app
+    end
+
+    def call(env)
+      env["action_dispatch.permissions_policy"] = POLICY
+      env["action_dispatch.show_exceptions"] = :none
+
+      @app.call(env)
+    end
+  end
+
+  APP = build_app(ROUTES) do |middleware|
+    middleware.use PolicyConfigMiddleware
+    middleware.use Rack::Lint
+    middleware.use ActionDispatch::PermissionsPolicy::Middleware
+    middleware.use Rack::Lint
+  end
+
+  def app
+    APP
+  end
+
+  def test_generates_permissions_policy_header
+    get "/"
+    assert_policy "gyroscope 'none'; usb 'self'"
+  end
+
+  private
+    def env_config
+      Rails.application.env_config
+    end
+
+    def permissions_policy
+      env_config["action_dispatch.permissions_policy"]
+    end
+
+    def permissions_policy=(policy)
+      env_config["action_dispatch.permissions_policy"] = policy
+    end
+
+    def assert_policy(expected)
+      assert_response :success
+      assert_equal expected, response.headers[ActionDispatch::Constants::FEATURE_POLICY]
+    end
+end
