@@ -18,27 +18,34 @@ class JsonParamsParsingTest < ActionDispatch::IntegrationTest
     TestController.last_request_parameters = nil
   end
 
-  test "parses json params for application json" do
+  test "parses JSON params for application JSON" do
     assert_parses(
       { "person" => { "name" => "David" } },
       "{\"person\": {\"name\": \"David\"}}", "CONTENT_TYPE" => "application/json"
     )
   end
 
-  test "parses boolean and number json params for application json" do
+  test "parses boolean and number JSON params for application JSON" do
     assert_parses(
       { "item" => { "enabled" => false, "count" => 10 } },
       "{\"item\": {\"enabled\": false, \"count\": 10}}", "CONTENT_TYPE" => "application/json"
     )
   end
 
-  test "parses json params for application jsonrequest" do
+  test "parses JSON params for application jsonrequest" do
     assert_parses(
       { "person" => { "name" => "David" } },
       "{\"person\": {\"name\": \"David\"}}", "CONTENT_TYPE" => "application/jsonrequest"
     )
   end
 
+  test "parses JSON params for application problem+json" do
+    assert_parses(
+      { "person" => { "name" => "David" } },
+      "{\"person\": {\"name\": \"David\"}}", "CONTENT_TYPE" => "application/problem+json"
+    )
+  end
+
   test "does not parse unregistered media types such as application/vnd.api+json" do
     assert_parses(
       {},
@@ -65,7 +72,7 @@ class JsonParamsParsingTest < ActionDispatch::IntegrationTest
     with_test_routing do
       output = StringIO.new
       json = "[\"person]\": {\"name\": \"David\"}}"
-      post "/parse", params: json, headers: { "CONTENT_TYPE" => "application/json", "action_dispatch.show_exceptions" => true, "action_dispatch.logger" => ActiveSupport::Logger.new(output) }
+      post "/parse", params: json, headers: { "CONTENT_TYPE" => "application/json", "action_dispatch.show_exceptions" => :all, "action_dispatch.logger" => ActiveSupport::Logger.new(output) }
       assert_response :bad_request
       output.rewind && err = output.read
       assert err.match?(/Error occurred while parsing request parameters/)
@@ -77,10 +84,10 @@ class JsonParamsParsingTest < ActionDispatch::IntegrationTest
       $stderr = StringIO.new # suppress the log
       json = "[\"person]\": {\"name\": \"David\"}}"
       exception = assert_raise(ActionDispatch::Http::Parameters::ParseError) do
-        post "/parse", params: json, headers: { "CONTENT_TYPE" => "application/json", "action_dispatch.show_exceptions" => false }
+        post "/parse", params: json, headers: { "CONTENT_TYPE" => "application/json", "action_dispatch.show_exceptions" => :none }
       end
       assert_equal JSON::ParserError, exception.cause.class
-      assert_equal exception.cause.message, exception.message
+      assert_equal "Error occurred while parsing request parameters", exception.message
     ensure
       $stderr = STDERR
     end
@@ -105,7 +112,7 @@ class JsonParamsParsingTest < ActionDispatch::IntegrationTest
     def with_test_routing
       with_routing do |set|
         set.draw do
-          ActiveSupport::Deprecation.silence do
+          ActionDispatch.deprecator.silence do
             post ":action", to: ::JsonParamsParsingTest::TestController
           end
         end
@@ -133,28 +140,35 @@ class RootLessJSONParamsParsingTest < ActionDispatch::IntegrationTest
     UsersController.last_request_parameters = nil
   end
 
-  test "parses json params for application json" do
+  test "parses JSON params for application JSON" do
     assert_parses(
       { "user" => { "username" => "sikachu" }, "username" => "sikachu" },
       "{\"username\": \"sikachu\"}", "CONTENT_TYPE" => "application/json"
     )
   end
 
-  test "parses json params for application jsonrequest" do
+  test "parses JSON params for application jsonrequest" do
     assert_parses(
       { "user" => { "username" => "sikachu" }, "username" => "sikachu" },
       "{\"username\": \"sikachu\"}", "CONTENT_TYPE" => "application/jsonrequest"
     )
   end
 
-  test "parses json with non-object JSON content" do
+  test "parses JSON params for application problem+json" do
+    assert_parses(
+      { "user" => { "username" => "sikachu" }, "username" => "sikachu" },
+      "{\"username\": \"sikachu\"}", "CONTENT_TYPE" => "application/problem+json"
+    )
+  end
+
+  test "parses JSON with non-object JSON content" do
     assert_parses(
       { "user" => { "_json" => "string content" }, "_json" => "string content" },
       "\"string content\"", "CONTENT_TYPE" => "application/json"
     )
   end
 
-  test "parses json params after custom json mime type registered" do
+  test "parses JSON params after custom JSON mime type registered" do
     Mime::Type.unregister :json
     Mime::Type.register "application/json", :json, %w(application/vnd.rails+json)
     assert_parses(
@@ -163,10 +177,10 @@ class RootLessJSONParamsParsingTest < ActionDispatch::IntegrationTest
     )
   ensure
     Mime::Type.unregister :json
-    Mime::Type.register "application/json", :json, %w( text/x-json application/jsonrequest )
+    Mime::Type.register "application/json", :json, %w( text/x-json application/jsonrequest application/problem+json )
   end
 
-  test "parses json params after custom json mime type registered with synonym" do
+  test "parses JSON params after custom JSON mime type registered with synonym" do
     Mime::Type.unregister :json
     Mime::Type.register "application/json", :json, %w(application/vnd.rails+json)
     assert_parses(
@@ -175,7 +189,7 @@ class RootLessJSONParamsParsingTest < ActionDispatch::IntegrationTest
     )
   ensure
     Mime::Type.unregister :json
-    Mime::Type.register "application/json", :json, %w( text/x-json application/jsonrequest )
+    Mime::Type.register "application/json", :json, %w( text/x-json application/jsonrequest application/problem+json )
   end
 
   private
@@ -191,7 +205,7 @@ class RootLessJSONParamsParsingTest < ActionDispatch::IntegrationTest
     def with_test_routing(controller)
       with_routing do |set|
         set.draw do
-          ActiveSupport::Deprecation.silence do
+          ActionDispatch.deprecator.silence do
             post ":action", to: controller
           end
         end
