@@ -33,39 +33,38 @@ class ShowExceptionsTest < ActionDispatch::IntegrationTest
     end
   end
 
-  ProductionApp = ActionDispatch::ShowExceptions.new(Boomer.new, ActionDispatch::PublicExceptions.new("#{FIXTURE_LOAD_PATH}/public"))
+  def setup
+    @app = build_app
+  end
 
   test "skip exceptions app if not showing exceptions" do
-    @app = ProductionApp
     assert_raise RuntimeError do
-      get "/", env: { "action_dispatch.show_exceptions" => false }
+      get "/", env: { "action_dispatch.show_exceptions" => :none }
     end
   end
 
   test "rescue with error page" do
-    @app = ProductionApp
-
-    get "/", env: { "action_dispatch.show_exceptions" => true }
+    get "/", env: { "action_dispatch.show_exceptions" => :all }
     assert_response 500
     assert_equal "500 error fixture\n", body
 
-    get "/bad_params", env: { "action_dispatch.show_exceptions" => true }
+    get "/bad_params", env: { "action_dispatch.show_exceptions" => :all }
     assert_response 400
     assert_equal "400 error fixture\n", body
 
-    get "/not_found", env: { "action_dispatch.show_exceptions" => true }
+    get "/not_found", env: { "action_dispatch.show_exceptions" => :all }
     assert_response 404
     assert_equal "404 error fixture\n", body
 
-    get "/method_not_allowed", env: { "action_dispatch.show_exceptions" => true }
+    get "/method_not_allowed", env: { "action_dispatch.show_exceptions" => :all }
     assert_response 405
     assert_equal "", body
 
-    get "/unknown_http_method", env: { "action_dispatch.show_exceptions" => true }
+    get "/unknown_http_method", env: { "action_dispatch.show_exceptions" => :all }
     assert_response 405
     assert_equal "", body
 
-    get "/invalid_mimetype", headers: { "Accept" => "text/html,*", "action_dispatch.show_exceptions" => true }
+    get "/invalid_mimetype", headers: { "Accept" => "text/html,*", "action_dispatch.show_exceptions" => :all }
     assert_response 406
     assert_equal "", body
   end
@@ -74,13 +73,11 @@ class ShowExceptionsTest < ActionDispatch::IntegrationTest
     old_locale, I18n.locale = I18n.locale, :da
 
     begin
-      @app = ProductionApp
-
-      get "/", env: { "action_dispatch.show_exceptions" => true }
+      get "/", env: { "action_dispatch.show_exceptions" => :all }
       assert_response 500
       assert_equal "500 localized error fixture\n", body
 
-      get "/not_found", env: { "action_dispatch.show_exceptions" => true }
+      get "/not_found", env: { "action_dispatch.show_exceptions" => :all }
       assert_response 404
       assert_equal "404 error fixture\n", body
     ensure
@@ -89,16 +86,12 @@ class ShowExceptionsTest < ActionDispatch::IntegrationTest
   end
 
   test "sets the HTTP charset parameter" do
-    @app = ProductionApp
-
-    get "/", env: { "action_dispatch.show_exceptions" => true }
-    assert_equal "text/html; charset=utf-8", response.headers["Content-Type"]
+    get "/", env: { "action_dispatch.show_exceptions" => :all }
+    assert_equal "text/html; charset=utf-8", response.headers["content-type"]
   end
 
   test "show registered original exception for wrapped exceptions" do
-    @app = ProductionApp
-
-    get "/not_found_original_exception", env: { "action_dispatch.show_exceptions" => true }
+    get "/not_found_original_exception", env: { "action_dispatch.show_exceptions" => :all }
     assert_response 404
     assert_match(/404 error/, body)
   end
@@ -108,37 +101,55 @@ class ShowExceptionsTest < ActionDispatch::IntegrationTest
       assert_kind_of AbstractController::ActionNotFound, env["action_dispatch.exception"]
       assert_equal "/404", env["PATH_INFO"]
       assert_equal "/not_found_original_exception", env["action_dispatch.original_path"]
-      [404, { "Content-Type" => "text/plain" }, ["YOU FAILED"]]
+      [404, { "content-type" => "text/plain" }, ["YOU FAILED"]]
     end
 
-    @app = ActionDispatch::ShowExceptions.new(Boomer.new, exceptions_app)
-    get "/not_found_original_exception", env: { "action_dispatch.show_exceptions" => true }
+    @app = build_app(exceptions_app)
+
+    get "/not_found_original_exception", env: { "action_dispatch.show_exceptions" => :all }
     assert_response 404
     assert_equal "YOU FAILED", body
   end
 
-  test "returns an empty response if custom exceptions app returns X-Cascade pass" do
+  test "returns an empty response if custom exceptions app returns x-cascade pass" do
     exceptions_app = lambda do |env|
-      [404, { "X-Cascade" => "pass" }, []]
+      [404, { ActionDispatch::Constants::X_CASCADE => "pass" }, []]
     end
 
-    @app = ActionDispatch::ShowExceptions.new(Boomer.new, exceptions_app)
-    get "/method_not_allowed", env: { "action_dispatch.show_exceptions" => true }
+    @app = build_app(exceptions_app)
+
+    get "/method_not_allowed", env: { "action_dispatch.show_exceptions" => :all }
     assert_response 405
     assert_equal "", body
   end
 
   test "bad params exception is returned in the correct format" do
-    @app = ProductionApp
-
-    get "/bad_params", env: { "action_dispatch.show_exceptions" => true }
-    assert_equal "text/html; charset=utf-8", response.headers["Content-Type"]
+    get "/bad_params", env: { "action_dispatch.show_exceptions" => :all }
+    assert_equal "text/html; charset=utf-8", response.headers["content-type"]
     assert_response 400
     assert_match(/400 error/, body)
 
-    get "/bad_params.json", env: { "action_dispatch.show_exceptions" => true }
-    assert_equal "application/json; charset=utf-8", response.headers["Content-Type"]
+    get "/bad_params.json", env: { "action_dispatch.show_exceptions" => :all }
+    assert_equal "application/json; charset=utf-8", response.headers["content-type"]
     assert_response 400
     assert_equal("{\"status\":400,\"error\":\"Bad Request\"}", body)
   end
+
+  test "failsafe prevents raising if exceptions_app raises" do
+    old_stderr, $stderr = $stderr, StringIO.new
+    @app = build_app(->(_) { raise })
+
+    get "/"
+
+    assert_response 500
+    assert_match(/500 Internal Server Error/, body)
+  ensure
+    $stderr = old_stderr
+  end
+
+  private
+    def build_app(exceptions_app = nil)
+      exceptions_app ||= ActionDispatch::PublicExceptions.new("#{FIXTURE_LOAD_PATH}/public")
+      Rack::Lint.new(ActionDispatch::ShowExceptions.new(Rack::Lint.new(Boomer.new), exceptions_app))
+    end
 end
