@@ -19,18 +19,18 @@ class TextHelperTest < ActionView::TestCase
   end
 
   def test_simple_format_should_be_html_safe
-    assert_predicate simple_format("<b> test with html tags </b>"), :html_safe?
+    assert_predicate simple_format("<b> test with HTML tags </b>"), :html_safe?
   end
 
   def test_simple_format_included_in_isolation
     helper_klass = Class.new { include ActionView::Helpers::TextHelper }
-    assert_predicate helper_klass.new.simple_format("<b> test with html tags </b>"), :html_safe?
+    assert_predicate helper_klass.new.simple_format("<b> test with HTML tags </b>"), :html_safe?
   end
 
   def test_simple_format
     assert_equal "<p></p>", simple_format(nil)
 
-    assert_equal "<p>crazy\n<br /> cross\n<br /> platform linebreaks</p>", simple_format("crazy\r\n cross\r platform linebreaks")
+    assert_equal "<p>ridiculous\n<br /> cross\n<br /> platform linebreaks</p>", simple_format("ridiculous\r\n cross\r platform linebreaks")
     assert_equal "<p>A paragraph</p>\n\n<p>and another one!</p>", simple_format("A paragraph\n\nand another one!")
     assert_equal "<p>A paragraph\n<br /> With a newline</p>", simple_format("A paragraph\n With a newline")
 
@@ -53,12 +53,22 @@ class TextHelperTest < ActionView::TestCase
       simple_format("<b> test with unsafe string </b><script>code!</script>", {}, { sanitize: true })
   end
 
+  def test_simple_format_should_sanitize_input_when_sanitize_options_is_specified
+    assert_equal "<p><a target=\"_blank\" href=\"http://example.com\">Continue</a></p>",
+      simple_format("<a target=\"_blank\" href=\"http://example.com\">Continue</a>", {}, { sanitize_options: { attributes: %w[target href] } })
+  end
+
+  def test_simple_format_should_sanitize_input_when_sanitize_options_is_not_specified
+    assert_equal "<p><a href=\"http://example.com\">Continue</a></p>", simple_format("<a target=\"_blank\" href=\"http://example.com\">Continue</a>")
+  end
+
   def test_simple_format_should_not_sanitize_input_when_sanitize_option_is_false
     assert_equal "<p><b> test with unsafe string </b><script>code!</script></p>", simple_format("<b> test with unsafe string </b><script>code!</script>", {}, { sanitize: false })
   end
 
   def test_simple_format_with_custom_wrapper
     assert_equal "<div></div>", simple_format(nil, {}, { wrapper_tag: "div" })
+    assert_equal "<p></p>", simple_format(nil, {}, { wrapper_tag: nil })
   end
 
   def test_simple_format_with_custom_wrapper_and_multi_line_breaks
@@ -353,27 +363,39 @@ class TextHelperTest < ActionView::TestCase
                  excerpt("This is a beautiful morning", "a", separator: nil)
   end
 
-  def test_word_wrap
-    assert_equal("my very very\nvery long\nstring", word_wrap("my very very very long string", line_width: 15))
+  test "word_wrap" do
+    input = "123 1234 12 12 123 1 1 1 123"
+    assert_equal "123\n1234\n12\n12\n123\n1 1\n1\n123", word_wrap(input, line_width: 3)
+    assert_equal "123-+1234-+12-+12-+123-+1 1-+1-+123", word_wrap(input, line_width: 3, break_sequence: "-+")
   end
 
-  def test_word_wrap_with_extra_newlines
-    assert_equal("my very very\nvery long\nstring\n\nwith another\nline", word_wrap("my very very very long string\n\nwith another line", line_width: 15))
+  test "word_wrap with newlines" do
+    input = "1\n1 1 1\n1"
+    assert_equal "1\n1 1\n1\n1", word_wrap(input, line_width: 3)
+    assert_equal "1-+1 1-+1-+1", word_wrap(input, line_width: 3, break_sequence: "-+")
   end
 
-  def test_word_wrap_with_leading_spaces
-    assert_equal("  This is a paragraph\nthat includes some\nindented lines:\n  Like this sample\n  blockquote", word_wrap("  This is a paragraph that includes some\nindented lines:\n  Like this sample\n  blockquote", line_width: 25))
+  test "word_wrap with multiple consecutive newlines" do
+    input = "1\n\n\n1 1 1\n\n\n1"
+    assert_equal "1\n\n\n1 1\n1\n\n\n1", word_wrap(input, line_width: 3)
+    assert_equal "1-+-+-+1 1-+1-+-+-+1", word_wrap(input, line_width: 3, break_sequence: "-+")
   end
 
-  def test_word_wrap_does_not_modify_the_options_hash
-    options = { line_width: 15 }
-    passed_options = options.dup
-    word_wrap("some text", **passed_options)
-    assert_equal options, passed_options
+  test "word_wrap with trailing newlines" do
+    input = "1\n1 1 1\n1\n\n\n"
+    assert_equal "1\n1 1\n1\n1", word_wrap(input, line_width: 3)
+    assert_equal "1-+1 1-+1-+1", word_wrap(input, line_width: 3, break_sequence: "-+")
   end
 
-  def test_word_wrap_with_custom_break_sequence
-    assert_equal("1234567890\r\n1234567890\r\n1234567890", word_wrap("1234567890 " * 3, line_width: 2, break_sequence: "\r\n"))
+  test "word_wrap with leading spaces" do
+    input = "  1 1\n  1 1\n"
+    assert_equal "  1\n1\n  1\n1", word_wrap(input, line_width: 3)
+    assert_equal "  1-+1-+  1-+1", word_wrap(input, line_width: 3, break_sequence: "-+")
+  end
+
+  test "word_wrap when no wrapping is necessary" do
+    assert_equal "1", word_wrap("1", line_width: 3)
+    assert_equal "", word_wrap("", line_width: 3)
   end
 
   def test_pluralization
@@ -516,7 +538,9 @@ class TextHelperTest < ActionView::TestCase
   end
 
   def test_reset_unknown_cycle
-    reset_cycle("colors")
+    assert_nothing_raised do
+      reset_cycle("colors")
+    end
   end
 
   def test_reset_named_cycle
