@@ -20,6 +20,14 @@ module ActiveModel
         type = Type::BigInteger.new
         assert_equal 9999999999999999999999999999999, type.serialize(9999999999999999999999999999999)
       end
+
+      test "serialize_cast_value is equivalent to serialize after cast" do
+        type = Type::BigInteger.new
+        value = type.cast(9999999999999999999999999999999)
+
+        assert_equal type.serialize(value), type.serialize_cast_value(value)
+        assert_equal type.serialize(-value), type.serialize_cast_value(-value)
+      end
     end
   end
 end
