@@ -1,6 +1,7 @@
 # frozen_string_literal: true
 
 require "cases/helper"
+require "active_support/core_ext/object/json"
 
 module ActiveModel
   module Type
@@ -10,6 +11,12 @@ module ActiveModel
         assert_not_equal Type::Value.new, Type::Integer.new
         assert_not_equal Type::Value.new(precision: 1), Type::Value.new(precision: 2)
       end
+
+      def test_as_json_not_defined
+        assert_raises NoMethodError do
+          Type::Value.new.as_json
+        end
+      end
     end
   end
 end
