@@ -13,8 +13,8 @@ class I18nValidationTest < ActiveModel::TestCase
     I18n.backend = I18n::Backend::Simple.new
     I18n.backend.store_translations("en", errors: { messages: { custom: nil } })
 
-    @original_i18n_customize_full_message = ActiveModel::Errors.i18n_customize_full_message
-    ActiveModel::Errors.i18n_customize_full_message = true
+    @original_i18n_customize_full_message = ActiveModel::Error.i18n_customize_full_message
+    ActiveModel::Error.i18n_customize_full_message = true
   end
 
   def teardown
@@ -24,7 +24,7 @@ class I18nValidationTest < ActiveModel::TestCase
     I18n.load_path.replace @old_load_path
     I18n.backend = @old_backend
     I18n.backend.reload!
-    ActiveModel::Errors.i18n_customize_full_message = @original_i18n_customize_full_message
+    ActiveModel::Error.i18n_customize_full_message = @original_i18n_customize_full_message
   end
 
   def test_full_message_encoding
@@ -37,7 +37,7 @@ class I18nValidationTest < ActiveModel::TestCase
 
   def test_errors_full_messages_translates_human_attribute_name_for_model_attributes
     @person.errors.add(:name, "not found")
-    assert_called_with(person_class, :human_attribute_name, ["name", default: "Name"], returns: "Person's name") do
+    assert_called_with(person_class, :human_attribute_name, ["name", default: "Name", base: @person], returns: "Person's name") do
       assert_equal ["Person's name not found"], @person.errors.full_messages
     end
   end
@@ -49,7 +49,7 @@ class I18nValidationTest < ActiveModel::TestCase
   end
 
   def test_errors_full_messages_doesnt_use_attribute_format_without_config
-    ActiveModel::Errors.i18n_customize_full_message = false
+    ActiveModel::Error.i18n_customize_full_message = false
 
     I18n.backend.store_translations("en", activemodel: {
       errors: { models: { person: { attributes: { name: { format: "%{message}" } } } } } })
@@ -59,8 +59,21 @@ class I18nValidationTest < ActiveModel::TestCase
     assert_equal "Name test cannot be blank", person.errors.full_message(:name_test, "cannot be blank")
   end
 
+  def test_errors_full_messages_on_nested_error_uses_attribute_format
+    ActiveModel::Error.i18n_customize_full_message = true
+    I18n.backend.store_translations("en", activemodel: {
+      errors: { models: { person: { attributes: { gender: "Gender" } } } },
+      attributes: { "person/contacts": { gender: "Gender" } }
+    })
+
+    person = person_class.new
+    error = ActiveModel::Error.new(person, :gender, "can't be blank")
+    person.errors.import(error, attribute: "person[0].contacts.gender")
+    assert_equal ["Gender can't be blank"], person.errors.full_messages
+  end
+
   def test_errors_full_messages_uses_attribute_format
-    ActiveModel::Errors.i18n_customize_full_message = true
+    ActiveModel::Error.i18n_customize_full_message = true
 
     I18n.backend.store_translations("en", activemodel: {
       errors: { models: { person: { attributes: { name: { format: "%{message}" } } } } } })
@@ -71,7 +84,7 @@ class I18nValidationTest < ActiveModel::TestCase
   end
 
   def test_errors_full_messages_uses_model_format
-    ActiveModel::Errors.i18n_customize_full_message = true
+    ActiveModel::Error.i18n_customize_full_message = true
 
     I18n.backend.store_translations("en", activemodel: {
       errors: { models: { person: { format: "%{message}" } } } })
@@ -82,7 +95,7 @@ class I18nValidationTest < ActiveModel::TestCase
   end
 
   def test_errors_full_messages_uses_deeply_nested_model_attributes_format
-    ActiveModel::Errors.i18n_customize_full_message = true
+    ActiveModel::Error.i18n_customize_full_message = true
 
     I18n.backend.store_translations("en", activemodel: {
       errors: { models: { 'person/contacts/addresses': { attributes: { street: { format: "%{message}" } } } } } })
@@ -93,7 +106,7 @@ class I18nValidationTest < ActiveModel::TestCase
   end
 
   def test_errors_full_messages_uses_deeply_nested_model_model_format
-    ActiveModel::Errors.i18n_customize_full_message = true
+    ActiveModel::Error.i18n_customize_full_message = true
 
     I18n.backend.store_translations("en", activemodel: {
       errors: { models: { 'person/contacts/addresses': { format: "%{message}" } } } })
@@ -104,41 +117,41 @@ class I18nValidationTest < ActiveModel::TestCase
   end
 
   def test_errors_full_messages_with_indexed_deeply_nested_attributes_and_attributes_format
-    ActiveModel::Errors.i18n_customize_full_message = true
+    ActiveModel::Error.i18n_customize_full_message = true
 
     I18n.backend.store_translations("en", activemodel: {
       errors: { models: { 'person/contacts/addresses': { attributes: { street: { format: "%{message}" } } } } } })
 
     person = person_class.new
-    assert_equal "cannot be blank", person.errors.full_message(:'contacts[0]/addresses[0].street', "cannot be blank")
-    assert_equal "Contacts/addresses country cannot be blank", person.errors.full_message(:'contacts[0]/addresses[0].country', "cannot be blank")
+    assert_equal "cannot be blank", person.errors.full_message(:'contacts[0]/addresses[123].street', "cannot be blank")
+    assert_equal "Contacts/addresses country cannot be blank", person.errors.full_message(:'contacts[0]/addresses[123].country', "cannot be blank")
   end
 
   def test_errors_full_messages_with_indexed_deeply_nested_attributes_and_model_format
-    ActiveModel::Errors.i18n_customize_full_message = true
+    ActiveModel::Error.i18n_customize_full_message = true
 
     I18n.backend.store_translations("en", activemodel: {
       errors: { models: { 'person/contacts/addresses': { format: "%{message}" } } } })
 
     person = person_class.new
-    assert_equal "cannot be blank", person.errors.full_message(:'contacts[0]/addresses[0].street', "cannot be blank")
-    assert_equal "cannot be blank", person.errors.full_message(:'contacts[0]/addresses[0].country', "cannot be blank")
+    assert_equal "cannot be blank", person.errors.full_message(:'contacts[0]/addresses[123].street', "cannot be blank")
+    assert_equal "cannot be blank", person.errors.full_message(:'contacts[0]/addresses[123].country', "cannot be blank")
   end
 
   def test_errors_full_messages_with_indexed_deeply_nested_attributes_and_i18n_attribute_name
-    ActiveModel::Errors.i18n_customize_full_message = true
+    ActiveModel::Error.i18n_customize_full_message = true
 
     I18n.backend.store_translations("en", activemodel: {
       attributes: { 'person/contacts/addresses': { country: "Country" } }
     })
 
     person = person_class.new
-    assert_equal "Contacts/addresses street cannot be blank", person.errors.full_message(:'contacts[0]/addresses[0].street', "cannot be blank")
-    assert_equal "Country cannot be blank", person.errors.full_message(:'contacts[0]/addresses[0].country', "cannot be blank")
+    assert_equal "Contacts/addresses street cannot be blank", person.errors.full_message(:'contacts[0]/addresses[123].street', "cannot be blank")
+    assert_equal "Country cannot be blank", person.errors.full_message(:'contacts[0]/addresses[123].country', "cannot be blank")
   end
 
   def test_errors_full_messages_with_indexed_deeply_nested_attributes_without_i18n_config
-    ActiveModel::Errors.i18n_customize_full_message = false
+    ActiveModel::Error.i18n_customize_full_message = false
 
     I18n.backend.store_translations("en", activemodel: {
       errors: { models: { 'person/contacts/addresses': { attributes: { street: { format: "%{message}" } } } } } })
@@ -149,7 +162,7 @@ class I18nValidationTest < ActiveModel::TestCase
   end
 
   def test_errors_full_messages_with_i18n_attribute_name_without_i18n_config
-    ActiveModel::Errors.i18n_customize_full_message = false
+    ActiveModel::Error.i18n_customize_full_message = false
 
     I18n.backend.store_translations("en", activemodel: {
       attributes: { 'person/contacts[0]/addresses[0]': { country: "Country" } }
@@ -169,8 +182,8 @@ class I18nValidationTest < ActiveModel::TestCase
     # [ case,                              validation_options,            generate_message_options]
     [ "given no options",                  {},                            {}],
     [ "given custom message",              { message: "custom" },         { message: "custom" }],
-    [ "given if condition",                { if:                          lambda { true } },  {}],
-    [ "given unless condition",            { unless:                      lambda { false } }, {}],
+    [ "given if condition",                { if: lambda { true } },       {}],
+    [ "given unless condition",            { unless: lambda { false } },  {}],
     [ "given option that is not reserved", { format: "jpg" },             { format: "jpg" }]
   ]
 
@@ -178,9 +191,10 @@ class I18nValidationTest < ActiveModel::TestCase
     test "validates_confirmation_of on generated message #{name}" do
       person_class.validates_confirmation_of :title, validation_options
       @person.title_confirmation = "foo"
-      call = [:title_confirmation, :confirmation, generate_message_options.merge(attribute: "Title")]
-      assert_called_with(@person.errors, :generate_message, call) do
+      call = [:title_confirmation, :confirmation, @person, generate_message_options.merge(attribute: "Title")]
+      assert_called_with(ActiveModel::Error, :generate_message, call) do
         @person.valid?
+        @person.errors.messages
       end
     end
   end
@@ -188,9 +202,10 @@ class I18nValidationTest < ActiveModel::TestCase
   COMMON_CASES.each do |name, validation_options, generate_message_options|
     test "validates_acceptance_of on generated message #{name}" do
       person_class.validates_acceptance_of :title, validation_options.merge(allow_nil: false)
-      call = [:title, :accepted, generate_message_options]
-      assert_called_with(@person.errors, :generate_message, call) do
+      call = [:title, :accepted, @person, generate_message_options]
+      assert_called_with(ActiveModel::Error, :generate_message, call) do
         @person.valid?
+        @person.errors.messages
       end
     end
   end
@@ -198,9 +213,10 @@ class I18nValidationTest < ActiveModel::TestCase
   COMMON_CASES.each do |name, validation_options, generate_message_options|
     test "validates_presence_of on generated message #{name}" do
       person_class.validates_presence_of :title, validation_options
-      call = [:title, :blank, generate_message_options]
-      assert_called_with(@person.errors, :generate_message, call) do
+      call = [:title, :blank, @person, generate_message_options]
+      assert_called_with(ActiveModel::Error, :generate_message, call) do
         @person.valid?
+        @person.errors.messages
       end
     end
   end
@@ -208,9 +224,10 @@ class I18nValidationTest < ActiveModel::TestCase
   COMMON_CASES.each do |name, validation_options, generate_message_options|
     test "validates_length_of for :within on generated message when too short #{name}" do
       person_class.validates_length_of :title, validation_options.merge(within: 3..5)
-      call = [:title, :too_short, generate_message_options.merge(count: 3)]
-      assert_called_with(@person.errors, :generate_message, call) do
+      call = [:title, :too_short, @person, generate_message_options.merge(count: 3)]
+      assert_called_with(ActiveModel::Error, :generate_message, call) do
         @person.valid?
+        @person.errors.messages
       end
     end
   end
@@ -219,9 +236,10 @@ class I18nValidationTest < ActiveModel::TestCase
     test "validates_length_of for :too_long generated message #{name}" do
       person_class.validates_length_of :title, validation_options.merge(within: 3..5)
       @person.title = "this title is too long"
-      call = [:title, :too_long, generate_message_options.merge(count: 5)]
-      assert_called_with(@person.errors, :generate_message, call) do
+      call = [:title, :too_long, @person, generate_message_options.merge(count: 5)]
+      assert_called_with(ActiveModel::Error, :generate_message, call) do
         @person.valid?
+        @person.errors.messages
       end
     end
   end
@@ -229,9 +247,10 @@ class I18nValidationTest < ActiveModel::TestCase
   COMMON_CASES.each do |name, validation_options, generate_message_options|
     test "validates_length_of for :is on generated message #{name}" do
       person_class.validates_length_of :title, validation_options.merge(is: 5)
-      call = [:title, :wrong_length, generate_message_options.merge(count: 5)]
-      assert_called_with(@person.errors, :generate_message, call) do
+      call = [:title, :wrong_length, @person, generate_message_options.merge(count: 5)]
+      assert_called_with(ActiveModel::Error, :generate_message, call) do
         @person.valid?
+        @person.errors.messages
       end
     end
   end
@@ -240,9 +259,10 @@ class I18nValidationTest < ActiveModel::TestCase
     test "validates_format_of on generated message #{name}" do
       person_class.validates_format_of :title, validation_options.merge(with: /\A[1-9][0-9]*\z/)
       @person.title = "72x"
-      call = [:title, :invalid, generate_message_options.merge(value: "72x")]
-      assert_called_with(@person.errors, :generate_message, call) do
+      call = [:title, :invalid, @person, generate_message_options.merge(value: "72x")]
+      assert_called_with(ActiveModel::Error, :generate_message, call) do
         @person.valid?
+        @person.errors.messages
       end
     end
   end
@@ -251,9 +271,10 @@ class I18nValidationTest < ActiveModel::TestCase
     test "validates_inclusion_of on generated message #{name}" do
       person_class.validates_inclusion_of :title, validation_options.merge(in: %w(a b c))
       @person.title = "z"
-      call = [:title, :inclusion, generate_message_options.merge(value: "z")]
-      assert_called_with(@person.errors, :generate_message, call) do
+      call = [:title, :inclusion, @person, generate_message_options.merge(value: "z")]
+      assert_called_with(ActiveModel::Error, :generate_message, call) do
         @person.valid?
+        @person.errors.messages
       end
     end
   end
@@ -262,9 +283,10 @@ class I18nValidationTest < ActiveModel::TestCase
     test "validates_inclusion_of using :within on generated message #{name}" do
       person_class.validates_inclusion_of :title, validation_options.merge(within: %w(a b c))
       @person.title = "z"
-      call = [:title, :inclusion, generate_message_options.merge(value: "z")]
-      assert_called_with(@person.errors, :generate_message, call) do
+      call = [:title, :inclusion, @person, generate_message_options.merge(value: "z")]
+      assert_called_with(ActiveModel::Error, :generate_message, call) do
         @person.valid?
+        @person.errors.messages
       end
     end
   end
@@ -273,9 +295,10 @@ class I18nValidationTest < ActiveModel::TestCase
     test "validates_exclusion_of generated message #{name}" do
       person_class.validates_exclusion_of :title, validation_options.merge(in: %w(a b c))
       @person.title = "a"
-      call = [:title, :exclusion, generate_message_options.merge(value: "a")]
-      assert_called_with(@person.errors, :generate_message, call) do
+      call = [:title, :exclusion, @person, generate_message_options.merge(value: "a")]
+      assert_called_with(ActiveModel::Error, :generate_message, call) do
         @person.valid?
+        @person.errors.messages
       end
     end
   end
@@ -284,9 +307,10 @@ class I18nValidationTest < ActiveModel::TestCase
     test "validates_exclusion_of using :within generated message #{name}" do
       person_class.validates_exclusion_of :title, validation_options.merge(within: %w(a b c))
       @person.title = "a"
-      call = [:title, :exclusion, generate_message_options.merge(value: "a")]
-      assert_called_with(@person.errors, :generate_message, call) do
+      call = [:title, :exclusion, @person, generate_message_options.merge(value: "a")]
+      assert_called_with(ActiveModel::Error, :generate_message, call) do
         @person.valid?
+        @person.errors.messages
       end
     end
   end
@@ -295,9 +319,10 @@ class I18nValidationTest < ActiveModel::TestCase
     test "validates_numericality_of generated message #{name}" do
       person_class.validates_numericality_of :title, validation_options
       @person.title = "a"
-      call = [:title, :not_a_number, generate_message_options.merge(value: "a")]
-      assert_called_with(@person.errors, :generate_message, call) do
+      call = [:title, :not_a_number, @person, generate_message_options.merge(value: "a")]
+      assert_called_with(ActiveModel::Error, :generate_message, call) do
         @person.valid?
+        @person.errors.messages
       end
     end
   end
@@ -306,9 +331,10 @@ class I18nValidationTest < ActiveModel::TestCase
     test "validates_numericality_of for :only_integer on generated message #{name}" do
       person_class.validates_numericality_of :title, validation_options.merge(only_integer: true)
       @person.title = "0.0"
-      call = [:title, :not_an_integer, generate_message_options.merge(value: "0.0")]
-      assert_called_with(@person.errors, :generate_message, call) do
+      call = [:title, :not_an_integer, @person, generate_message_options.merge(value: "0.0")]
+      assert_called_with(ActiveModel::Error, :generate_message, call) do
         @person.valid?
+        @person.errors.messages
       end
     end
   end
@@ -317,9 +343,10 @@ class I18nValidationTest < ActiveModel::TestCase
     test "validates_numericality_of for :odd on generated message #{name}" do
       person_class.validates_numericality_of :title, validation_options.merge(only_integer: true, odd: true)
       @person.title = 0
-      call = [:title, :odd, generate_message_options.merge(value: 0)]
-      assert_called_with(@person.errors, :generate_message, call) do
+      call = [:title, :odd, @person, generate_message_options.merge(value: 0)]
+      assert_called_with(ActiveModel::Error, :generate_message, call) do
         @person.valid?
+        @person.errors.messages
       end
     end
   end
@@ -328,9 +355,10 @@ class I18nValidationTest < ActiveModel::TestCase
     test "validates_numericality_of for :less_than on generated message #{name}" do
       person_class.validates_numericality_of :title, validation_options.merge(only_integer: true, less_than: 0)
       @person.title = 1
-      call = [:title, :less_than, generate_message_options.merge(value: 1, count: 0)]
-      assert_called_with(@person.errors, :generate_message, call) do
+      call = [:title, :less_than, @person, generate_message_options.merge(value: 1, count: 0)]
+      assert_called_with(ActiveModel::Error, :generate_message, call) do
         @person.valid?
+        @person.errors.messages
       end
     end
   end
