@@ -2,14 +2,18 @@
 
 class User
   extend ActiveModel::Callbacks
+  include ActiveModel::Attributes
+  include ActiveModel::Dirty
   include ActiveModel::SecurePassword
 
   define_model_callbacks :create
 
+  attribute :password_digest
   has_secure_password
+
+  attribute :recovery_password_digest
   has_secure_password :recovery_password, validations: false
 
-  attr_accessor :password_digest, :recovery_password_digest
   attr_accessor :password_called
 
   def password=(unencrypted_password)
