@@ -7,8 +7,8 @@ module ActiveRecord
     class PostgreSQLAdapter
       class QuotingTest < ActiveRecord::PostgreSQLTestCase
         def setup
-          @conn = ActiveRecord::Base.connection
-          @raise_int_wider_than_64bit = ActiveRecord::Base.raise_int_wider_than_64bit
+          @conn = ActiveRecord::Base.lease_connection
+          @raise_int_wider_than_64bit = ActiveRecord.raise_int_wider_than_64bit
         end
 
         def test_type_cast_true
@@ -46,6 +46,25 @@ module ActiveRecord
           assert_equal "\"user posts\"", @conn.quote_table_name(value)
         end
 
+        def test_quote_string
+          assert_equal "''", @conn.quote_string("'")
+        end
+
+        def test_quote_column_name
+          [@conn, @conn.class].each do |adapter|
+            assert_equal '"foo"', adapter.quote_column_name("foo")
+            assert_equal '"hel""lo"', adapter.quote_column_name(%{hel"lo})
+          end
+        end
+
+        def test_quote_table_name
+          [@conn, @conn.class].each do |adapter|
+            assert_equal '"foo"', adapter.quote_table_name("foo")
+            assert_equal '"foo"."bar"', adapter.quote_table_name("foo.bar")
+            assert_equal '"hel""lo.wol\\d"', adapter.quote_column_name('hel"lo.wol\\d')
+          end
+        end
+
         def test_raise_when_int_is_wider_than_64bit
           value = 9223372036854775807 + 1
           assert_raise ActiveRecord::ConnectionAdapters::PostgreSQL::Quoting::IntegerOutOf64BitRange do
@@ -67,10 +86,10 @@ module ActiveRecord
         end
 
         def test_do_not_raise_when_raise_int_wider_than_64bit_is_false
-          ActiveRecord::Base.raise_int_wider_than_64bit = false
+          ActiveRecord.raise_int_wider_than_64bit = false
           value = 9223372036854775807 + 1
           assert_equal "9223372036854775808", @conn.quote(value)
-          ActiveRecord::Base.raise_int_wider_than_64bit = @raise_int_wider_than_64bit
+          ActiveRecord.raise_int_wider_than_64bit = @raise_int_wider_than_64bit
         end
       end
     end
