@@ -1,7 +1,6 @@
 # frozen_string_literal: true
 
 require_relative "../helper"
-require "ostruct"
 
 module Arel
   module Attributes
@@ -393,7 +392,7 @@ module Arel
           attribute = Attribute.new nil, nil
           equality = attribute.eq 1
           _(equality.left).must_equal attribute
-          _(equality.right.val).must_equal 1
+          _(equality.right.value).must_equal 1
           _(equality).must_be_kind_of Nodes::Equality
         end
 
@@ -638,28 +637,34 @@ module Arel
           )
         end
 
-        if Gem::Version.new("2.7.0") <= Gem::Version.new(RUBY_VERSION)
-          it "can be constructed with a range implicitly starting at Infinity" do
-            attribute = Attribute.new nil, nil
-            node = attribute.between(eval("..0")) # eval for backwards compatibility
+        it "can be constructed with a range implicitly starting at Infinity" do
+          attribute = Attribute.new nil, nil
+          node = attribute.between(..0)
 
-            node.must_equal Nodes::LessThanOrEqual.new(
-              attribute,
-              Nodes::Casted.new(0, attribute)
-            )
-          end
+          _(node).must_equal Nodes::LessThanOrEqual.new(
+            attribute,
+            Nodes::Casted.new(0, attribute)
+          )
         end
 
-        if Gem::Version.new("2.6.0") <= Gem::Version.new(RUBY_VERSION)
-          it "can be constructed with a range implicitly ending at Infinity" do
-            attribute = Attribute.new nil, nil
-            node = attribute.between(eval("0..")) # Use eval for compatibility with Ruby < 2.6 parser
+        it "can be constructed with a range implicitly ending at Infinity" do
+          attribute = Attribute.new nil, nil
+          node = attribute.between(0..)
 
-            _(node).must_equal Nodes::GreaterThanOrEqual.new(
-              attribute,
-              Nodes::Casted.new(0, attribute)
-            )
-          end
+          _(node).must_equal Nodes::GreaterThanOrEqual.new(
+            attribute,
+            Nodes::Casted.new(0, attribute)
+          )
+        end
+
+        it "can be constructed with an exclusive range implicitly ending at Infinity" do
+          attribute = Attribute.new nil, nil
+          node = attribute.between(0...)
+
+          _(node).must_equal Nodes::GreaterThanOrEqual.new(
+            attribute,
+            Nodes::Casted.new(0, attribute)
+          )
         end
 
         it "can be constructed with a quoted range ending at Infinity" do
@@ -672,6 +677,20 @@ module Arel
           )
         end
 
+        it "can be constructed with an endless range starting from Infinity" do
+          attribute = Attribute.new nil, nil
+          node = attribute.between(::Float::INFINITY..)
+
+          _(node).must_equal Nodes::In.new(attribute, [])
+        end
+
+        it "can be constructed with a beginless range ending in -Infinity" do
+          attribute = Attribute.new nil, nil
+          node = attribute.between(..-::Float::INFINITY)
+
+          _(node).must_equal Nodes::In.new(attribute, [])
+        end
+
         it "can be constructed with an exclusive range" do
           attribute = Attribute.new nil, nil
           node = attribute.between(0...3)
@@ -687,6 +706,16 @@ module Arel
             )
           ])
         end
+
+        it "can be constructed with a range where the begin and end are equal" do
+          attribute = Attribute.new nil, nil
+          node = attribute.between(1..1)
+
+          _(node).must_equal Nodes::Equality.new(
+            attribute,
+            Nodes::Casted.new(1, attribute)
+          )
+        end
       end
 
       describe "#in" do
@@ -774,7 +803,7 @@ module Arel
           node = attribute.not_between(1..3)
 
           _(node).must_equal Nodes::Grouping.new(
-            Nodes::Or.new(
+            Nodes::Or.new([
               Nodes::LessThan.new(
                 attribute,
                 Nodes::Casted.new(1, attribute)
@@ -783,7 +812,7 @@ module Arel
                 attribute,
                 Nodes::Casted.new(3, attribute)
               )
-            )
+            ])
           )
         end
 
@@ -851,28 +880,24 @@ module Arel
           )
         end
 
-        if Gem::Version.new("2.7.0") <= Gem::Version.new(RUBY_VERSION)
-          it "can be constructed with a range implicitly starting at Infinity" do
-            attribute = Attribute.new nil, nil
-            node = attribute.not_between(eval("..0")) # eval for backwards compatibility
+        it "can be constructed with a range implicitly starting at Infinity" do
+          attribute = Attribute.new nil, nil
+          node = attribute.not_between(..0)
 
-            node.must_equal Nodes::GreaterThan.new(
-              attribute,
-              Nodes::Casted.new(0, attribute)
-            )
-          end
+          _(node).must_equal Nodes::GreaterThan.new(
+            attribute,
+            Nodes::Casted.new(0, attribute)
+          )
         end
 
-        if Gem::Version.new("2.6.0") <= Gem::Version.new(RUBY_VERSION)
-          it "can be constructed with a range implicitly ending at Infinity" do
-            attribute = Attribute.new nil, nil
-            node = attribute.not_between(eval("0..")) # Use eval for compatibility with Ruby < 2.6 parser
+        it "can be constructed with a range implicitly ending at Infinity" do
+          attribute = Attribute.new nil, nil
+          node = attribute.not_between(0..)
 
-            _(node).must_equal Nodes::LessThan.new(
-              attribute,
-              Nodes::Casted.new(0, attribute)
-            )
-          end
+          _(node).must_equal Nodes::LessThan.new(
+            attribute,
+            Nodes::Casted.new(0, attribute)
+          )
         end
 
         it "can be constructed with a quoted range ending at Infinity" do
@@ -885,12 +910,26 @@ module Arel
           )
         end
 
+        it "can be constructed with an endless range starting from Infinity" do
+          attribute = Attribute.new nil, nil
+          node = attribute.not_between(::Float::INFINITY..)
+
+          _(node).must_equal Nodes::NotIn.new(attribute, [])
+        end
+
+        it "can be constructed with a beginless range ending in -Infinity" do
+          attribute = Attribute.new nil, nil
+          node = attribute.not_between(..-::Float::INFINITY)
+
+          _(node).must_equal Nodes::NotIn.new(attribute, [])
+        end
+
         it "can be constructed with an exclusive range" do
           attribute = Attribute.new nil, nil
           node = attribute.not_between(0...3)
 
           _(node).must_equal Nodes::Grouping.new(
-            Nodes::Or.new(
+            Nodes::Or.new([
               Nodes::LessThan.new(
                 attribute,
                 Nodes::Casted.new(0, attribute)
@@ -899,7 +938,7 @@ module Arel
                 attribute,
                 Nodes::Casted.new(3, attribute)
               )
-            )
+            ])
           )
         end
       end
@@ -1043,6 +1082,34 @@ module Arel
         end
       end
 
+      describe "#contains" do
+        it "should create a Contains node" do
+          relation = Table.new(:products)
+          _(relation[:tags].contains(["foo", "bar"])).must_be_kind_of Nodes::Contains
+        end
+
+        it "should generate @> in sql" do
+          relation = Table.new(:products, type_caster: fake_pg_caster)
+          mgr = relation.project relation[:id]
+          mgr.where relation[:tags].contains(["foo", "bar"])
+          _(mgr.to_sql).must_be_like %{ SELECT "products"."id" FROM "products" WHERE "products"."tags" @> '{foo,bar}' }
+        end
+      end
+
+      describe "#overlaps" do
+        it "should create an Overlaps node" do
+          relation = Table.new(:products)
+          _(relation[:tags].overlaps(["foo", "bar"])).must_be_kind_of Nodes::Overlaps
+        end
+
+        it "should generate && in sql" do
+          relation = Table.new(:products, type_caster: fake_pg_caster)
+          mgr = relation.project relation[:id]
+          mgr.where relation[:tags].overlaps(["foo", "bar"])
+          _(mgr.to_sql).must_be_like %{ SELECT "products"."id" FROM "products" WHERE "products"."tags" && '{foo,bar}' }
+        end
+      end
+
       describe "equality" do
         describe "#to_sql" do
           it "should produce sql" do
@@ -1074,7 +1141,7 @@ module Arel
           table = Table.new(:foo, type_caster: fake_caster)
           condition = table["id"].eq("1").and(table["other_id"].eq("2"))
 
-          assert table.able_to_type_cast?
+          assert_predicate table, :able_to_type_cast?
           _(condition.to_sql).must_equal %("foo"."id" = 1 AND "foo"."other_id" = '2')
         end
 
@@ -1086,19 +1153,32 @@ module Arel
           table = Table.new(:foo, type_caster: fake_caster)
           condition = table["id"].eq(Arel.sql("(select 1)"))
 
-          assert table.able_to_type_cast?
+          assert_predicate table, :able_to_type_cast?
           _(condition.to_sql).must_equal %("foo"."id" = (select 1))
         end
       end
 
       private
         def quoted_range(begin_val, end_val, exclude)
-          OpenStruct.new(
-            begin: Nodes::Quoted.new(begin_val),
-            end: Nodes::Quoted.new(end_val),
-            exclude_end?: exclude,
+          Struct.new(:begin, :end, :exclude_end?).new(
+            Nodes::Quoted.new(begin_val),
+            Nodes::Quoted.new(end_val),
+            exclude,
           )
         end
+
+        # Mimic PG::TextDecoder::Array casting
+        def fake_pg_caster
+          Object.new.tap do |caster|
+            def caster.type_cast_for_database(attr_name, value)
+              if attr_name == "tags"
+                "{#{value.join(",")}}"
+              else
+                value
+              end
+            end
+          end
+        end
     end
   end
 end
