@@ -8,7 +8,7 @@ module Arel
     class TestBind < Arel::Test
       def setup
         @conn = FakeRecord::Base.new
-        @visitor = Visitors::ToSql.new @conn.connection
+        @visitor = Visitors::ToSql.new @conn.lease_connection
         super
       end
 
