@@ -10,14 +10,19 @@ require "models/categorization"
 require "models/person"
 require "models/tagging"
 require "models/tag"
+require "models/sharded/blog_post"
+require "models/sharded/comment"
+require "models/friendship"
+require "models/reference"
+require "models/job"
 
 class InnerJoinAssociationTest < ActiveRecord::TestCase
   fixtures :authors, :author_addresses, :essays, :posts, :comments, :categories, :categories_posts, :categorizations,
-           :taggings, :tags, :people
+           :taggings, :tags, :people, :sharded_comments, :sharded_blog_posts
 
   def test_construct_finder_sql_applies_aliases_tables_on_association_conditions
-    result = Author.joins(:thinking_posts, :welcome_posts).to_a
-    assert_equal authors(:david), result.first
+    result = Author.joins(:thinking_posts, :welcome_posts).first
+    assert_equal authors(:david), result
   end
 
   def test_construct_finder_sql_does_not_table_name_collide_on_duplicate_associations
@@ -41,7 +46,7 @@ class InnerJoinAssociationTest < ActiveRecord::TestCase
     SQL
 
     expected = people(:susan)
-    assert_sql(/agents_people_2/i) do
+    assert_queries_match(/agents_people_2/i) do
       assert_equal [expected], Person.joins(:agents).joins(string_join)
     end
   end
@@ -52,7 +57,7 @@ class InnerJoinAssociationTest < ActiveRecord::TestCase
     constraint = agents[:primary_contact_id].eq(agents_2[:id]).and(agents[:id].gt(agents_2[:id]))
 
     expected = people(:susan)
-    assert_sql(/agents_people_2/i) do
+    assert_queries_match(/agents_people_2/i) do
       assert_equal [expected], Person.joins(:agents).joins(agents.create_join(agents, agents.create_on(constraint)))
     end
   end
@@ -178,7 +183,7 @@ class InnerJoinAssociationTest < ActiveRecord::TestCase
 
   def test_find_with_conditions_on_reflection
     assert_not_empty posts(:welcome).comments
-    assert Post.joins(:nonexistent_comments).where(id: posts(:welcome).id).empty? # [sic!]
+    assert_predicate Post.joins(:nonexistent_comments).where(id: posts(:welcome).id), :empty? # [sic!]
   end
 
   def test_find_with_conditions_on_through_reflection
@@ -212,4 +217,29 @@ class InnerJoinAssociationTest < ActiveRecord::TestCase
     assert_equal 0, categories.first.special_categorizations.size
     assert_equal 1, categories.second.special_categorizations.size
   end
+
+  test "joins a belongs_to association with a composite foreign key" do
+    first_post_comments = Sharded::Comment.joins(:blog_post).where(blog_post: { title: "My first post in my Blog1!" }).to_a
+    expected_blog_post = sharded_blog_posts(:great_post_blog_one)
+
+    assert_not_empty comments
+    assert_equal(expected_blog_post.comments.to_a.sort, first_post_comments.sort)
+  end
+
+  test "joins a has_many association with a composite foreign key" do
+    blog_posts = Sharded::BlogPost.joins(:comments).where(comments: { body: "Your first blog post is great!" }).to_a
+
+    expected_comment = sharded_comments(:unique_comment_blog_post_one)
+
+    assert_not_empty blog_posts
+    assert_equal(expected_comment.blog_post, blog_posts.first)
+  end
+
+  def test_inner_joins_includes_all_nested_associations
+    sql, = capture_sql { Friendship.joins(:friend_favorite_reference_job, :follower_favorite_reference_job).to_a }
+
+    escape = -> name { Regexp.escape(Friendship.lease_connection.quote_table_name(name)) }
+    assert_match %r(#{escape["friendships.friend_id"]}), sql
+    assert_match %r(#{escape["friendships.follower_id"]}), sql
+  end
 end
