@@ -64,7 +64,7 @@ module ActiveRecord
         assert_equal "70000", default_after
       end
 
-      if current_adapter?(:Mysql2Adapter)
+      if current_adapter?(:Mysql2Adapter, :TrilogyAdapter)
         def test_mysql_rename_column_preserves_auto_increment
           rename_column "test_models", "id", "id_test"
           assert_predicate connection.columns("test_models").find { |c| c.name == "id_test" }, :auto_increment?
@@ -75,7 +75,7 @@ module ActiveRecord
       end
 
       def test_rename_nonexistent_column
-        exception = if current_adapter?(:PostgreSQLAdapter, :OracleAdapter)
+        exception = if current_adapter?(:PostgreSQLAdapter)
           ActiveRecord::StatementInvalid
         else
           ActiveRecord::ActiveRecordError
@@ -136,7 +136,7 @@ module ActiveRecord
       def test_remove_column_with_multi_column_index
         # MariaDB starting with 10.2.8
         # Dropping a column that is part of a multi-column UNIQUE constraint is not permitted.
-        skip if current_adapter?(:Mysql2Adapter) && connection.mariadb? && connection.database_version >= "10.2.8"
+        skip if current_adapter?(:Mysql2Adapter, :TrilogyAdapter) && connection.mariadb? && connection.database_version >= "10.2.8"
 
         add_column "test_models", :hat_size, :integer
         add_column "test_models", :hat_style, :string, limit: 100
@@ -147,7 +147,7 @@ module ActiveRecord
 
         # Every database and/or database adapter has their own behavior
         # if it drops the multi-column index when any of the indexed columns dropped by remove_column.
-        if current_adapter?(:PostgreSQLAdapter, :OracleAdapter)
+        if current_adapter?(:PostgreSQLAdapter)
           assert_equal [], connection.indexes("test_models").map(&:name)
         else
           assert_equal ["index_test_models_on_hat_style_and_hat_size"], connection.indexes("test_models").map(&:name)
@@ -273,9 +273,23 @@ module ActiveRecord
       def test_change_column_default_to_null
         add_column "test_models", "first_name", :string
         connection.change_column_default "test_models", "first_name", nil
+
         assert_nil TestModel.new.first_name
       end
 
+      def test_change_column_default_to_null_with_not_null
+        add_column "test_models", "first_name", :string, null: false
+        add_column "test_models", "age", :integer, null: false
+
+        connection.change_column_default "test_models", "first_name", nil
+
+        assert_nil TestModel.new.first_name
+
+        connection.change_column_default "test_models", "age", nil
+
+        assert_nil TestModel.new.age
+      end
+
       def test_change_column_default_with_from_and_to
         add_column "test_models", "first_name", :string
         connection.change_column_default "test_models", "first_name", from: nil, to: "Tester"
@@ -283,6 +297,55 @@ module ActiveRecord
         assert_equal "Tester", TestModel.new.first_name
       end
 
+      def test_change_column_default_preserves_existing_column_default_function
+        skip unless current_adapter?(:SQLite3Adapter)
+
+        connection.change_column_default "test_models", "created_at", -> { "CURRENT_TIMESTAMP" }
+        TestModel.reset_column_information
+        assert_equal "CURRENT_TIMESTAMP", TestModel.columns_hash["created_at"].default_function
+
+        add_column "test_models", "edited_at", :datetime
+        connection.change_column_default "test_models", "edited_at", -> { "CURRENT_TIMESTAMP" }
+        TestModel.reset_column_information
+        assert_equal "CURRENT_TIMESTAMP", TestModel.columns_hash["created_at"].default_function
+        assert_equal "CURRENT_TIMESTAMP", TestModel.columns_hash["edited_at"].default_function
+      end
+
+      def test_change_column_default_supports_default_function_with_concatenation_operator
+        skip unless current_adapter?(:SQLite3Adapter)
+
+        add_column "test_models", "ruby_on_rails", :string
+        connection.change_column_default "test_models", "ruby_on_rails", -> { "('Ruby ' || 'on ' || 'Rails')" }
+        TestModel.reset_column_information
+        assert_equal "'Ruby ' || 'on ' || 'Rails'", TestModel.columns_hash["ruby_on_rails"].default_function
+      end
+
+      def test_change_column_null_false
+        add_column "test_models", "first_name", :string
+        connection.change_column_null "test_models", "first_name", false
+
+        assert_raise(ActiveRecord::NotNullViolation) do
+          TestModel.create!(first_name: nil)
+        end
+      end
+
+      def test_change_column_null_true
+        add_column "test_models", "first_name", :string
+        connection.change_column_null "test_models", "first_name", true
+
+        assert_difference("TestModel.count" => 1) do
+          TestModel.create!(first_name: nil)
+        end
+      end
+
+      def test_change_column_null_with_non_boolean_arguments_raises
+        add_column "test_models", "first_name", :string
+        e = assert_raise(ArgumentError) do
+          connection.change_column_null "test_models", "first_name", from: true, to: false
+        end
+        assert_equal "change_column_null expects a boolean value (true for NULL, false for NOT NULL). Got: #{{ from: true, to: false }}", e.message
+      end
+
       def test_remove_column_no_second_parameter_raises_exception
         assert_raise(ArgumentError) { connection.remove_column("funny") }
       end
@@ -321,6 +384,43 @@ module ActiveRecord
       ensure
         connection.drop_table :my_table, if_exists: true
       end
+
+      def test_remove_columns_single_statement
+        connection.create_table "my_table" do |t|
+          t.integer "col_one"
+          t.integer "col_two"
+        end
+
+        # SQLite3's ALTER TABLE statement has several limitations. To manage
+        # this, the adapter creates a temporary table, copies the data, drops
+        # the old table, creates the new table, then copies the data back.
+        expected_query_count = current_adapter?(:SQLite3Adapter) ? 14 : 1
+        assert_queries_count(expected_query_count) do
+          connection.remove_columns("my_table", "col_one", "col_two")
+        end
+
+        columns = connection.columns("my_table").map(&:name)
+        assert_equal ["id"], columns
+      ensure
+        connection.drop_table :my_table, if_exists: true
+      end
+
+      def test_add_timestamps_single_statement
+        connection.create_table "my_table"
+
+        # SQLite3's ALTER TABLE statement has several limitations. To manage
+        # this, the adapter creates a temporary table, copies the data, drops
+        # the old table, creates the new table, then copies the data back.
+        expected_query_count = current_adapter?(:SQLite3Adapter) ? 14 : 1
+        assert_queries_count(expected_query_count) do
+          connection.add_timestamps("my_table")
+        end
+
+        columns = connection.columns("my_table").map(&:name)
+        assert_equal ["id", "created_at", "updated_at"], columns
+      ensure
+        connection.drop_table :my_table, if_exists: true
+      end
     end
   end
 end
