@@ -18,7 +18,7 @@ module ActiveRecord
 
       def setup
         super
-        @connection = ActiveRecord::Base.connection
+        @connection = ActiveRecord::Base.lease_connection
         connection.create_table :test_models do |t|
           t.timestamps null: true
         end
