@@ -17,20 +17,23 @@ module ActiveRecord
 
       def setup
         super
-        @schema_migration = ActiveRecord::Base.connection.schema_migration
+        @schema_migration = ActiveRecord::Base.connection_pool.schema_migration
         @schema_migration.create_table
-        @schema_migration.delete_all
+        @schema_migration.delete_all_versions
+        @internal_metadata = ActiveRecord::Base.connection_pool.internal_metadata
       end
 
       teardown do
-        @schema_migration.drop_table
+        @schema_migration&.delete_all_versions
       end
 
       def test_migration_should_be_run_without_logger
         previous_logger = ActiveRecord::Base.logger
         ActiveRecord::Base.logger = nil
         migrations = [Migration.new("a", 1), Migration.new("b", 2), Migration.new("c", 3)]
-        ActiveRecord::Migrator.new(:up, migrations, @schema_migration).migrate
+        assert_nothing_raised do
+          ActiveRecord::Migrator.new(:up, migrations, @schema_migration, @internal_metadata).migrate
+        end
       ensure
         ActiveRecord::Base.logger = previous_logger
       end
