@@ -10,19 +10,19 @@ module ActiveRecord
 
     def setup
       @original_logger = ActiveRecord::Base.logger
-      @original_warn_on_records_fetched_greater_than = ActiveRecord::Base.warn_on_records_fetched_greater_than
+      @original_warn_on_records_fetched_greater_than = ActiveRecord.warn_on_records_fetched_greater_than
       @log = StringIO.new
     end
 
     def teardown
       ActiveRecord::Base.logger = @original_logger
-      ActiveRecord::Base.warn_on_records_fetched_greater_than = @original_warn_on_records_fetched_greater_than
+      ActiveRecord.warn_on_records_fetched_greater_than = @original_warn_on_records_fetched_greater_than
     end
 
     def test_warn_on_records_fetched_greater_than_allowed_limit
       ActiveRecord::Base.logger = ActiveSupport::Logger.new(@log)
       ActiveRecord::Base.logger.level = Logger::WARN
-      ActiveRecord::Base.warn_on_records_fetched_greater_than = 1
+      ActiveRecord.warn_on_records_fetched_greater_than = 1
 
       Post.all.to_a
 
@@ -32,7 +32,7 @@ module ActiveRecord
     def test_does_not_warn_on_records_fetched_less_than_allowed_limit
       ActiveRecord::Base.logger = ActiveSupport::Logger.new(@log)
       ActiveRecord::Base.logger.level = Logger::WARN
-      ActiveRecord::Base.warn_on_records_fetched_greater_than = 100
+      ActiveRecord.warn_on_records_fetched_greater_than = 100
 
       Post.all.to_a
 
