@@ -1,5 +1,6 @@
 # frozen_string_literal: true
 
 class Message < ActiveRecord::Base
-  has_one :entry, as: :entryable
+  has_one  :entry, as: :entryable, touch: true
+  has_many :recipients
 end
