@@ -4,6 +4,7 @@ class Reference < ActiveRecord::Base
   belongs_to :person
   belongs_to :job
 
+  has_many :ideal_jobs, class_name: "Job", foreign_key: :ideal_reference_id
   has_many :agents_posts_authors, through: :person
 
   class << self; attr_accessor :make_comments; end
@@ -20,5 +21,5 @@ end
 
 class BadReference < ActiveRecord::Base
   self.table_name = "references"
-  default_scope { where(favourite: false) }
+  default_scope { where(favorite: false) }
 end
