@@ -1,6 +1,6 @@
 # frozen_string_literal: true
 
 class TrafficLight < ActiveRecord::Base
-  serialize :state, Array
-  serialize :long_state, Array
+  serialize :state, type: Array
+  serialize :long_state, type: Array
 end
