@@ -3,7 +3,7 @@
 module ConnectionHelper
   def run_without_connection
     original_connection = ActiveRecord::Base.remove_connection
-    yield original_connection
+    yield original_connection.configuration_hash
   ensure
     ActiveRecord::Base.establish_connection(original_connection)
   end
