@@ -28,8 +28,8 @@ class ProtectedParams
   end
   alias to_unsafe_h to_h
 
-  def stringify_keys
-    dup
+  def each_pair(&block)
+    @parameters.each_pair(&block)
   end
 
   def dup
