@@ -4,7 +4,11 @@ module ActiveStorage
   class Previewer::PopplerPDFPreviewer < Previewer
     class << self
       def accept?(blob)
-        blob.content_type == "application/pdf" && pdftoppm_exists?
+        pdf?(blob.content_type) && pdftoppm_exists?
+      end
+
+      def pdf?(content_type)
+        Marcel::Magic.child? content_type, "application/pdf"
       end
 
       def pdftoppm_path
@@ -12,7 +16,7 @@ module ActiveStorage
       end
 
       def pdftoppm_exists?
-        return @pdftoppm_exists if defined?(@pdftoppm_exists)
+        return @pdftoppm_exists unless @pdftoppm_exists.nil?
 
         @pdftoppm_exists = system(pdftoppm_path, "-v", out: File::NULL, err: File::NULL)
       end
