@@ -1,13 +1,17 @@
 # frozen_string_literal: true
 
-require_relative "behaviors/autoloading_cache_behavior"
 require_relative "behaviors/cache_delete_matched_behavior"
 require_relative "behaviors/cache_increment_decrement_behavior"
 require_relative "behaviors/cache_instrumentation_behavior"
+require_relative "behaviors/cache_logging_behavior"
 require_relative "behaviors/cache_store_behavior"
 require_relative "behaviors/cache_store_version_behavior"
 require_relative "behaviors/cache_store_coder_behavior"
+require_relative "behaviors/cache_store_compression_behavior"
+require_relative "behaviors/cache_store_format_version_behavior"
+require_relative "behaviors/cache_store_serializer_behavior"
 require_relative "behaviors/connection_pool_behavior"
 require_relative "behaviors/encoded_key_cache_behavior"
 require_relative "behaviors/failure_safety_behavior"
+require_relative "behaviors/failure_raising_behavior"
 require_relative "behaviors/local_cache_behavior"
