@@ -2,6 +2,7 @@
 
 require_relative "../abstract_unit"
 require "active_support/core_ext/class"
+require "active_support/descendants_tracker"
 require "set"
 
 class ClassTest < ActiveSupport::TestCase
@@ -37,4 +38,18 @@ class ClassTest < ActiveSupport::TestCase
     klass = Parent.new.singleton_class
     assert_not Parent.subclasses.include?(klass), "subclasses should not include singleton classes"
   end
+
+  def test_subclasses_exclude_reloaded_classes
+    subclass = Class.new(Parent)
+    assert_includes Parent.subclasses, subclass
+    ActiveSupport::DescendantsTracker.clear(Set[subclass])
+    assert_not_includes Parent.subclasses, subclass
+  end
+
+  def test_descendants_exclude_reloaded_classes
+    subclass = Class.new(Parent)
+    assert_includes Parent.descendants, subclass
+    ActiveSupport::DescendantsTracker.clear(Set[subclass])
+    assert_not_includes Parent.descendants, subclass
+  end
 end
