@@ -1,10 +1,10 @@
 # frozen_string_literal: true
 
 require_relative "../abstract_unit"
-require "active_support/core_ext/hash"
 require "bigdecimal"
+require "yaml"
+require "active_support/core_ext/hash"
 require "active_support/core_ext/string/access"
-require "active_support/ordered_hash"
 require "active_support/core_ext/object/conversions"
 require "active_support/core_ext/date/conversions"
 require "active_support/core_ext/object/deep_dup"
@@ -584,15 +584,6 @@ class HashToXmlTest < ActiveSupport::TestCase
     assert_includes xml, %(<addresses type="array"><address><streets type="array"><street><name>)
   end
 
-  def test_timezoned_attributes
-    xml = {
-      created_at: Time.utc(1999, 2, 2),
-      local_created_at: Time.utc(1999, 2, 2).in_time_zone("Eastern Time (US & Canada)")
-    }.to_xml(@xml_options)
-    assert_match %r{<created-at type=\"dateTime\">1999-02-02T00:00:00Z</created-at>}, xml
-    assert_match %r{<local-created-at type=\"dateTime\">1999-02-01T19:00:00-05:00</local-created-at>}, xml
-  end
-
   def test_multiple_records_from_xml_with_attributes_other_than_type_ignores_them_without_exploding
     topics_xml = <<-EOT
       <topics type="array" page="1" page-count="1000" per-page="2">
