@@ -10,10 +10,6 @@ class FileUpdateCheckerTest < ActiveSupport::TestCase
     ActiveSupport::FileUpdateChecker.new(files, dirs, &block)
   end
 
-  def wait
-    # noop
-  end
-
   def touch(files)
     sleep 1 # let's wait a bit to ensure there's a new mtime
     super
