@@ -29,13 +29,13 @@ module ApplicationTests
 
       app_file "config/environments/development.rb", <<-RUBY
         Rails.application.configure do
-          config.database = "db/development.sqlite3"
+          config.database = "storage/development.sqlite3"
         end
       RUBY
 
       primary, replica = PTY.open
       spawn_dbconsole(replica)
-      assert_output("sqlite>", primary)
+      assert_output("sqlite>", primary, 100)
     ensure
       primary.puts ".exit"
     end
@@ -49,16 +49,16 @@ module ApplicationTests
 
         development:
           <<: *default
-          database: db/development.sqlite3
+          database: storage/development.sqlite3
 
         production:
           <<: *default
-          database: db/production.sqlite3
+          database: storage/production.sqlite3
       YAML
 
       primary, replica = PTY.open
       spawn_dbconsole(replica, "-e production")
-      assert_output("sqlite>", primary)
+      assert_output("sqlite>", primary, 100)
 
       primary.puts "pragma database_list;"
       assert_output("production.sqlite3", primary)
