@@ -35,6 +35,11 @@ module ApplicationTests
       assert File.exist?(File.join(rails_root, "vendor/plugins/bukkits/test/dummy/config/application.rb"))
     end
 
+    test "allow generating plugin inside Rails app directory" do
+      rails "generate", "plugin", "vendor/plugins/bukkits"
+      assert File.exist?(File.join(rails_root, "vendor/plugins/bukkits/test/dummy/config/application.rb"))
+    end
+
     test "generators default values" do
       with_bare_config do |c|
         assert_equal(true, c.generators.colorize_logging)
@@ -233,6 +238,7 @@ module ApplicationTests
             app/views/posts/show.html.erb
             app/views/posts/new.html.erb
             app/views/posts/_form.html.erb
+            app/views/posts/_post.html.erb
             test/controllers/posts_controller_test.rb
             test/system/posts_test.rb
             app/helpers/posts_helper.rb
@@ -249,5 +255,26 @@ module ApplicationTests
 
       assert_match(/# Add comment to model/, File.read(model_file))
     end
+
+    if RUBY_VERSION < "3.4" # Revisit when 3.4 is added to the matrix
+      test "generators with apply_rubocop_autocorrect_after_generate!" do
+        with_config do |c|
+          c.generators.apply_rubocop_autocorrect_after_generate!
+        end
+
+        output = rails("generate", "model", "post", "title:string", "body:string")
+        assert_no_match(/3 files inspected, no offenses detected/, output)
+      end
+
+      test "generators with apply_rubocop_autocorrect_after_generate! and pretend" do
+        with_config do |c|
+          c.generators.apply_rubocop_autocorrect_after_generate!
+        end
+
+        assert_nothing_raised do
+          rails("generate", "model", "post", "title:string", "body:string", "--pretend")
+        end
+      end
+    end
   end
 end
