@@ -3,12 +3,14 @@
 require "isolation/abstract_unit"
 require "rack/test"
 require "base64"
+require "rails-dom-testing"
 
 module ApplicationTests
   class MailerPreviewsTest < ActiveSupport::TestCase
     include ActiveSupport::Testing::Isolation
     include Rack::Test::Methods
     include ERB::Util
+    include Rails::Dom::Testing::Assertions
 
     def setup
       build_app
@@ -26,14 +28,14 @@ module ApplicationTests
 
     test "/rails/mailers is not accessible in production" do
       app("production")
-      get "/rails/mailers"
+      get("/rails/mailers", {}, { "HTTPS" => "on" })
       assert_equal 404, last_response.status
     end
 
     test "/rails/mailers is accessible with correct configuration" do
       add_to_config "config.action_mailer.show_previews = true"
       app("production")
-      get "/rails/mailers", {}, { "REMOTE_ADDR" => "4.2.42.42" }
+      get "/rails/mailers", {}, { "REMOTE_ADDR" => "4.2.42.42", "HTTPS" => "on" }
       assert_equal 200, last_response.status
     end
 
@@ -55,7 +57,19 @@ module ApplicationTests
       assert_equal 200, last_response.status
     end
 
-    test "mailer previews are loaded from the default preview_path" do
+    test "request without mailer previews links to documentation" do
+      app("development")
+
+      get "/rails/mailers"
+      assert_select "title", text: "Action Mailer Previews"
+      assert_select "h1", text: "Action Mailer Previews"
+      assert_select "p", text: "You have not defined any Action Mailer Previews."
+      assert_select "p", text: "Read Action Mailer Basics to learn how to define your first." do
+        assert_select "a[href=?]", "https://guides.rubyonrails.org/action_mailer_basics.html#previewing-emails", text: "Action Mailer Basics"
+      end
+    end
+
+    test "mailer previews are loaded from the default preview_paths" do
       mailer "notifier", <<-RUBY
         class Notifier < ActionMailer::Base
           default from: "from@example.com"
@@ -81,41 +95,47 @@ module ApplicationTests
       app("development")
 
       get "/rails/mailers"
+      assert_select "title", text: "Action Mailer Previews"
+      assert_select "h1", text: "Action Mailer Previews"
       assert_match '<h3><a href="/rails/mailers/notifier">Notifier</a></h3>', last_response.body
       assert_match '<li><a href="/rails/mailers/notifier/foo">foo</a></li>', last_response.body
     end
 
-    test "mailer previews are loaded from a custom preview_path" do
+    test "mailer previews are loaded from custom preview_paths" do
       app_dir "lib/mailer_previews"
-      add_to_config "config.action_mailer.preview_path = '#{app_path}/lib/mailer_previews'"
+      add_to_config "config.action_mailer.preview_paths = ['#{app_path}/lib/notifier_previews', '#{app_path}/lib/confirm_previews']"
 
-      mailer "notifier", <<-RUBY
-        class Notifier < ActionMailer::Base
-          default from: "from@example.com"
+      ["notifier", "confirm"].each do |keyword|
+        mailer keyword, <<-RUBY
+          class #{keyword.camelize} < ActionMailer::Base
+            default from: "from@example.com"
 
-          def foo
-            mail to: "to@example.org"
+            def foo
+              mail to: "to@example.org"
+            end
           end
-        end
-      RUBY
+        RUBY
 
-      text_template "notifier/foo", <<-RUBY
-        Hello, World!
-      RUBY
+        text_template "#{keyword}/foo", <<-RUBY
+          Hello, World!
+        RUBY
 
-      app_file "lib/mailer_previews/notifier_preview.rb", <<-RUBY
-        class NotifierPreview < ActionMailer::Preview
-          def foo
-            Notifier.foo
+        app_file "lib/#{keyword}_previews/notifier_preview.rb", <<-RUBY
+          class #{keyword.camelize}Preview < ActionMailer::Preview
+            def foo
+              #{keyword.camelize}.foo
+            end
           end
-        end
-      RUBY
+        RUBY
+      end
 
       app("development")
 
       get "/rails/mailers"
       assert_match '<h3><a href="/rails/mailers/notifier">Notifier</a></h3>', last_response.body
       assert_match '<li><a href="/rails/mailers/notifier/foo">foo</a></li>', last_response.body
+      assert_match '<h3><a href="/rails/mailers/confirm">Confirm</a></h3>', last_response.body
+      assert_match '<li><a href="/rails/mailers/confirm/foo">foo</a></li>', last_response.body
     end
 
     test "mailer previews are reloaded across requests" do
@@ -255,9 +275,9 @@ module ApplicationTests
       assert_no_match '<li><a href="/rails/mailers/notifier/bar">bar</a></li>', last_response.body
     end
 
-    test "mailer previews are reloaded from a custom preview_path" do
+    test "mailer previews are reloaded from custom preview_paths" do
       app_dir "lib/mailer_previews"
-      add_to_config "config.action_mailer.preview_path = '#{app_path}/lib/mailer_previews'"
+      add_to_config "config.action_mailer.preview_paths = ['#{app_path}/lib/mailer_previews']"
 
       app("development")
 
@@ -296,6 +316,54 @@ module ApplicationTests
       assert_no_match '<h3><a href="/rails/mailers/notifier">Notifier</a></h3>', last_response.body
     end
 
+    test "mailer without previews" do
+      mailer "notifier", <<-RUBY
+        class Notifier < ActionMailer::Base
+        end
+      RUBY
+
+      mailer_preview "notifier", <<-RUBY
+        class NotifierPreview < ActionMailer::Preview
+        end
+      RUBY
+
+      app("development")
+      get "/rails/mailers/notifier"
+
+      assert_predicate last_response, :ok?
+      assert_select "title", text: "Action Mailer Previews for notifier"
+      assert_select "h1", text: "Action Mailer Previews for notifier"
+      assert_select "p", text: "You have not defined any actions for NotifierPreview."
+      assert_select "p", text: "Read Action Mailer Basics to learn how to define your first." do
+        assert_select "a[href=?]", "https://guides.rubyonrails.org/action_mailer_basics.html#previewing-emails", text: "Action Mailer Basics"
+      end
+    end
+
+    test "mailer with previews" do
+      mailer "notifier", <<-RUBY
+        class Notifier < ActionMailer::Base
+          def foo
+          end
+        end
+      RUBY
+
+      mailer_preview "notifier", <<-RUBY
+        class NotifierPreview < ActionMailer::Preview
+          def foo
+            Notifier.foo
+          end
+        end
+      RUBY
+
+      app("development")
+      get "/rails/mailers/notifier"
+
+      assert_predicate last_response, :ok?
+      assert_select "title", text: "Action Mailer Previews for notifier"
+      assert_select "h1", text: "Action Mailer Previews for notifier"
+      assert_select "ul li a[href=?]", "/rails/mailers/notifier/foo"
+    end
+
     test "mailer preview not found" do
       app("development")
       get "/rails/mailers/notifier"
@@ -331,6 +399,10 @@ module ApplicationTests
       get "/rails/mailers/notifier/bar"
       assert_predicate last_response, :not_found?
       assert_match "Email &#39;bar&#39; not found in NotifierPreview", h(last_response.body)
+
+      get "/rails/mailers/download/notifier/bar"
+      assert_predicate last_response, :not_found?
+      assert_match "Email &#39;bar&#39; not found in NotifierPreview", h(last_response.body)
     end
 
     test "mailer preview NullMail" do
@@ -417,9 +489,54 @@ module ApplicationTests
 
       get "/rails/mailers/notifier/foo"
       assert_equal 200, last_response.status
-      assert_match "Ruby on Rails &lt;core@rubyonrails.org&gt;", last_response.body
-      assert_match "Andrew White &lt;andyw@pixeltrix.co.uk&gt;", last_response.body
-      assert_match "David Heinemeier Hansson &lt;david@heinemeierhansson.com&gt;", last_response.body
+      assert_match '<dd id="from">Ruby on Rails &lt;core@rubyonrails.org&gt;</dd>', last_response.body
+      assert_match '<dd id="to">Andrew White &lt;andyw@pixeltrix.co.uk&gt;</dd>', last_response.body
+      assert_match '<dd id="cc">David Heinemeier Hansson &lt;david@heinemeierhansson.com&gt;</dd>', last_response.body
+      assert_no_match '<dd id="smtp_from">', last_response.body
+      assert_no_match '<dd id="smtp_to">', last_response.body
+
+      get "/rails/mailers/download/notifier/foo"
+      email = Mail.read_from_string(last_response.body)
+      assert_equal "attachment; filename=\"foo.eml\"; filename*=UTF-8''foo.eml", last_response.headers["Content-Disposition"]
+      assert_equal 200, last_response.status
+      assert_equal ["andyw@pixeltrix.co.uk"], email.to
+      assert_equal ["david@heinemeierhansson.com"], email.cc
+    end
+
+    test "message header shows SMTP envelope To and From when different than message headers" do
+      mailer "notifier", <<-RUBY
+        class Notifier < ActionMailer::Base
+          default from: "from@example.com"
+
+          def foo
+            message.smtp_envelope_from = "smtp-from@example.com"
+            message.smtp_envelope_to = ["to@example.com", "bcc@example.com"]
+
+            mail to: "to@example.com"
+          end
+        end
+      RUBY
+
+      mailer_preview "notifier", <<-RUBY
+        class NotifierPreview < ActionMailer::Preview
+          def foo
+            Notifier.foo
+          end
+        end
+      RUBY
+
+      text_template "notifier/foo", <<-RUBY
+        Hello, World!
+      RUBY
+
+      app("development")
+
+      get "/rails/mailers/notifier/foo"
+      assert_equal 200, last_response.status
+      assert_match '<dd id="from">from@example.com</dd>', last_response.body
+      assert_match '<dd id="smtp_from">smtp-from@example.com</dd>', last_response.body
+      assert_match '<dd id="to">to@example.com</dd>', last_response.body
+      assert_match '<dd id="smtp_to">to@example.com, bcc@example.com</dd>', last_response.body
     end
 
     test "part menu selects correct option" do
@@ -585,7 +702,7 @@ module ApplicationTests
 
       get "/rails/mailers/notifier/foo.txt"
       assert_equal 200, last_response.status
-      assert_match '<iframe seamless name="messageBody" src="?part=text%2Fplain">', last_response.body
+      assert_match '<iframe name="messageBody" src="?part=text%2Fplain">', last_response.body
       assert_match '<option selected value="part=text%2Fplain">', last_response.body
       assert_match '<option  value="part=text%2Fhtml">', last_response.body
 
@@ -595,7 +712,7 @@ module ApplicationTests
 
       get "/rails/mailers/notifier/foo.html?name=Ruby"
       assert_equal 200, last_response.status
-      assert_match '<iframe seamless name="messageBody" src="?name=Ruby&amp;part=text%2Fhtml">', last_response.body
+      assert_match '<iframe name="messageBody" src="?name=Ruby&amp;part=text%2Fhtml">', last_response.body
       assert_match '<option selected value="name=Ruby&amp;part=text%2Fhtml">', last_response.body
       assert_match '<option  value="name=Ruby&amp;part=text%2Fplain">', last_response.body
 
@@ -634,11 +751,21 @@ module ApplicationTests
 
       get "/rails/mailers/notifier/foo"
       assert_equal 200, last_response.status
-      assert_match %r[<iframe seamless name="messageBody"], last_response.body
+      assert_match %[<iframe name="messageBody"], last_response.body
+      assert_match %[<dt>Attachments:</dt>], last_response.body
+      assert_no_match %[Inline:], last_response.body
+      assert_match %[<a download="pixel.png" href="data:application/octet-stream;charset=utf-8;base64,iVBORw0K], last_response.body
 
       get "/rails/mailers/notifier/foo?part=text/plain"
       assert_equal 200, last_response.status
       assert_match %r[Hello, World!], last_response.body
+
+      get "/rails/mailers/download/notifier/foo"
+      assert_equal 200, last_response.status
+      email = Mail.read_from_string(last_response.body)
+      assert_equal 2, email.parts.size
+      assert_equal "text/plain; charset=UTF-8", email.parts[0].content_type
+      assert_equal "image/png; filename=pixel.png", email.parts[1].content_type
     end
 
     test "multipart mailer preview with attachment" do
@@ -675,7 +802,10 @@ module ApplicationTests
 
       get "/rails/mailers/notifier/foo"
       assert_equal 200, last_response.status
-      assert_match %r[<iframe seamless name="messageBody"], last_response.body
+      assert_match %[<iframe name="messageBody"], last_response.body
+      assert_match %[<dt>Attachments:</dt>], last_response.body
+      assert_no_match %[Inline:], last_response.body
+      assert_match %[<a download="pixel.png" href="data:application/octet-stream;charset=utf-8;base64,iVBORw0K], last_response.body
 
       get "/rails/mailers/notifier/foo?part=text/plain"
       assert_equal 200, last_response.status
@@ -694,7 +824,7 @@ module ApplicationTests
           default from: "from@example.com"
 
           def foo
-            attachments['pixel.png'] = File.binread("#{app_path}/public/images/pixel.png")
+            attachments.inline['pixel.png'] = File.binread("#{app_path}/public/images/pixel.png")
             mail to: "to@example.org"
           end
         end
@@ -721,7 +851,9 @@ module ApplicationTests
 
       get "/rails/mailers/notifier/foo"
       assert_equal 200, last_response.status
-      assert_match %r[<iframe seamless name="messageBody"], last_response.body
+      assert_match %[<iframe name="messageBody"], last_response.body
+      assert_match %[<dt>Attachments:</dt>], last_response.body
+      assert_match %r[\(Inline:\s+<a download="pixel.png" href="data:application/octet-stream;charset=utf-8;base64,iVBORw0K], last_response.body
 
       get "/rails/mailers/notifier/foo?part=text/plain"
       assert_equal 200, last_response.status
@@ -731,6 +863,10 @@ module ApplicationTests
       assert_equal 200, last_response.status
       assert_match %r[<p>Hello, World!</p>], last_response.body
       assert_match %r[src="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAEAAAABAQMAAAAl21bKAAAAA1BMVEWzIioca/JlAAAACklEQVQI12NgAAAAAgAB4iG8MwAAAABJRU5ErkJgggo="], last_response.body
+
+      get "/rails/mailers/download/notifier/foo"
+      email = Mail.read_from_string(last_response.body)
+      assert_equal "inline; filename=pixel.png", email.attachments.inline["pixel.png"].content_disposition
     end
 
     test "multipart mailer preview with attached email" do
@@ -779,7 +915,10 @@ module ApplicationTests
 
       get "/rails/mailers/notifier/foo"
       assert_equal 200, last_response.status
-      assert_match %r[<iframe seamless name="messageBody"], last_response.body
+      assert_match %[<iframe name="messageBody"], last_response.body
+      assert_match %[<dt>Attachments:</dt>], last_response.body
+      assert_no_match %[Inline:], last_response.body
+      assert_match %[<a download="message.eml" href="data:application/octet-stream;charset=utf-8;base64,RGF0ZTog], last_response.body
 
       get "/rails/mailers/notifier/foo?part=text/plain"
       assert_equal 200, last_response.status
@@ -853,6 +992,131 @@ module ApplicationTests
       assert_match "<title>Mailer Preview for notifier#foo</title>", last_response.body
     end
 
+    test "mailer preview sender tags" do
+      mailer "notifier", <<-RUBY
+        class Notifier < ActionMailer::Base
+          default from: "from@example.com"
+          def foo
+            mail to: "to@example.org", cc: "cc@example.com", bcc: "bcc@example.com"
+          end
+        end
+      RUBY
+
+      text_template "notifier/foo", <<-RUBY
+        Hello, World!
+      RUBY
+
+      mailer_preview "notifier", <<-RUBY
+        class NotifierPreview < ActionMailer::Preview
+          def foo
+            Notifier.foo
+          end
+        end
+      RUBY
+
+      app("development")
+
+      get "/rails/mailers/notifier/foo"
+      assert_match "<dd id=\"to\">to@example.org</dd>", last_response.body
+      assert_match "<dd id=\"cc\">cc@example.com</dd>", last_response.body
+      assert_match "<dd id=\"bcc\">bcc@example.com</dd>", last_response.body
+    end
+
+    test "mailer preview date tag renders date from message header" do
+      mailer "notifier", <<-RUBY
+        class Notifier < ActionMailer::Base
+          default from: "from@example.com"
+
+          def foo
+            mail to: "to@example.org", date: Time.utc(2023, 10, 20, 10, 20, 30)
+          end
+        end
+      RUBY
+
+      text_template "notifier/foo", <<-RUBY
+        Hello, World!
+      RUBY
+
+      mailer_preview "notifier", <<-RUBY
+        class NotifierPreview < ActionMailer::Preview
+          def foo
+            Notifier.foo
+          end
+        end
+      RUBY
+
+      app("development")
+
+      get "/rails/mailers/notifier/foo"
+      assert_match "<dd id=\"date\">Fri, 20 Oct 2023 10:20:30 +0000</dd>", last_response.body
+    end
+
+    test "mailer preview date tag falls back to current time when date header is not present" do
+      mailer "notifier", <<-RUBY
+        class Notifier < ActionMailer::Base
+          default from: "from@example.com"
+
+          def foo
+            mail to: "to@example.org"
+          end
+        end
+      RUBY
+
+      text_template "notifier/foo", <<-RUBY
+        Hello, World!
+      RUBY
+
+      mailer_preview "notifier", <<-RUBY
+        class NotifierPreview < ActionMailer::Preview
+          def foo
+            Notifier.foo
+          end
+        end
+      RUBY
+
+      app("development")
+
+      travel_to(Time.utc(2023, 10, 20, 10, 20, 30)) do
+        get "/rails/mailers/notifier/foo"
+      end
+      assert_match "<dd id=\"date\">Fri, 20 Oct 2023 10:20:30 +0000</dd>", last_response.body
+    end
+
+    test "mailer preview has access to rendering context" do
+      mailer "notifier", <<-RUBY
+        class Notifier < ActionMailer::Base
+          default from: "from@example.com"
+          def foo
+            @template = params[:template]
+            mail to: "to@example.org", cc: "cc@example.com", bcc: "bcc@example.com"
+          end
+        end
+      RUBY
+
+      text_template "notifier/foo", <<-RUBY
+        <%= @template %>
+      RUBY
+
+      text_template "notifier/bar", <<-RUBY
+        bar
+      RUBY
+
+      mailer_preview "notifier", <<-RUBY
+        class NotifierPreview < ActionMailer::Preview
+          def foo
+            # This is meant to simulate how Action Text's renderer works. See #47072.
+            template = Rails::MailersController.renderer.render_to_string("notifier/bar")
+            Notifier.with(template: template).foo
+          end
+        end
+      RUBY
+
+      app("development")
+
+      get "/rails/mailers/notifier/foo?part=text%2Fplain"
+      assert_includes last_response.body, "bar"
+    end
+
     private
       def build_app
         super
@@ -860,6 +1124,10 @@ module ApplicationTests
         app_dir "test/mailers/previews"
       end
 
+      def document_root_element
+        Nokogiri::HTML5.parse(last_response.body)
+      end
+
       def mailer(name, contents)
         app_file("app/mailers/#{name}.rb", contents)
       end
