@@ -2,9 +2,8 @@
 
 require "isolation/abstract_unit"
 require "rails/command"
-require "rails/commands/db/system/change/change_command"
 
-class Rails::Command::Db::System::ChangeCommandTest < ActiveSupport::TestCase
+class Rails::Command::DbSystemChangeTest < ActiveSupport::TestCase
   include ActiveSupport::Testing::Isolation
 
   setup { build_app }
@@ -26,9 +25,7 @@ class Rails::Command::Db::System::ChangeCommandTest < ActiveSupport::TestCase
     assert_match <<~MSG.squish, output
       Invalid value for --to option.
       Supported preconfigurations are:
-      mysql, postgresql, sqlite3, oracle,
-      sqlserver, jdbcmysql, jdbcsqlite3,
-      jdbcpostgresql, jdbc.
+      mysql, trilogy, postgresql, sqlite3.
     MSG
   end
 
@@ -54,6 +51,21 @@ class Rails::Command::Db::System::ChangeCommandTest < ActiveSupport::TestCase
     assert_match "gsub  Gemfile", output
   end
 
+  test "change can be forced" do
+    output = `cd #{app_path}; bin/rails db:system:change --to=postgresql --force`
+
+    assert_match "force  config/database.yml", output
+    assert_match "gsub  Gemfile", output
+  end
+
+  test "change works with no Dockerfile" do
+    remove_file("Dockerfile")
+
+    output = change_database(to: "sqlite3")
+
+    assert_match "gsub  Gemfile", output
+  end
+
   private
     def change_database(to:, **options)
       args = ["--to", to]
