@@ -4,21 +4,17 @@ require "rails"
 
 module EnvHelpers
   private
-    def with_rails_env(env)
+    def with_rails_env(env, &block)
       Rails.instance_variable_set :@_env, nil
       switch_env "RAILS_ENV", env do
-        switch_env "RACK_ENV", nil do
-          yield
-        end
+        switch_env "RACK_ENV", nil, &block
       end
     end
 
-    def with_rack_env(env)
+    def with_rack_env(env, &block)
       Rails.instance_variable_set :@_env, nil
       switch_env "RACK_ENV", env do
-        switch_env "RAILS_ENV", nil do
-          yield
-        end
+        switch_env "RAILS_ENV", nil, &block
       end
     end
 
