@@ -11,7 +11,7 @@ class ControllerGeneratorTest < Rails::Generators::TestCase
 
   def test_help_does_not_show_invoked_generators_options_if_they_already_exist
     content = run_generator ["--help"]
-    assert_no_match(/Helper options\:/, content)
+    assert_no_match(/Helper options:/, content)
   end
 
   def test_controller_skeleton_is_created
@@ -37,16 +37,6 @@ class ControllerGeneratorTest < Rails::Generators::TestCase
     assert_no_file "app/helpers/account_helper.rb"
   end
 
-  def test_invokes_assets
-    run_generator
-    assert_file "app/assets/stylesheets/account.css"
-  end
-
-  def test_does_not_invoke_assets_if_required
-    run_generator ["account", "--skip-assets"]
-    assert_no_file "app/assets/stylesheets/account.css"
-  end
-
   def test_invokes_default_test_framework
     run_generator
     assert_file "test/controllers/account_controller_test.rb"
@@ -65,7 +55,7 @@ class ControllerGeneratorTest < Rails::Generators::TestCase
 
   def test_add_routes
     run_generator
-    assert_file "config/routes.rb", /^  get 'account\/foo'/, /^  get 'account\/bar'/
+    assert_file "config/routes.rb", /^  get "account\/foo"/, /^  get "account\/bar"/
   end
 
   def test_skip_routes
@@ -104,14 +94,14 @@ class ControllerGeneratorTest < Rails::Generators::TestCase
   def test_namespaced_routes_are_created_in_routes
     run_generator ["admin/dashboard", "index"]
     assert_file "config/routes.rb" do |route|
-      assert_match(/^  namespace :admin do\n    get 'dashboard\/index'\n  end$/, route)
+      assert_match(/^  namespace :admin do\n    get "dashboard\/index"\n  end$/, route)
     end
   end
 
   def test_namespaced_routes_with_multiple_actions_are_created_in_routes
     run_generator ["admin/dashboard", "index", "show"]
     assert_file "config/routes.rb" do |route|
-      assert_match(/^  namespace :admin do\n    get 'dashboard\/index'\n    get 'dashboard\/show'\n  end$/, route)
+      assert_match(/^  namespace :admin do\n    get "dashboard\/index"\n    get "dashboard\/show"\n  end$/, route)
     end
   end
 
@@ -122,6 +112,13 @@ class ControllerGeneratorTest < Rails::Generators::TestCase
     end
   end
 
+  def test_controller_parent_param
+    run_generator ["admin/dashboard", "--parent", "admin_controller"]
+    assert_file "app/controllers/admin/dashboard_controller.rb" do |controller|
+      assert_match(/class Admin::DashboardController < AdminController/, controller)
+    end
+  end
+
   def test_controller_suffix_is_not_duplicated
     run_generator ["account_controller"]
 
@@ -136,8 +133,5 @@ class ControllerGeneratorTest < Rails::Generators::TestCase
 
     assert_no_file "app/helpers/account_controller_helper.rb"
     assert_file "app/helpers/account_helper.rb"
-
-    assert_no_file "app/assets/stylesheets/account_controller.css"
-    assert_file "app/assets/stylesheets/account.css"
   end
 end
