@@ -49,6 +49,10 @@ class MigrationGeneratorTest < Rails::Generators::TestCase
     end
   end
 
+  def test_exit_on_failure
+    assert_equal true, generator_class.exit_on_failure?
+  end
+
   def test_add_migration_with_attributes
     migration = "add_title_body_to_posts"
     run_generator [migration, "title:string", "body:text"]
@@ -127,7 +131,7 @@ class MigrationGeneratorTest < Rails::Generators::TestCase
     assert_migration "db/migrate/#{migration}.rb" do |content|
       assert_method :change, content do |change|
         assert_match(/remove_reference :books, :author,.*\sforeign_key: true/, change)
-        assert_match(/remove_reference :books, :distributor/, change) # sanity check
+        assert_match(/remove_reference :books, :distributor/, change) # Ensure the line isn't gone completely
         assert_no_match(/remove_reference :books, :distributor,.*\sforeign_key: true/, change)
       end
     end
@@ -159,21 +163,6 @@ class MigrationGeneratorTest < Rails::Generators::TestCase
     end
   end
 
-  def test_add_migration_with_attributes_and_wrong_index_declaration
-    migration = "add_title_and_content_to_books"
-    run_generator [migration, "title:string:inex", "content:text", "user_id:integer:unik"]
-
-    assert_migration "db/migrate/#{migration}.rb" do |content|
-      assert_method :change, content do |change|
-        assert_match(/add_column :books, :title, :string/, change)
-        assert_match(/add_column :books, :content, :text/, change)
-        assert_match(/add_column :books, :user_id, :integer/, change)
-      end
-      assert_no_match(/add_index :books, :title/, content)
-      assert_no_match(/add_index :books, :user_id/, content)
-    end
-  end
-
   def test_add_migration_with_attributes_without_type_and_index
     migration = "add_title_with_index_and_body_to_posts"
     run_generator [migration, "title:index", "body:text", "user_uuid:uniq"]
@@ -252,7 +241,7 @@ class MigrationGeneratorTest < Rails::Generators::TestCase
     assert_migration "db/migrate/#{migration}.rb" do |content|
       assert_method :change, content do |change|
         assert_match(/add_reference :books, :author,.*\sforeign_key: true/, change)
-        assert_match(/add_reference :books, :distributor/, change) # sanity check
+        assert_match(/add_reference :books, :distributor/, change) # Ensure the line isn't gone completely
         assert_no_match(/add_reference :books, :distributor,.*\sforeign_key: true/, change)
       end
     end
