@@ -1,13 +1,17 @@
 # frozen_string_literal: true
 
 require "generators/plugin_test_helper"
+require "env_helpers"
+require "plugin_helpers"
 
 class PluginTestRunnerTest < ActiveSupport::TestCase
   include PluginTestHelper
+  include EnvHelpers
+  include PluginHelpers
 
   def setup
     @destination_root = Dir.mktmpdir("bukkits")
-    Dir.chdir(@destination_root) { `bundle exec rails plugin new bukkits --skip-bundle` }
+    generate_plugin("#{@destination_root}/bukkits")
     plugin_file "test/dummy/db/schema.rb", ""
   end
 
@@ -15,6 +19,10 @@ class PluginTestRunnerTest < ActiveSupport::TestCase
     FileUtils.rm_rf(@destination_root)
   end
 
+  def test_run_default
+    assert_match "0 failures, 0 errors", run_test_command
+  end
+
   def test_run_single_file
     create_test_file "foo"
     create_test_file "bar"
@@ -65,7 +73,7 @@ class PluginTestRunnerTest < ActiveSupport::TestCase
     create_test_file "post", pass: false
 
     output = run_test_command("test/post_test.rb")
-    expect = %r{Running:\n\nPostTest\nF\n\nFailure:\nPostTest#test_truth \[[^\]]+test/post_test.rb:6\]:\nwups!\n\nbin/test (/private)?#{plugin_path}/test/post_test.rb:4}
+    expect = %r{Running:\n\nPostTest\nF\n\nFailure:\nPostTest#test_truth \[.*?test/post_test.rb:6\]:\nwups!\n\nbin/test (/private)?#{plugin_path}/test/post_test.rb:4}
     assert_match expect, output
   end
 
@@ -105,18 +113,14 @@ class PluginTestRunnerTest < ActiveSupport::TestCase
       capture(:stderr) { run_test_command("test/models/warnings_test.rb -w") })
   end
 
-  def test_run_rake_test
-    create_test_file "foo"
-    result = Dir.chdir(plugin_path) { `rake test TEST=test/foo_test.rb` }
-    assert_match "1 runs, 1 assertions, 0 failures", result
-  end
-
   private
     def plugin_path
       "#{@destination_root}/bukkits"
     end
 
-    def run_test_command(arguments)
-      Dir.chdir(plugin_path) { `bin/test #{arguments}` }
+    def run_test_command(arguments = "")
+      Dir.chdir(plugin_path) do
+        switch_env("BUNDLE_GEMFILE", "") { `bin/test #{arguments}` }
+      end
     end
 end
