@@ -1,9 +1,11 @@
 # frozen_string_literal: true
 
+require "plugin_helpers"
 require "generators/generators_test_helper"
 require "rails/generators/rails/scaffold/scaffold_generator"
 
 class ScaffoldGeneratorTest < Rails::Generators::TestCase
+  include PluginHelpers
   include GeneratorsTestHelper
   arguments %w(product_line title:string approved:boolean product:belongs_to user:references)
 
@@ -51,7 +53,7 @@ class ScaffoldGeneratorTest < Rails::Generators::TestCase
       end
 
       assert_instance_method :destroy, content do |m|
-        assert_match(/@product_line\.destroy/, m)
+        assert_match(/@product_line\.destroy!/, m)
       end
 
       assert_instance_method :set_product_line, content do |m|
@@ -82,7 +84,7 @@ class ScaffoldGeneratorTest < Rails::Generators::TestCase
     end
 
     %w(edit new).each do |view|
-      assert_file "app/views/product_lines/#{view}.html.erb", /render 'form', product_line: @product_line/
+      assert_file "app/views/product_lines/#{view}.html.erb", /render "form", product_line: @product_line/
     end
 
     assert_file "app/views/product_lines/_form.html.erb" do |test|
@@ -92,10 +94,6 @@ class ScaffoldGeneratorTest < Rails::Generators::TestCase
 
     # Helpers
     assert_file "app/helpers/product_lines_helper.rb"
-
-    # Assets
-    assert_file "app/assets/stylesheets/scaffold.css"
-    assert_file "app/assets/stylesheets/product_lines.css"
   end
 
   def test_api_scaffold_on_invoke
@@ -118,7 +116,7 @@ class ScaffoldGeneratorTest < Rails::Generators::TestCase
       assert_match(/class ProductLinesController < ApplicationController/, content)
       assert_no_match(/respond_to/, content)
 
-      assert_match(/before_action :set_product_line, only: \[:show, :update, :destroy\]/, content)
+      assert_match(/before_action :set_product_line, only: %i\[ show update destroy \]/, content)
 
       assert_instance_method :index, content do |m|
         assert_match(/@product_lines = ProductLine\.all/, m)
@@ -141,7 +139,7 @@ class ScaffoldGeneratorTest < Rails::Generators::TestCase
       end
 
       assert_instance_method :destroy, content do |m|
-        assert_match(/@product_line\.destroy/, m)
+        assert_match(/@product_line\.destroy!/, m)
       end
     end
 
@@ -164,10 +162,6 @@ class ScaffoldGeneratorTest < Rails::Generators::TestCase
 
     # Helpers
     assert_no_file "app/helpers/product_lines_helper.rb"
-
-    # Assets
-    assert_no_file "app/assets/stylesheets/scaffold.css"
-    assert_no_file "app/assets/stylesheets/product_lines.css"
   end
 
   def test_functional_tests_without_attributes
@@ -176,8 +170,8 @@ class ScaffoldGeneratorTest < Rails::Generators::TestCase
     assert_file "test/controllers/product_lines_controller_test.rb" do |content|
       assert_match(/class ProductLinesControllerTest < ActionDispatch::IntegrationTest/, content)
       assert_match(/test "should get index"/, content)
-      assert_match(/post product_lines_url, params: \{ product_line: \{  \} \}/, content)
-      assert_match(/patch product_line_url\(@product_line\), params: \{ product_line: \{  \} \}/, content)
+      assert_match(/post product_lines_url, params: \{ product_line: \{\} \}/, content)
+      assert_match(/patch product_line_url\(@product_line\), params: \{ product_line: \{\} \}/, content)
     end
   end
 
@@ -219,10 +213,6 @@ class ScaffoldGeneratorTest < Rails::Generators::TestCase
 
     # Helpers
     assert_no_file "app/helpers/product_lines_helper.rb"
-
-    # Assets
-    assert_file "app/assets/stylesheets/scaffold.css", /:visited/
-    assert_no_file "app/assets/stylesheets/product_lines.css"
   end
 
   def test_scaffold_with_namespace_on_invoke
@@ -266,7 +256,7 @@ class ScaffoldGeneratorTest < Rails::Generators::TestCase
       end
 
       assert_instance_method :destroy, content do |m|
-        assert_match(/@admin_role\.destroy/, m)
+        assert_match(/@admin_role\.destroy!/, m)
       end
 
       assert_instance_method :set_admin_role, content do |m|
@@ -282,23 +272,43 @@ class ScaffoldGeneratorTest < Rails::Generators::TestCase
 
     # Views
     assert_file "app/views/admin/roles/index.html.erb" do |content|
-      assert_match("'Show', admin_role", content)
-      assert_match("'Edit', edit_admin_role_path(admin_role)", content)
-      assert_match("'Destroy', admin_role", content)
-      assert_match("'New Admin Role', new_admin_role_path", content)
+      assert_match %{@admin_roles.each do |admin_role|}, content
+      assert_match %{render admin_role}, content
+      assert_match %{"Show this role", admin_role}, content
+      assert_match %{"New role", new_admin_role_path}, content
     end
 
-    %w(edit new show _form).each do |view|
-      assert_file "app/views/admin/roles/#{view}.html.erb"
+    assert_file "app/views/admin/roles/show.html.erb" do |content|
+      assert_match %{render @admin_role}, content
+      assert_match %{"Edit this role", edit_admin_role_path(@admin_role)}, content
+      assert_match %{"Back to roles", admin_roles_path}, content
+      assert_match %{"Destroy this role", @admin_role}, content
     end
+
+    assert_file "app/views/admin/roles/_role.html.erb" do |content|
+      assert_match "role", content
+      assert_no_match "admin_role", content
+    end
+
+    assert_file "app/views/admin/roles/new.html.erb"  do |content|
+      assert_match %{render "form", admin_role: @admin_role}, content
+      assert_match %{"Back to roles", admin_roles_path}, content
+    end
+
+    assert_file "app/views/admin/roles/edit.html.erb" do |content|
+      assert_match %{render "form", admin_role: @admin_role}, content
+      assert_match %{"Show this role", @admin_role}, content
+      assert_match %{"Back to roles", admin_roles_path}, content
+    end
+
+    assert_file "app/views/admin/roles/_form.html.erb"  do |content|
+      assert_match %{model: admin_role}, content
+    end
+
     assert_no_file "app/views/layouts/admin/roles.html.erb"
 
     # Helpers
     assert_file "app/helpers/admin/roles_helper.rb"
-
-    # Assets
-    assert_file "app/assets/stylesheets/scaffold.css", /:visited/
-    assert_file "app/assets/stylesheets/admin/roles.css"
   end
 
   def test_scaffold_with_namespace_on_revoke
@@ -330,10 +340,6 @@ class ScaffoldGeneratorTest < Rails::Generators::TestCase
 
     # Helpers
     assert_no_file "app/helpers/admin/roles_helper.rb"
-
-    # Assets
-    assert_file "app/assets/stylesheets/scaffold.css"
-    assert_no_file "app/assets/stylesheets/admin/roles.css"
   end
 
   def test_scaffold_generator_on_revoke_does_not_mutilate_legacy_map_parameter
@@ -357,8 +363,11 @@ class ScaffoldGeneratorTest < Rails::Generators::TestCase
     route_path = File.expand_path("config/routes.rb", destination_root)
     content = File.read(route_path)
 
-    # Remove all of the comments and blank lines from the routes file
+    # Remove all of the comments, blank lines, and default actions from the routes file
     content.gsub!(/^  \#.*\n/, "")
+    content.gsub!(/^  get "up".*\n/, "")
+    content.gsub!(/^  get "service-worker".*\n/, "")
+    content.gsub!(/^  get "manifest".*\n/, "")
     content.gsub!(/^\n/, "")
 
     File.write(route_path, content)
@@ -374,30 +383,6 @@ class ScaffoldGeneratorTest < Rails::Generators::TestCase
     assert_file "config/routes.rb", /\.routes\.draw do\n  resources :products\n/
   end
 
-  def test_scaffold_generator_no_assets_with_switch_no_assets
-    run_generator [ "posts", "--no-assets" ]
-    assert_no_file "app/assets/stylesheets/scaffold.css"
-    assert_no_file "app/assets/stylesheets/posts.css"
-  end
-
-  def test_scaffold_generator_no_assets_with_switch_assets_false
-    run_generator [ "posts", "--assets=false" ]
-    assert_no_file "app/assets/stylesheets/scaffold.css"
-    assert_no_file "app/assets/stylesheets/posts.css"
-  end
-
-  def test_scaffold_generator_no_scaffold_stylesheet_with_switch_no_scaffold_stylesheet
-    run_generator [ "posts", "--no-scaffold-stylesheet" ]
-    assert_no_file "app/assets/stylesheets/scaffold.css"
-    assert_file "app/assets/stylesheets/posts.css"
-  end
-
-  def test_scaffold_generator_no_scaffold_stylesheet_with_switch_scaffold_stylesheet_false
-    run_generator [ "posts", "--scaffold-stylesheet=false" ]
-    assert_no_file "app/assets/stylesheets/scaffold.css"
-    assert_file "app/assets/stylesheets/posts.css"
-  end
-
   def test_scaffold_generator_with_switch_resource_route_false
     run_generator [ "posts", "--resource-route=false" ]
     assert_file "config/routes.rb" do |route|
@@ -419,12 +404,6 @@ class ScaffoldGeneratorTest < Rails::Generators::TestCase
     assert_no_file "app/helpers/posts_helper.rb"
   end
 
-  def test_scaffold_generator_no_stylesheets
-    run_generator [ "posts", "--no-stylesheets" ]
-    assert_no_file "app/assets/stylesheets/scaffold.css"
-    assert_no_file "app/assets/stylesheets/posts.css"
-  end
-
   def test_scaffold_generator_outputs_error_message_on_missing_attribute_type
     run_generator ["post", "title", "body:text", "author"]
 
@@ -461,13 +440,16 @@ class ScaffoldGeneratorTest < Rails::Generators::TestCase
     end
 
     assert_file "app/views/accounts/index.html.erb" do |content|
-      assert_match(/^\W{8}<td><%= account\.name %><\/td>/, content)
-      assert_match(/^\W{8}<td><%= account\.user_id %><\/td>/, content)
+      assert_match(/^\W{2}<%= @accounts.each do |account| %>/, content)
+      assert_match(/^\W{4}<%= render account %>/, content)
+      assert_match(/<%= link_to "Show this account", account %>/, content)
     end
 
     assert_file "app/views/accounts/show.html.erb" do |content|
-      assert_match(/^\W{2}<%= @account\.name %>/, content)
-      assert_match(/^\W{2}<%= @account\.user_id %>/, content)
+      assert_match(/<%= render @account %>/, content)
+      assert_match(/link_to "Edit this account"/, content)
+      assert_match(/button_to "Destroy this account"/, content)
+      assert_match(/link_to "Back to accounts"/, content)
     end
   end
 
@@ -488,9 +470,9 @@ class ScaffoldGeneratorTest < Rails::Generators::TestCase
       assert_match(/^\W{4}<%= form\.file_field :photos, multiple: true %>/, content)
     end
 
-    assert_file "app/views/messages/show.html.erb" do |content|
-      assert_match(/^\W{2}<%= link_to @message\.video\.filename, @message\.video if @message\.video\.attached\? %>/, content)
-      assert_match(/^\W{4}<div><%= link_to photo\.filename, photo %>/, content)
+    assert_file "app/views/messages/_message.html.erb" do |content|
+      assert_match(/^\W{4}<%= link_to message\.video\.filename, message\.video if message\.video\.attached\? %>/, content)
+      assert_match(/^\W{6}<div><%= link_to photo\.filename, photo %>/, content)
     end
 
     assert_file "test/system/messages_test.rb" do |content|
@@ -572,40 +554,36 @@ class ScaffoldGeneratorTest < Rails::Generators::TestCase
     end
 
     assert_file "test/controllers/users_controller_test.rb" do |content|
-      assert_match(/password: 'secret'/, content)
-      assert_match(/password_confirmation: 'secret'/, content)
+      assert_match(/password: "secret"/, content)
+      assert_match(/password_confirmation: "secret"/, content)
     end
 
     assert_file "test/system/users_test.rb" do |content|
-      assert_match(/fill_in "Password", with: 'secret'/, content)
-      assert_match(/fill_in "Password confirmation", with: 'secret'/, content)
+      assert_match(/fill_in "Password", with: "secret"/, content)
+      assert_match(/fill_in "Password confirmation", with: "secret"/, content)
     end
 
     assert_file "test/fixtures/users.yml" do |content|
-      assert_match(/password_digest: <%= BCrypt::Password.create\('secret'\) %>/, content)
+      assert_match(/password_digest: <%= BCrypt::Password.create\("secret"\) %>/, content)
     end
   end
 
   def test_scaffold_tests_pass_by_default_inside_mountable_engine
-    Dir.chdir(destination_root) { `bundle exec rails plugin new bukkits --mountable` }
-
     engine_path = File.join(destination_root, "bukkits")
 
-    Dir.chdir(engine_path) do
+    with_new_plugin(engine_path, "--mountable") do
       quietly do
         `bin/rails g scaffold User name:string age:integer;
         bin/rails db:migrate`
       end
-      assert_match(/8 runs, 10 assertions, 0 failures, 0 errors/, `bin/rails test 2>&1`)
+      assert_match(/8 runs, 12 assertions, 0 failures, 0 errors/, `bin/rails test 2>&1`)
     end
   end
 
   def test_scaffold_tests_pass_by_default_inside_namespaced_mountable_engine
-    Dir.chdir(destination_root) { `bundle exec rails plugin new bukkits-admin --mountable` }
-
     engine_path = File.join(destination_root, "bukkits-admin")
 
-    Dir.chdir(engine_path) do
+    with_new_plugin(engine_path, "--mountable") do
       quietly do
         `bin/rails g scaffold User name:string age:integer;
         bin/rails db:migrate`
@@ -616,49 +594,43 @@ class ScaffoldGeneratorTest < Rails::Generators::TestCase
         assert_match(/class UsersController < ApplicationController/, content)
       end
 
-      assert_match(/8 runs, 10 assertions, 0 failures, 0 errors/, `bin/rails test 2>&1`)
+      assert_match(/8 runs, 12 assertions, 0 failures, 0 errors/, `bin/rails test 2>&1`)
     end
   end
 
   def test_scaffold_tests_pass_by_default_inside_full_engine
-    Dir.chdir(destination_root) { `bundle exec rails plugin new bukkits --full` }
-
     engine_path = File.join(destination_root, "bukkits")
 
-    Dir.chdir(engine_path) do
+    with_new_plugin(engine_path, "--full") do
       quietly do
         `bin/rails g scaffold User name:string age:integer;
         bin/rails db:migrate`
       end
-      assert_match(/8 runs, 10 assertions, 0 failures, 0 errors/, `bin/rails test 2>&1`)
+      assert_match(/8 runs, 12 assertions, 0 failures, 0 errors/, `bin/rails test 2>&1`)
     end
   end
 
   def test_scaffold_tests_pass_by_default_inside_api_mountable_engine
-    Dir.chdir(destination_root) { `bundle exec rails plugin new bukkits --mountable --api` }
-
     engine_path = File.join(destination_root, "bukkits")
 
-    Dir.chdir(engine_path) do
+    with_new_plugin(engine_path, "--mountable", "--api") do
       quietly do
         `bin/rails g scaffold User name:string age:integer;
         bin/rails db:migrate`
       end
-      assert_match(/6 runs, 8 assertions, 0 failures, 0 errors/, `bin/rails test 2>&1`)
+      assert_match(/6 runs, 10 assertions, 0 failures, 0 errors/, `bin/rails test 2>&1`)
     end
   end
 
   def test_scaffold_tests_pass_by_default_inside_api_full_engine
-    Dir.chdir(destination_root) { `bundle exec rails plugin new bukkits --full --api` }
-
     engine_path = File.join(destination_root, "bukkits")
 
-    Dir.chdir(engine_path) do
+    with_new_plugin(engine_path, "--full", "--api") do
       quietly do
         `bin/rails g scaffold User name:string age:integer;
         bin/rails db:migrate`
       end
-      assert_match(/6 runs, 8 assertions, 0 failures, 0 errors/, `bin/rails test 2>&1`)
+      assert_match(/6 runs, 10 assertions, 0 failures, 0 errors/, `bin/rails test 2>&1`)
     end
   end
 
@@ -685,8 +657,6 @@ class ScaffoldGeneratorTest < Rails::Generators::TestCase
       assert File.exist?("app/views/bukkits/users/_form.html.erb")
 
       assert File.exist?("app/helpers/bukkits/users_helper.rb")
-
-      assert File.exist?("app/assets/stylesheets/bukkits/users.css")
     end
   end
 
@@ -714,8 +684,6 @@ class ScaffoldGeneratorTest < Rails::Generators::TestCase
       assert_not File.exist?("app/views/bukkits/users/_form.html.erb")
 
       assert_not File.exist?("app/helpers/bukkits/users_helper.rb")
-
-      assert_not File.exist?("app/assets/stylesheets/bukkits/users.css")
     end
   end
 end
