@@ -1,9 +1,15 @@
 # frozen_string_literal: true
 
+require "find"
+require "shellwords"
+require "env_helpers"
+
 #
 # Tests, setup, and teardown common to the application and plugin generator suites.
 #
 module SharedGeneratorTests
+  include EnvHelpers
+
   def setup
     Rails.application = TestApp::Application
     super
@@ -26,9 +32,51 @@ module SharedGeneratorTests
     destination_root
   end
 
+  def test_implied_options
+    generator([destination_root], ["--skip-active-job"])
+
+    assert_option :skip_action_mailer
+    assert_option :skip_active_storage
+    assert_option :skip_action_mailbox
+    assert_option :skip_action_text
+    assert_not_option :skip_active_record
+  end
+
+  def test_implied_options_with_conflicting_option
+    error = assert_raises do
+      run_generator [destination_root, "--skip-active-job", "--no-skip-active-storage"]
+    end
+
+    assert_match %r/conflicting option/i, error.message
+  end
+
   def test_skeleton_is_created
     run_generator
 
+    generated_files_and_folders = []
+
+    Find.find(destination_root) do |absolute_path|
+      next if absolute_path == destination_root
+
+      pathname = Pathname.new(absolute_path)
+      git_folder = pathname.basename.to_s == ".git"
+
+      if git_folder || pathname.file?
+        generated_files_and_folders << pathname
+          .relative_path_from(destination_root)
+          .to_s
+      elsif pathname.directory? && pathname.children.empty?
+        flunk "`#{pathname} was generated but is an empty directory"
+      end
+
+      Find.prune if git_folder
+    end
+
+    # assert differences first for better error messages
+    assert_empty generated_files_and_folders.difference(default_files)
+    assert_empty default_files.difference(generated_files_and_folders)
+    assert_equal generated_files_and_folders.sort, default_files, "The expected list of generated files is not alphabetical"
+
     default_files.each { |path| assert_file path }
   end
 
@@ -39,7 +87,7 @@ module SharedGeneratorTests
 
   def test_invalid_database_option_raises_an_error
     content = capture(:stderr) { run_generator([destination_root, "-d", "unknown"]) }
-    assert_match(/Invalid value for \-\-database option/, content)
+    assert_match(/Expected '--database' to be one of/, content)
   end
 
   def test_test_files_are_skipped_if_required
@@ -72,6 +120,42 @@ module SharedGeneratorTests
     assert_file "bin/rails", /#!\/usr\/bin\/env/
   end
 
+  def test_template_from_absolute_path
+    assert_match "It works from file!", run_generator([destination_root, "-m", "#{fixtures_root}/lib/template.rb"])
+  end
+
+  def test_template_from_relative_path
+    FileUtils.cd(fixtures_root)
+    assert_match "It works from file!", run_generator([destination_root, "-m", "lib/template.rb"])
+  end
+
+  def test_template_with_env_var_in_path
+    switch_env "FIXTURES_ROOT", fixtures_root do
+      assert_match "It works from file!", run_generator([destination_root, "-m", "$FIXTURES_ROOT/lib/template.rb"])
+    end
+  end
+
+  def test_template_with_curly_brace_env_var_in_path
+    switch_env "FIXTURES_ROOT", fixtures_root do
+      assert_match "It works from file!", run_generator([destination_root, "-m", "${FIXTURES_ROOT}/lib/template.rb"])
+    end
+  end
+
+  def test_template_with_windows_env_var_in_path
+    switch_env "FIXTURES_ROOT", fixtures_root do
+      assert_match "It works from file!", run_generator([destination_root, "-m", "%FIXTURES_ROOT%/lib/template.rb"])
+    end
+  end
+
+  def test_template_with_special_characters_in_path
+    Dir.mktmpdir do |dir|
+      template_path = "#{dir}/company's $99 template  (original).rb"
+      FileUtils.cp "#{fixtures_root}/lib/template.rb", template_path
+
+      assert_match "It works from file!", run_generator([destination_root, "-m", template_path])
+    end
+  end
+
   def test_template_raises_an_error_with_invalid_path
     quietly do
       content = capture(:stderr) { run_generator([destination_root, "-m", "non/existent/path"]) }
@@ -83,28 +167,22 @@ module SharedGeneratorTests
 
   def test_template_is_executed_when_supplied_an_https_path
     url = "https://gist.github.com/josevalim/103208/raw/"
-    generator([destination_root], template: url, skip_webpack_install: true)
+    generator([destination_root], template: url)
 
     applied = nil
     apply_stub = -> (path, *) { applied = path }
 
     generator.stub(:apply, apply_stub) do
-      quietly { generator.invoke_all }
+      run_generator_instance
     end
 
     assert_equal url, applied
   end
 
-  def test_skip_gemfile
-    assert_not_called(generator([destination_root], skip_gemfile: true, skip_webpack_install: true), :bundle_command) do
-      quietly { generator.invoke_all }
-      assert_no_file "Gemfile"
-    end
-  end
-
   def test_skip_git
     run_generator [destination_root, "--skip-git", "--full"]
     assert_no_file(".gitignore")
+    assert_no_file(".gitattributes")
     assert_no_directory(".git")
   end
 
@@ -126,8 +204,7 @@ module SharedGeneratorTests
       "--skip-action-mailer",
       "--skip-action-mailbox",
       "--skip-action-text",
-      "--skip-action-cable",
-      "--skip-sprockets"
+      "--skip-action-cable"
     ]
 
     assert_file "#{application_path}/config/application.rb", /^require\s+["']rails["']/
@@ -143,7 +220,6 @@ module SharedGeneratorTests
     end
     assert_file "#{application_path}/config/application.rb", /^require\s+["']action_view\/railtie["']/
     assert_file "#{application_path}/config/application.rb", /^# require\s+["']action_cable\/engine["']/
-    assert_file "#{application_path}/config/application.rb", /^# require\s+["']sprockets\/railtie["']/
     assert_file "#{application_path}/config/application.rb", /^require\s+["']rails\/test_unit\/railtie["']/
   end
 
@@ -162,22 +238,6 @@ module SharedGeneratorTests
     end
   end
 
-  def test_gitignore_when_sqlite3
-    run_generator
-
-    assert_file ".gitignore" do |content|
-      assert_match(/sqlite3/, content)
-    end
-  end
-
-  def test_gitignore_when_non_sqlite3_db
-    run_generator([destination_root, "-d", "mysql"])
-
-    assert_file ".gitignore" do |content|
-      assert_no_match(/sqlite/i, content)
-    end
-  end
-
   def test_generator_if_skip_active_record_is_given
     run_generator [destination_root, "--skip-active-record"]
     assert_no_directory "#{application_path}/db/"
@@ -190,20 +250,10 @@ module SharedGeneratorTests
     assert_file "#{application_path}/bin/setup" do |setup_content|
       assert_no_match(/db:prepare/, setup_content)
     end
-    assert_file ".gitignore" do |content|
-      assert_no_match(/sqlite/i, content)
-    end
   end
 
   def test_generator_for_active_storage
-    run_generator
-
-    unless generator_class.name == "Rails::Generators::PluginGenerator"
-      assert_file "#{application_path}/app/javascript/packs/application.js" do |content|
-        assert_match(/^import \* as ActiveStorage from "@rails\/activestorage"/, content)
-        assert_match(/^ActiveStorage.start\(\)/, content)
-      end
-    end
+    run_generator([destination_root])
 
     assert_file "#{application_path}/config/environments/development.rb" do |content|
       assert_match(/config\.active_storage/, content)
@@ -220,10 +270,6 @@ module SharedGeneratorTests
     assert_file "#{application_path}/config/storage.yml"
     assert_directory "#{application_path}/storage"
     assert_directory "#{application_path}/tmp/storage"
-
-    assert_file ".gitignore" do |content|
-      assert_match(/\/storage\//, content)
-    end
   end
 
   def test_generator_if_skip_active_storage_is_given
@@ -231,10 +277,6 @@ module SharedGeneratorTests
 
     assert_file "#{application_path}/config/application.rb", /#\s+require\s+["']active_storage\/engine["']/
 
-    assert_file "#{application_path}/app/javascript/packs/application.js" do |content|
-      assert_no_match(/activestorage/, content)
-    end
-
     assert_file "#{application_path}/config/environments/development.rb" do |content|
       assert_no_match(/config\.active_storage/, content)
     end
@@ -248,12 +290,6 @@ module SharedGeneratorTests
     end
 
     assert_no_file "#{application_path}/config/storage.yml"
-    assert_no_directory "#{application_path}/storage"
-    assert_no_directory "#{application_path}/tmp/storage"
-
-    assert_file ".gitignore" do |content|
-      assert_no_match(/\/storage\//, content)
-    end
   end
 
   def test_generator_does_not_generate_active_storage_contents_if_skip_active_record_is_given
@@ -261,10 +297,6 @@ module SharedGeneratorTests
 
     assert_file "#{application_path}/config/application.rb", /#\s+require\s+["']active_storage\/engine["']/
 
-    assert_file "#{application_path}/app/javascript/packs/application.js" do |content|
-      assert_no_match(/activestorage/i, content)
-    end
-
     assert_file "#{application_path}/config/environments/development.rb" do |content|
       assert_no_match(/config\.active_storage/, content)
     end
@@ -278,12 +310,13 @@ module SharedGeneratorTests
     end
 
     assert_no_file "#{application_path}/config/storage.yml"
+  end
+
+  def test_generator_does_not_create_storage_dir_if_skip_active_storage_is_given_and_not_using_sqlite
+    run_generator [destination_root, "--skip-active-storage", "--database=postgresql"]
+
     assert_no_directory "#{application_path}/storage"
     assert_no_directory "#{application_path}/tmp/storage"
-
-    assert_file ".gitignore" do |content|
-      assert_no_match(/\/storage\//, content)
-    end
   end
 
   def test_generator_if_skip_action_mailer_is_given
@@ -300,26 +333,26 @@ module SharedGeneratorTests
     end
     assert_no_directory "#{application_path}/app/mailers"
     assert_no_directory "#{application_path}/test/mailers"
+    assert_no_file "#{application_path}/app/views/layouts/mailer.html.erb"
   end
 
   def test_generator_if_skip_action_cable_is_given
-    run_generator [destination_root, "--skip-action-cable"]
+    run_generator [destination_root, "--skip-action-cable", "--webpack"]
     assert_file "#{application_path}/config/application.rb", /#\s+require\s+["']action_cable\/engine["']/
     assert_no_file "#{application_path}/config/cable.yml"
     assert_no_file "#{application_path}/app/javascript/consumer.js"
     assert_no_directory "#{application_path}/app/javascript/channels"
     assert_no_directory "#{application_path}/app/channels"
     assert_file "Gemfile" do |content|
-      assert_no_match(/redis/, content)
+      assert_no_match(/"redis"/, content)
     end
   end
 
-  def test_generator_if_skip_sprockets_is_given
-    run_generator [destination_root, "--skip-sprockets"]
+  def test_generator_when_sprockets_is_not_used
+    run_generator [destination_root, "-a", "none"]
 
     assert_no_file "#{application_path}/config/initializers/assets.rb"
-
-    assert_file "#{application_path}/config/application.rb", /#\s+require\s+["']sprockets\/railtie["']/
+    assert_no_file "#{application_path}/app/assets/config/manifest.js"
 
     assert_file "Gemfile" do |content|
       assert_no_match(/sass-rails/, content)
@@ -336,21 +369,122 @@ module SharedGeneratorTests
     end
   end
 
-  def test_generator_for_yarn
-    skip "#34009 disabled JS by default for plugins" if generator_class.name == "Rails::Generators::PluginGenerator"
-    run_generator
-    assert_file "#{application_path}/package.json", /dependencies/
-    assert_file "#{application_path}/bin/yarn"
-    assert_file "#{application_path}/config/initializers/assets.rb", /node_modules/
+  def test_dev_option
+    run_generator_using_prerelease [destination_root, "--dev"]
+    rails_path = File.expand_path("../../..", Rails.root)
+    assert_file "Gemfile", %r{^gem ["']rails["'], path: ["']#{Regexp.escape rails_path}["']$}
   end
 
-  def test_generator_for_yarn_skipped
-    run_generator([destination_root, "--skip-javascript"])
-    assert_no_file "#{application_path}/package.json"
-    assert_no_file "#{application_path}/bin/yarn"
+  def test_edge_option
+    Rails.stub(:gem_version, Gem::Version.new("2.1.0")) do
+      run_generator_using_prerelease [destination_root, "--edge"]
+    end
+    assert_file "Gemfile", %r{^gem ["']rails["'], github: ["']rails/rails["'], branch: ["']2-1-stable["']$}
+  end
 
-    assert_file "#{application_path}/config/initializers/assets.rb" do |content|
-      assert_no_match(/node_modules/, content)
+  def test_edge_option_during_alpha
+    Rails.stub(:gem_version, Gem::Version.new("2.1.0.alpha")) do
+      run_generator_using_prerelease [destination_root, "--edge"]
+    end
+    assert_file "Gemfile", %r{^gem ["']rails["'], github: ["']rails/rails["'], branch: ["']main["']$}
+  end
+
+  def test_main_option
+    run_generator_using_prerelease [destination_root, "--main"]
+    assert_file "Gemfile", %r{^gem ["']rails["'], github: ["']rails/rails["'], branch: ["']main["']$}
+  end
+
+  def test_master_option
+    run_generator_using_prerelease [destination_root, "--master"]
+    assert_file "Gemfile", %r{^gem ["']rails["'], github: ["']rails/rails["'], branch: ["']main["']$}
+  end
+
+  def test_target_rails_prerelease_with_relative_app_path
+    run_generator_using_prerelease ["myproject", "--main"]
+    assert_file "myproject/Gemfile", %r{^gem ["']rails["'], github: ["']rails/rails["'], branch: ["']main["']$}
+  end
+
+  def test_generated_files_have_no_rubocop_warnings
+    run_generator
+
+    Dir.chdir(destination_root) do
+      output = `./bin/rubocop`
+
+      assert_predicate $?, :success?, "bin/rubocop did not exit successfully:\n#{output}"
     end
   end
+
+  private
+    def fixtures_root
+      File.expand_path("../fixtures", __dir__)
+    end
+
+    def run_generator_instance
+      @bundle_commands = []
+      @bundle_command_stub ||= -> (command, *) { @bundle_commands << command }
+
+      generator.stub(:bundle_command, @bundle_command_stub) do
+        super
+      end
+    end
+
+    def run_generator_using_prerelease(args)
+      option_args, positional_args = args.partition { |arg| arg.start_with?("--") }
+      project_path = File.expand_path(positional_args.first, destination_root)
+      expected_args = [project_path, *positional_args.drop(1), *option_args]
+
+      generator(positional_args, option_args)
+
+      prerelease_commands = []
+      prerelease_command_rails_gems = []
+      rails_gem_pattern = /^gem ["']rails["'], .+/
+
+      @bundle_command_stub = -> (command, *) do
+        @bundle_commands << command
+
+        if command.start_with?("install", "exec rails")
+          prerelease_commands << command
+          assert_file File.expand_path("Gemfile", project_path) do |gemfile|
+            prerelease_command_rails_gems << gemfile[rails_gem_pattern]
+          end
+        end
+      end
+
+      # run target_rails_prerelease on exit to mimic re-running generator
+      generator.stub :exit, generator.method(:target_rails_prerelease) do
+        run_generator_instance
+      end
+
+      assert_file File.expand_path("Gemfile", project_path) do |gemfile|
+        assert_match %r/^install/, prerelease_commands[0]
+        assert_equal gemfile[rails_gem_pattern], prerelease_command_rails_gems[0]
+
+        assert_match %r/^exec rails (?:plugin )?new #{Regexp.escape Shellwords.join(expected_args)}/, prerelease_commands[1]
+        assert_equal gemfile[rails_gem_pattern], prerelease_command_rails_gems[1]
+      end
+    end
+
+    def assert_option(option)
+      assert generator.options[option], "Expected generator option #{option.inspect} to be truthy."
+    end
+
+    def assert_not_option(option)
+      assert_not generator.options[option], "Expected generator option #{option.inspect} to be falsy."
+    end
+
+    def assert_gem(name, constraint = nil)
+      constraint_pattern =
+        if constraint.is_a?(String)
+          /, #{Regexp.escape constraint}/
+        elsif constraint
+          /, #{constraint}/
+        end
+      assert_file "Gemfile", %r/^\s*gem ["']#{name}["']#{constraint_pattern}/
+    end
+
+    def assert_no_gem(name)
+      assert_file "Gemfile" do |content|
+        assert_no_match %r/gem ["']#{name}["']/, content
+      end
+    end
 end
